@@ -43,12 +43,14 @@
 #  include "config.h"
 #endif
 
-#include <string.h>
-#include <sys/types.h>
+#include <ctype.h>
 #include <regex.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <stdint.h>
 #include <stdlib.h>
-#include <ctype.h>
+#include <unistd.h>
 
 /* #include "src/common/slurm_protocol_defs.h" */
 #include "src/common/log.h"
@@ -136,8 +138,7 @@ static s_p_values_t *_conf_hashtbl_lookup(
 	return NULL;
 }
 
-s_p_hashtbl_t *s_p_hashtbl_create(
-	s_p_options_t options[])
+s_p_hashtbl_t *s_p_hashtbl_create(s_p_options_t options[])
 {
 	s_p_options_t *op = NULL;
 	s_p_values_t *value = NULL;
@@ -146,7 +147,6 @@ s_p_hashtbl_t *s_p_hashtbl_create(
 
 	len = CONF_HASH_LEN * sizeof(s_p_values_t *);
 	hashtbl = (s_p_hashtbl_t *)xmalloc(len);
-	memset(hashtbl, 0, len);
 
 	for (op = options; op->key != NULL; op++) {
 		value = xmalloc(sizeof(s_p_values_t));
@@ -212,7 +212,7 @@ static void _keyvalue_regex_init(void)
 		if (regcomp(&keyvalue_re, keyvalue_pattern,
 			    REG_EXTENDED) != 0) {
 			/* FIXME - should be fatal? */
-			error("keyvalue regex compilation failed\n");
+			error("keyvalue regex compilation failed");
 		}
 		keyvalue_initialized = true;
 	}
@@ -341,6 +341,30 @@ static void _strip_escapes(char *line)
 	}
 }
 
+/* This can be used to make sure files are the same across nodes if
+ * needed */
+static void _compute_hash_val(uint32_t *hash_val, char *line)
+{
+	int idx, i, len;
+
+	if(!hash_val)
+		return;
+
+	len = strlen(line);
+	for (i = 0; i < len; i++) {
+		(*hash_val) = ( (*hash_val) ^ line[i] << 8 );
+
+		for (idx = 0; idx < 8; ++idx) {
+			if ((*hash_val) & 0x8000) {
+				(*hash_val) <<= 1;
+				(*hash_val) = (*hash_val) ^ 4129;
+			} else
+				(*hash_val) <<= 1;
+		}
+	}
+}
+
+
 /*
  * Reads the next line from the "file" into buffer "buf".
  *
@@ -348,7 +372,8 @@ static void _strip_escapes(char *line)
  * the next line by a trailing "\".  Strips out comments,
  * replaces escaped "\#" with "#", and replaces "\\" with "\".
  */
-static int _get_next_line(char *buf, int buf_size, FILE *file)
+static int _get_next_line(char *buf, int buf_size,
+			  uint32_t *hash_val, FILE *file)
 {
 	char *ptr = buf;
 	int leftover = buf_size;
@@ -357,6 +382,7 @@ static int _get_next_line(char *buf, int buf_size, FILE *file)
 
 	while (fgets(ptr, leftover, file)) {
 		lines++;
+		_compute_hash_val(hash_val, ptr);
 		_strip_comments(ptr);
 		read_size = strlen(ptr);
 		new_size = _strip_continuation(ptr, read_size);
@@ -778,7 +804,7 @@ static int _parse_next_key(s_p_hashtbl_t *hashtbl,
  * directive but the included file contained errors.  Returns 0 if
  * no include directive is found.
  */
-static int _parse_include_directive(s_p_hashtbl_t *hashtbl,
+static int _parse_include_directive(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
 				    const char *line, char **leftover)
 {
 	char *ptr;
@@ -797,7 +823,8 @@ static int _parse_include_directive(s_p_hashtbl_t *hashtbl,
 			ptr++;
 		fn_stop = *leftover = ptr;
 		filename = xstrndup(fn_start, fn_stop-fn_start);
-		if (s_p_parse_file(hashtbl, filename) == SLURM_SUCCESS) {
+		if (s_p_parse_file(hashtbl, hash_val, filename)
+		    == SLURM_SUCCESS) {
 			xfree(filename);
 			return 1;
 		} else {
@@ -809,7 +836,7 @@ static int _parse_include_directive(s_p_hashtbl_t *hashtbl,
 	}
 }
 
-int s_p_parse_file(s_p_hashtbl_t *hashtbl, char *filename)
+int s_p_parse_file(s_p_hashtbl_t *hashtbl, uint32_t *hash_val, char *filename)
 {
 	FILE *f;
 	char line[BUFFER_SIZE];
@@ -818,6 +845,7 @@ int s_p_parse_file(s_p_hashtbl_t *hashtbl, char *filename)
 	int line_number;
 	int merged_lines;
 	int inc_rc;
+	struct stat stat_buf;
 
 	if (!filename) {
 		error("s_p_parse_file: No filename given.");
@@ -825,7 +853,14 @@ int s_p_parse_file(s_p_hashtbl_t *hashtbl, char *filename)
 	}
 
 	_keyvalue_regex_init();
-
+	if (stat(filename, &stat_buf) < 0) {
+		info("s_p_parse_file: unable to status file \"%s\"", filename);
+		return SLURM_ERROR;
+	}
+	if (stat_buf.st_size == 0) {
+		info("s_p_parse_file: file \"%s\" is empty", filename);
+		return SLURM_SUCCESS;
+	}
 	f = fopen(filename, "r");
 	if (f == NULL) {
 		error("s_p_parse_file: unable to read \"%s\": %m",
@@ -834,14 +869,16 @@ int s_p_parse_file(s_p_hashtbl_t *hashtbl, char *filename)
 	}
 
 	line_number = 1;
-	while ((merged_lines = _get_next_line(line, BUFFER_SIZE, f)) > 0) {
+	while ((merged_lines = _get_next_line(
+			line, BUFFER_SIZE, hash_val, f)) > 0) {
 		/* skip empty lines */
 		if (line[0] == '\0') {
 			line_number += merged_lines;
 			continue;
 		}
 
-		inc_rc = _parse_include_directive(hashtbl, line, &leftover);
+		inc_rc = _parse_include_directive(hashtbl, hash_val,
+						  line, &leftover);
 		if (inc_rc == 0) {
 			_parse_next_key(hashtbl, line, &leftover);
 		} else if (inc_rc < 0) {
@@ -897,7 +934,7 @@ int s_p_get_string(char **str, const char *key, const s_p_hashtbl_t *hashtbl)
 		return 0;
 	}
 	if (p->type != S_P_STRING) {
-		error("Key \"%s\" is not a string\n", key);
+		error("Key \"%s\" is not a string", key);
 		return 0;
 	}
 	if (p->data_count == 0) {
@@ -935,7 +972,7 @@ int s_p_get_long(long *num, const char *key, const s_p_hashtbl_t *hashtbl)
 		return 0;
 	}
 	if (p->type != S_P_LONG) {
-		error("Key \"%s\" is not a long\n", key);
+		error("Key \"%s\" is not a long", key);
 		return 0;
 	}
 	if (p->data_count == 0) {
@@ -974,7 +1011,7 @@ int s_p_get_uint16(uint16_t *num, const char *key,
 		return 0;
 	}
 	if (p->type != S_P_UINT16) {
-		error("Key \"%s\" is not a uint16_t\n", key);
+		error("Key \"%s\" is not a uint16_t", key);
 		return 0;
 	}
 	if (p->data_count == 0) {
@@ -1013,7 +1050,7 @@ int s_p_get_uint32(uint32_t *num, const char *key,
 		return 0;
 	}
 	if (p->type != S_P_UINT32) {
-		error("Key \"%s\" is not a uint32_t\n", key);
+		error("Key \"%s\" is not a uint32_t", key);
 		return 0;
 	}
 	if (p->data_count == 0) {
@@ -1051,7 +1088,7 @@ int s_p_get_pointer(void **ptr, const char *key, const s_p_hashtbl_t *hashtbl)
 		return 0;
 	}
 	if (p->type != S_P_POINTER) {
-		error("Key \"%s\" is not a pointer\n", key);
+		error("Key \"%s\" is not a pointer", key);
 		return 0;
 	}
 	if (p->data_count == 0) {
@@ -1096,7 +1133,7 @@ int s_p_get_array(void **ptr_array[], int *count,
 		return 0;
 	}
 	if (p->type != S_P_ARRAY) {
-		error("Key \"%s\" is not an array\n", key);
+		error("Key \"%s\" is not an array", key);
 		return 0;
 	}
 	if (p->data_count == 0) {
@@ -1135,7 +1172,7 @@ int s_p_get_boolean(bool *flag, const char *key, const s_p_hashtbl_t *hashtbl)
 		return 0;
 	}
 	if (p->type != S_P_BOOLEAN) {
-		error("Key \"%s\" is not a boolean\n", key);
+		error("Key \"%s\" is not a boolean", key);
 		return 0;
 	}
 	if (p->data_count == 0) {
@@ -1197,7 +1234,7 @@ void s_p_dump_values(const s_p_hashtbl_t *hashtbl,
 			break;
 		case S_P_POINTER:
 			if (s_p_get_pointer(&ptr, op->key, hashtbl))
-				verbose("%s = %x", op->key, ptr);
+				verbose("%s = %zx", op->key, (size_t)ptr);
 			else
 				verbose("%s", op->key);
 			break;
