@@ -86,6 +86,17 @@ static int _isvalue(char *arg) {
 	return 0;	/* not a value */
 }
 
+static bool _have_task_affinity(void)
+{
+	bool rc = true;
+	char *plugin_name = slurm_get_task_plugin();
+
+	if (plugin_name && !strcmp(plugin_name, "task/none"))
+		rc = false;
+	xfree(plugin_name);
+	return rc;
+}
+
 /*
  * slurm_get_avail_procs - Get the number of "available" cpus on a node
  *	given this number given the number of cpus_per_task and
@@ -119,7 +130,7 @@ int slurm_get_avail_procs(const uint16_t socket_cnt,
 			  uint16_t *cores,
 			  uint16_t *threads,
 			  const uint16_t *alloc_cores,
-			  const select_type_plugin_info_t cr_type,
+			  const uint16_t cr_type,
 			  uint32_t job_id,
 			  char *name)
 {
@@ -167,32 +178,10 @@ int slurm_get_avail_procs(const uint16_t socket_cnt,
 		     job_id, name, i, alloc_cores[i]);
 #endif
 	allocated_cpus = allocated_cores * (*threads);
-	switch(cr_type) {
+
 	/* For the following CR types, nodes have no notion of socket, core,
 	   and thread.  Only one level of logical processors */
-	case SELECT_TYPE_INFO_NONE:
-		/* Default for select/linear */
-	case CR_CPU:
-	case CR_CPU_MEMORY:
-
-		if (*cpus >= allocated_cpus)
-			*cpus -= allocated_cpus;
-		else {
-			*cpus = 0;
-			error("cons_res: *cpus underflow");
-		}
-
-	case CR_MEMORY:
-		/*** compute an overall maximum cpu count honoring ntasks* ***/
-		max_cpus  = *cpus;
-		if (ntaskspernode > 0) {
-			max_cpus = MIN(max_cpus, ntaskspernode);
-		}
-		break;
-
-	/* For all other types, nodes contain sockets, cores, and threads */
-	case CR_CORE:
-	case CR_CORE_MEMORY:
+	if (cr_type & CR_CORE) {
 		if (*cpus >= allocated_cpus)
 			*cpus -= allocated_cpus;
 		else {
@@ -246,11 +235,7 @@ int slurm_get_avail_procs(const uint16_t socket_cnt,
 		if (ntaskspernode > 0) {
 			max_cpus = MIN(max_cpus, ntaskspernode);
 		}
-		break;
-
-	case CR_SOCKET:
-	case CR_SOCKET_MEMORY:
-	default:
+	} else if (cr_type & CR_SOCKET) {
 		if (*sockets >= allocated_sockets)
 			*sockets -= allocated_sockets; /* sockets count */
 		else {
@@ -288,7 +273,22 @@ int slurm_get_avail_procs(const uint16_t socket_cnt,
 
 		/*** honor any availability maximum ***/
 		max_cpus = MIN(max_cpus, max_avail_cpus);
-		break;
+	} else {	/* CR_CPU (default) */
+		if ((cr_type & CR_CPU) ||
+		    (!(cr_type & CR_MEMORY))) {
+			if (*cpus >= allocated_cpus)
+				*cpus -= allocated_cpus;
+			else {
+				*cpus = 0;
+				error("cons_res: *cpus underflow");
+			}
+		}
+
+		/*** compute an overall maximum cpu count honoring ntasks* ***/
+		max_cpus  = *cpus;
+		if (ntaskspernode > 0) {
+			max_cpus = MIN(max_cpus, ntaskspernode);
+		}
 	}
 
 	/*** factor cpus_per_task into max_cpus ***/
@@ -389,7 +389,11 @@ void slurm_sprint_mem_bind_type(char *str, mem_bind_type_t mem_bind_type)
 
 void slurm_print_cpu_bind_help(void)
 {
-	printf(
+	if (!_have_task_affinity()) {
+		printf("CPU bind options not supported with current "
+		       "configuration\n");
+	} else {
+		printf(
 "CPU bind options:\n"
 "    --cpu_bind=         Bind tasks to CPUs\n"
 "        q[uiet]         quietly bind before task runs (default)\n"
@@ -410,6 +414,7 @@ void slurm_print_cpu_bind_help(void)
 "        threads         auto-generated masks bind to threads\n"
 "        ldoms           auto-generated masks bind to NUMA locality domains\n"
 "        help            show this help message\n");
+	}
 }
 
 /*
@@ -437,6 +442,8 @@ int slurm_verify_cpu_bind(const char *arg, char **cpu_bind,
 	int bind_to_bits =
 		CPU_BIND_TO_SOCKETS|CPU_BIND_TO_CORES|CPU_BIND_TO_THREADS;
 	uint16_t task_plugin_param = slurm_get_task_plugin_param();
+	bool have_binding = _have_task_affinity();
+	bool log_binding = true;
 
 	bind_bits    |= CPU_BIND_LDRANK|CPU_BIND_LDMAP|CPU_BIND_LDMASK;
 	bind_to_bits |= CPU_BIND_TO_LDOMS;
@@ -482,7 +489,13 @@ int slurm_verify_cpu_bind(const char *arg, char **cpu_bind,
 		if (strcasecmp(tok, "help") == 0) {
 			slurm_print_cpu_bind_help();
 			return 1;
-		} else if ((strcasecmp(tok, "q") == 0) ||
+		}
+		if (!have_binding && log_binding) {
+			info("cluster configuration lacks support for cpu "
+			     "binding");
+			log_binding = false;
+		}
+		if ((strcasecmp(tok, "q") == 0) ||
 			   (strcasecmp(tok, "quiet") == 0)) {
 		        *flags &= ~CPU_BIND_VERBOSE;
 		} else if ((strcasecmp(tok, "v") == 0) ||
