@@ -72,7 +72,7 @@
  * of the plugin.  If major and minor revisions are desired, the major
  * version number may be multiplied by a suitable magnitude constant such
  * as 100 or 1000.  Various SLURM versions will likely require a certain
- * minimum versions for their plugins as the job accounting API
+ * minimum version for their plugins as the job accounting API
  * matures.
  */
 const char plugin_name[] = "Job completion POSTGRESQL plugin";
@@ -164,7 +164,7 @@ static int _pgsql_jobcomp_check_tables(char *user)
 	PQclear(result);
 
 	if(!job_found)
-		if(pgsql_db_create_table(jobcomp_pgsql_db, jobcomp_table,
+		if(pgsql_db_create_table(jobcomp_pgsql_db, "public", jobcomp_table,
 					 jobcomp_table_fields,
 					 ")") == SLURM_ERROR)
 			return SLURM_ERROR;
@@ -309,11 +309,11 @@ extern int slurm_jobcomp_log_record(struct job_record *job_ptr)
 	int rc = SLURM_SUCCESS;
 	char *usr_str = NULL, *grp_str = NULL, lim_str[32];
 	char *connect_type = NULL, *reboot = NULL, *rotate = NULL,
-		*maxprocs = NULL, *geometry = NULL, *start = NULL,
+		*geometry = NULL, *start = NULL,
 		*blockid = NULL;
 	enum job_states job_state;
 	char *query = NULL;
-	uint32_t time_limit;
+	uint32_t time_limit, start_time, end_time;
 
 	if(!jobcomp_pgsql_db || PQstatus(jobcomp_pgsql_db) != CONNECTION_OK) {
 		char *loc = slurm_get_jobcomp_loc();
@@ -341,7 +341,25 @@ extern int slurm_jobcomp_log_record(struct job_record *job_ptr)
 	/* Job will typically be COMPLETING when this is called.
 	 * We remove the flags to get the eventual completion state:
 	 * JOB_FAILED, JOB_TIMEOUT, etc. */
-	job_state = job_ptr->job_state & JOB_STATE_BASE;
+	if (IS_JOB_RESIZING(job_ptr)) {
+		job_state = JOB_RESIZING;
+		if (job_ptr->resize_time)
+			start_time = job_ptr->resize_time;
+		else
+			start_time = job_ptr->start_time;
+		end_time = time(NULL);
+	} else {
+		job_state = job_ptr->job_state & JOB_STATE_BASE;
+		if (job_ptr->resize_time)
+			start_time = job_ptr->resize_time;
+		else if (job_ptr->start_time > job_ptr->end_time) {
+			/* Job cancelled while pending and
+			 * expected start time is in the future. */
+			start_time = 0;
+		} else
+			start_time = job_ptr->start_time;
+		end_time = job_ptr->end_time;
+	}
 
 	connect_type = select_g_select_jobinfo_xstrdup(job_ptr->select_jobinfo,
 						       SELECT_PRINT_CONNECTION);
@@ -349,8 +367,6 @@ extern int slurm_jobcomp_log_record(struct job_record *job_ptr)
 						 SELECT_PRINT_REBOOT);
 	rotate = select_g_select_jobinfo_xstrdup(job_ptr->select_jobinfo,
 						 SELECT_PRINT_ROTATE);
-	maxprocs = select_g_select_jobinfo_xstrdup(job_ptr->select_jobinfo,
-						   SELECT_PRINT_MAX_CPUS);
 	geometry = select_g_select_jobinfo_xstrdup(job_ptr->select_jobinfo,
 						   SELECT_PRINT_GEOMETRY);
 	start = select_g_select_jobinfo_xstrdup(job_ptr->select_jobinfo,
@@ -376,7 +392,7 @@ extern int slurm_jobcomp_log_record(struct job_record *job_ptr)
 		xstrcat(query, ", reboot");
 	if(rotate)
 		xstrcat(query, ", rotate");
-	if(maxprocs)
+	if(job_ptr->details && (job_ptr->details->max_cpus != NO_VAL))
 		xstrcat(query, ", maxprocs");
 	if(geometry)
 		xstrcat(query, ", geometry");
@@ -389,9 +405,8 @@ extern int slurm_jobcomp_log_record(struct job_record *job_ptr)
 		   "'%s', \"%s\", %u, %u,  %u",
 		   job_ptr->job_id, job_ptr->user_id, usr_str,
 		   job_ptr->group_id, grp_str, job_ptr->name,
-		   job_state, job_ptr->total_procs, job_ptr->partition, lim_str,
-		   (int)job_ptr->start_time, (int)job_ptr->end_time,
-		   job_ptr->node_cnt);
+		   job_state, job_ptr->total_cpus, job_ptr->partition, lim_str,
+		   start_time, end_time, job_ptr->node_cnt);
 
 	if(job_ptr->nodes)
 		xstrfmtcat(query, ", '%s'", job_ptr->nodes);
@@ -408,10 +423,8 @@ extern int slurm_jobcomp_log_record(struct job_record *job_ptr)
 		xstrfmtcat(query, ", '%s'", rotate);
 		xfree(rotate);
 	}
-	if(maxprocs) {
-		xstrfmtcat(query, ", '%s'", maxprocs);
-		xfree(maxprocs);
-	}
+	if(job_ptr->details && (job_ptr->details->max_cpus != NO_VAL))
+		xstrfmtcat(query, ", '%u'", job_ptr->details->max_cpus);
 	if(geometry) {
 		xstrfmtcat(query, ", '%s'", geometry);
 		xfree(geometry);
@@ -449,7 +462,7 @@ extern char *slurm_jobcomp_strerror(int errnum)
  * in/out job_list List of job_rec_t *
  * note List needs to be freed when called
  */
-extern List slurm_jobcomp_get_jobs(acct_job_cond_t *job_cond)
+extern List slurm_jobcomp_get_jobs(slurmdb_job_cond_t *job_cond)
 {
 	List job_list = NULL;
 
@@ -470,7 +483,7 @@ extern List slurm_jobcomp_get_jobs(acct_job_cond_t *job_cond)
 /*
  * expire old info from the storage
  */
-extern int slurm_jobcomp_archive(acct_archive_cond_t *arch_cond)
+extern int slurm_jobcomp_archive(slurmdb_archive_cond_t *arch_cond)
 {
 	if(!jobcomp_pgsql_db || PQstatus(jobcomp_pgsql_db) != CONNECTION_OK) {
 		char *loc = slurm_get_jobcomp_loc();
