@@ -1,7 +1,8 @@
 /*****************************************************************************\
  *  builtin_wrapper.c - NO-OP plugin for SLURM's internal scheduler.
  *****************************************************************************
- *  Copyright (C) 2002 The Regents of the University of California.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
+ *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Kevin Tew <tew1@llnl.gov> et. al.
  *  CODE-OCEC-09-009. All rights reserved.
@@ -41,11 +42,12 @@
 
 #include "src/common/plugin.h"
 #include "src/common/log.h"
-#include "src/slurmctld/slurmctld.h"
+#include "src/common/node_select.h"
 #include "src/common/slurm_priority.h"
 #include "src/slurmctld/job_scheduler.h"
 #include "src/slurmctld/reservation.h"
-#include "src/common/node_select.h"
+#include "src/slurmctld/slurmctld.h"
+#include "src/plugins/sched/builtin/builtin.h"
 
 const char		plugin_name[]	= "SLURM Built-in Scheduler plugin";
 const char		plugin_type[]	= "sched/builtin";
@@ -54,12 +56,33 @@ const uint32_t		plugin_version	= 100;
 /* A plugin-global errno. */
 static int plugin_errno = SLURM_SUCCESS;
 
+static pthread_t builtin_thread = 0;
+static pthread_mutex_t thread_flag_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 /**************************************************************************/
 /*  TAG(                              init                              ) */
 /**************************************************************************/
 int init( void )
 {
-	verbose( "Built-in scheduler plugin loaded" );
+	pthread_attr_t attr;
+
+	verbose( "sched: Built-in scheduler plugin loaded" );
+
+	pthread_mutex_lock( &thread_flag_mutex );
+	if ( builtin_thread ) {
+		debug2( "Built-in scheduler thread already running, "
+			"not starting another" );
+		pthread_mutex_unlock( &thread_flag_mutex );
+		return SLURM_ERROR;
+	}
+
+	slurm_attr_init( &attr );
+	/* since we do a join on this later we don't make it detached */
+	if (pthread_create( &builtin_thread, &attr, builtin_agent, NULL))
+		error("Unable to start built-in scheduler thread: %m");
+	pthread_mutex_unlock( &thread_flag_mutex );
+	slurm_attr_destroy( &attr );
+
 	return SLURM_SUCCESS;
 }
 
@@ -68,7 +91,14 @@ int init( void )
 /**************************************************************************/
 void fini( void )
 {
-	/* Empty. */
+	pthread_mutex_lock( &thread_flag_mutex );
+	if ( builtin_thread ) {
+		verbose( "Built-in scheduler plugin shutting down" );
+		stop_builtin_agent();
+		pthread_join(builtin_thread, NULL);
+		builtin_thread = 0;
+	}
+	pthread_mutex_unlock( &thread_flag_mutex );
 }
 
 /**************************************************************************/
@@ -76,6 +106,7 @@ void fini( void )
 /**************************************************************************/
 int slurm_sched_plugin_reconfig( void )
 {
+	builtin_reconfig();
 	return SLURM_SUCCESS;
 }
 
@@ -125,59 +156,7 @@ slurm_sched_plugin_initial_priority( uint32_t last_prio,
 /**************************************************************************/
 void slurm_sched_plugin_job_is_pending( void )
 {
-	int rc = SLURM_SUCCESS;
-	int i, j, job_queue_size;
-	List preemptee_candidates = NULL;
-	struct job_queue *job_queue = NULL;
-	struct job_record *job_ptr;
-	struct part_record *part_ptr;
-	bitstr_t *avail_bitmap = NULL;
-	uint32_t max_nodes, min_nodes, req_nodes;
-	time_t now = time(NULL);
-
-	job_queue_size = build_job_queue(&job_queue);
-	if (job_queue_size == 0) return;
-
-	sort_job_queue(job_queue, job_queue_size);
-
-	for (i = 0; i < job_queue_size; i++) {
-		job_ptr = job_queue[i].job_ptr;
-		part_ptr = job_ptr->part_ptr;
-
-		/* Determine minimum and maximum node counts */
-		min_nodes = MAX(job_ptr->details->min_nodes,
-				part_ptr->min_nodes);
-
-		if (job_ptr->details->max_nodes == 0)
-			max_nodes = part_ptr->max_nodes;
-		else
-			max_nodes = MIN(job_ptr->details->max_nodes,
-					part_ptr->max_nodes);
-
-		max_nodes = MIN(max_nodes, 500000);     /* prevent overflows */
-
-		if (job_ptr->details->max_nodes)
-			req_nodes = max_nodes;
-		else
-			req_nodes = min_nodes;
-
-		if (min_nodes > max_nodes) {
-			/* job's min_nodes exceeds partition's max_nodes */
-			continue;
-		}
-
-		j = job_test_resv(job_ptr, &now, true, &avail_bitmap);
-		if (j != SLURM_SUCCESS)
-			continue;
-
-		rc = select_g_job_test(job_ptr, avail_bitmap,
-				       min_nodes, max_nodes, req_nodes,
-				       SELECT_MODE_WILL_RUN,
-				       preemptee_candidates, NULL);
-
-		FREE_NULL_BITMAP(avail_bitmap);
-	}
-	xfree(job_queue);
+	/* Empty. */
 }
 
 /**************************************************************************/
