@@ -57,18 +57,16 @@ extern List create_dynamic_block(List block_list,
 {
 	int rc = SLURM_SUCCESS;
 
-	ListIterator itr;
-	bg_record_t *bg_record = NULL;
+	ListIterator itr, itr2;
+	bg_record_t *bg_record = NULL, *found_record = NULL;
 	List results = NULL;
 	List new_blocks = NULL;
 	bitstr_t *my_bitmap = NULL;
-	int geo[BA_SYSTEM_DIMENSIONS];
-	int i;
 	blockreq_t blockreq;
 	int cnodes = request->procs / bg_conf->cpu_ratio;
 	char *unusable_nodes = NULL;
 
-	if(cnodes < bg_conf->smallest_block) {
+	if (cnodes < bg_conf->smallest_block) {
 		error("Can't create this size %d "
 		      "on this system numpsets is %d",
 		      request->procs,
@@ -77,50 +75,110 @@ extern List create_dynamic_block(List block_list,
 	}
 	memset(&blockreq, 0, sizeof(blockreq_t));
 
+	/* We need to lock this just incase a blocks_overlap is called
+	   which will in turn reset and set the system as it sees fit.
+	*/
 	slurm_mutex_lock(&block_state_mutex);
-	if(my_block_list) {
+	if (my_block_list) {
 		reset_ba_system(track_down_nodes);
 		itr = list_iterator_create(my_block_list);
 		while ((bg_record = list_next(itr))) {
-			if(!my_bitmap) {
+			if (bg_record->magic != BLOCK_MAGIC) {
+				/* This should never happen since we
+				   only call this on copies of blocks
+				   and we check on this during the
+				   copy.
+				*/
+				error("create_dynamic_block: "
+				      "got a block with bad magic?");
+				continue;
+			}
+			if (bg_record->free_cnt) {
+				if (bg_conf->slurm_debug_flags
+				    & DEBUG_FLAG_BG_PICK) {
+					char *start_geo = give_geo(
+						bg_record->start);
+					char *geo = give_geo(bg_record->geo);
+
+					info("not adding %s(%s) %s %s %s %u "
+					     "(free_cnt)",
+					     bg_record->bg_block_id,
+					     bg_record->nodes,
+					     bg_block_state_string(
+						     bg_record->state),
+					     start_geo,
+					     geo,
+					     bg_record->node_cnt);
+					xfree(start_geo);
+					xfree(geo);
+				}
+				continue;
+			}
+
+			if (!my_bitmap) {
 				my_bitmap =
 					bit_alloc(bit_size(bg_record->bitmap));
 			}
 
-			if(!bit_super_set(bg_record->bitmap, my_bitmap)) {
+			if (!bit_super_set(bg_record->bitmap, my_bitmap)) {
 				bit_or(my_bitmap, bg_record->bitmap);
-				for(i=0; i<BA_SYSTEM_DIMENSIONS; i++)
-					geo[i] = bg_record->geo[i];
-				debug2("adding %s %c%c%c %c%c%c",
-				       bg_record->nodes,
-				       alpha_num[bg_record->start[X]],
-				       alpha_num[bg_record->start[Y]],
-				       alpha_num[bg_record->start[Z]],
-				       alpha_num[geo[X]],
-				       alpha_num[geo[Y]],
-				       alpha_num[geo[Z]]);
 
-				if(check_and_set_node_list(
-					   bg_record->bg_block_list)
-				   == SLURM_ERROR) {
-					debug2("something happened in "
-					       "the load of %s",
-					       bg_record->bg_block_id);
+				if (bg_conf->slurm_debug_flags
+				    & DEBUG_FLAG_BG_PICK) {
+					char *start_geo =
+						give_geo(bg_record->start);
+					char *geo =
+						give_geo(bg_record->geo);
+
+					info("adding %s(%s) %s %s %s %u",
+					     bg_record->bg_block_id,
+					     bg_record->nodes,
+					     bg_block_state_string(
+						     bg_record->state),
+					     start_geo, geo,
+					     bg_record->node_cnt);
+					xfree(start_geo);
+					xfree(geo);
+				}
+				if (check_and_set_node_list(
+					    bg_record->bg_block_list)
+				    == SLURM_ERROR) {
+					if (bg_conf->slurm_debug_flags
+					    & DEBUG_FLAG_BG_PICK)
+						info("something happened in "
+						     "the load of %s",
+						     bg_record->bg_block_id);
 					list_iterator_destroy(itr);
 					FREE_NULL_BITMAP(my_bitmap);
 					rc = SLURM_ERROR;
 					goto finished;
 				}
+			} else if (bg_conf->slurm_debug_flags
+				   & DEBUG_FLAG_BG_PICK) {
+				char *start_geo = give_geo(bg_record->start);
+				char *geo = give_geo(bg_record->geo);
+
+				info("not adding %s(%s) %s %s %s %u ",
+				     bg_record->bg_block_id,
+				     bg_record->nodes,
+				     bg_block_state_string(
+					     bg_record->state),
+				     start_geo,
+				     geo,
+				     bg_record->node_cnt);
+				xfree(start_geo);
+				xfree(geo);
 			}
 		}
 		list_iterator_destroy(itr);
 		FREE_NULL_BITMAP(my_bitmap);
 	} else {
 		reset_ba_system(false);
-		debug("No list was given");
+		if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+			info("No list was given");
 	}
 
-	if(request->avail_node_bitmap) {
+	if (request->avail_node_bitmap) {
 		bitstr_t *bitmap = bit_alloc(node_record_count);
 
 		/* we want the bps that aren't in this partition to
@@ -136,7 +194,7 @@ extern List create_dynamic_block(List block_list,
 		FREE_NULL_BITMAP(bitmap);
 	}
 
-	if(request->size==1 && cnodes < bg_conf->bp_node_cnt) {
+	if (request->size==1 && cnodes < bg_conf->bp_node_cnt) {
 		switch(cnodes) {
 #ifdef HAVE_BGL
 		case 32:
@@ -185,51 +243,51 @@ extern List create_dynamic_block(List block_list,
 		request->conn_type = SELECT_SMALL;
 		new_blocks = list_create(destroy_bg_record);
 		/* check only blocks that are free and small */
-		if(_breakup_blocks(block_list, new_blocks,
-				   request, my_block_list,
-				   true, true)
-		   == SLURM_SUCCESS)
+		if (_breakup_blocks(block_list, new_blocks,
+				    request, my_block_list,
+				    true, true)
+		    == SLURM_SUCCESS)
 			goto finished;
 
 		/* check only blocks that are free and any size */
-		if(_breakup_blocks(block_list, new_blocks,
-				   request, my_block_list,
-				   true, false)
-		   == SLURM_SUCCESS)
+		if (_breakup_blocks(block_list, new_blocks,
+				    request, my_block_list,
+				    true, false)
+		    == SLURM_SUCCESS)
 			goto finished;
 
 		/* check usable blocks that are small with any state */
-		if(_breakup_blocks(block_list, new_blocks,
-				   request, my_block_list,
-				   false, true)
-		   == SLURM_SUCCESS)
+		if (_breakup_blocks(block_list, new_blocks,
+				    request, my_block_list,
+				    false, true)
+		    == SLURM_SUCCESS)
 			goto finished;
 
 		/* check all usable blocks */
-		if(_breakup_blocks(block_list, new_blocks,
-				   request, my_block_list,
-				   false, false)
-		   == SLURM_SUCCESS)
+		if (_breakup_blocks(block_list, new_blocks,
+				    request, my_block_list,
+				    false, false)
+		    == SLURM_SUCCESS)
 			goto finished;
 
 		/* Re-sort the list back to the original order. */
 		list_sort(block_list, (ListCmpF)bg_record_sort_aval_inc);
 		list_destroy(new_blocks);
 		new_blocks = NULL;
-		debug2("small block not able to be placed inside others");
+		if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+			info("small block not able to be placed inside others");
 	}
 
-	if(request->conn_type == SELECT_NAV)
+	if (request->conn_type == SELECT_NAV)
 		request->conn_type = SELECT_TORUS;
 
 	//debug("going to create %d", request->size);
-	if(!new_ba_request(request)) {
-		if(geo[X] == (uint16_t)NO_VAL) {
-			error("Problems with request for size %d geo %dx%dx%d",
-			      request->size,
-			      request->geometry[X],
-			      request->geometry[Y],
-			      request->geometry[Z]);
+	if (!new_ba_request(request)) {
+		if (request->geometry[X] != (uint16_t)NO_VAL) {
+			char *geo = give_geo(request->geometry);
+			error("Problems with request for size %d geo %s",
+			      request->size, geo);
+			xfree(geo);
 		} else {
 			error("Problems with request for size %d.  "
 			      "No geo given.",
@@ -241,7 +299,7 @@ extern List create_dynamic_block(List block_list,
 
 	/* try on free midplanes */
 	rc = SLURM_SUCCESS;
-	if(results)
+	if (results)
 		list_flush(results);
 	else
 		results = list_create(NULL);
@@ -249,19 +307,22 @@ extern List create_dynamic_block(List block_list,
 	if (allocate_block(request, results))
 		goto setup_records;
 
-	debug2("allocate failure for size %d base "
-	       "partitions of free midplanes",
-	       request->size);
+	if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+		info("allocate failure for size %d base "
+		     "partitions of free midplanes",
+		     request->size);
 	rc = SLURM_ERROR;
 
-	if(!list_count(block_list) || !my_block_list)
+	if (!list_count(block_list) || !my_block_list)
 		goto finished;
 
 	/*Try to put block starting in the smallest of the exisiting blocks*/
 	itr = list_iterator_create(block_list);
+	itr2 = list_iterator_create(block_list);
 	while ((bg_record = (bg_record_t *) list_next(itr)) != NULL) {
-		/* never check a block with a job running */
-		if(bg_record->job_running != NO_JOB_RUNNING)
+		/* never check a block being deleted or with a job running */
+		if (bg_record->free_cnt
+		    || bg_record->job_running != NO_JOB_RUNNING)
 			continue;
 
 		/* Here we are only looking for the first
@@ -270,33 +331,53 @@ extern List create_dynamic_block(List block_list,
 		   bg_conf->bp_node_cnt or the first bit is
 		   set in the ionode_bitmap.
 		*/
-		if((bg_record->node_cnt < bg_conf->bp_node_cnt)
-		   && (bit_ffs(bg_record->ionode_bitmap) != 0))
-			continue;
+		if (bg_record->node_cnt < bg_conf->bp_node_cnt) {
+			bool found = 0;
+			if (bit_ffs(bg_record->ionode_bitmap) != 0)
+				continue;
+			/* Check to see if we have other blocks in
+			   this midplane that have jobs running.
+			*/
+			while ((found_record = list_next(itr2))) {
+				if ((found_record->job_running
+				     != NO_JOB_RUNNING)
+				    && bit_overlap(bg_record->bitmap,
+						   found_record->bitmap)) {
+					found = 1;
+					break;
+				}
+			}
+			list_iterator_reset(itr2);
+			if (found)
+				continue;
+		}
 
-		debug3("removing %s for request %d",
-		       bg_record->nodes, request->size);
+		if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+			info("removing %s for request %d",
+			     bg_record->nodes, request->size);
 		remove_block(bg_record->bg_block_list, (int)NO_VAL,
 			     (int)bg_record->conn_type);
 		/* need to set any unusable nodes that this last block
 		   used */
 		removable_set_bps(unusable_nodes);
 		rc = SLURM_SUCCESS;
-		if(results)
+		if (results)
 			list_flush(results);
 		else
 			results = list_create(NULL);
 		if (allocate_block(request, results))
 			break;
 
-		debug2("allocate failure for size %d base partitions",
-		       request->size);
+		if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+			info("allocate failure for size %d base partitions",
+			     request->size);
 		rc = SLURM_ERROR;
 	}
 	list_iterator_destroy(itr);
+	list_iterator_destroy(itr2);
 
 setup_records:
-	if(rc == SLURM_SUCCESS) {
+	if (rc == SLURM_SUCCESS) {
 		/*set up bg_record(s) here */
 		new_blocks = list_create(destroy_bg_record);
 
@@ -314,18 +395,18 @@ setup_records:
 
 finished:
 	reset_all_removed_bps();
+	slurm_mutex_unlock(&block_state_mutex);
 
 	xfree(unusable_nodes);
 	xfree(request->save_name);
 
-	if(request->elongate_geos) {
+	if (request->elongate_geos) {
 		list_destroy(request->elongate_geos);
 		request->elongate_geos = NULL;
 	}
-	if(results)
+	if (results)
 		list_destroy(results);
 	errno = rc;
-	slurm_mutex_unlock(&block_state_mutex);
 
 	return new_blocks;
 }
@@ -339,15 +420,16 @@ extern bg_record_t *create_small_record(bg_record_t *bg_record,
 	char bitstring[BITSIZE];
 
 	found_record = (bg_record_t*) xmalloc(sizeof(bg_record_t));
+	found_record->magic = BLOCK_MAGIC;
 
 	found_record->job_running = NO_JOB_RUNNING;
 	found_record->user_name = xstrdup(bg_record->user_name);
 	found_record->user_uid = bg_record->user_uid;
 	found_record->bg_block_list = list_create(destroy_ba_node);
-	if(bg_record->bg_block_list)
+	if (bg_record->bg_block_list)
 		ba_node = list_peek(bg_record->bg_block_list);
-	if(!ba_node) {
-		if(bg_record->nodes) {
+	if (!ba_node) {
+		if (bg_record->nodes) {
 			hostlist_t hl = hostlist_create(bg_record->nodes);
 			char *host = hostlist_shift(hl);
 			hostlist_destroy(hl);
@@ -369,11 +451,11 @@ extern bg_record_t *create_small_record(bg_record_t *bg_record,
 	} else {
 		int i=0,j=0;
 		new_ba_node = ba_copy_node(ba_node);
-		for (i=0; i<BA_SYSTEM_DIMENSIONS; i++){
+		for (i=0; i<SYSTEM_DIMENSIONS; i++){
 			for(j=0;j<NUM_PORTS_PER_NODE;j++) {
 				ba_node->axis_switch[i].int_wire[j].used = 0;
-				if(i!=X) {
-					if(j==3 || j==4)
+				if (i!=X) {
+					if (j==3 || j==4)
 						ba_node->axis_switch[i].
 							int_wire[j].
 							used = 1;
@@ -410,8 +492,9 @@ extern bg_record_t *create_small_record(bg_record_t *bg_record,
 	found_record->ionode_bitmap = bit_copy(ionodes);
 	bit_fmt(bitstring, BITSIZE, found_record->ionode_bitmap);
 	found_record->ionodes = xstrdup(bitstring);
-	debug4("made small block of %s[%s]",
-	       found_record->nodes, found_record->ionodes);
+	if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+		info("made small block of %s[%s]",
+		     found_record->nodes, found_record->ionodes);
 	return found_record;
 }
 
@@ -580,23 +663,25 @@ static int _split_block(List block_list, List new_blocks,
 #endif
 	}
 
-	if(!full_bp && bg_record->ionode_bitmap) {
-		if((start = bit_ffs(bg_record->ionode_bitmap)) == -1)
+	if (!full_bp && bg_record->ionode_bitmap) {
+		if ((start = bit_ffs(bg_record->ionode_bitmap)) == -1)
 			start = 0;
 	}
 
 #ifdef HAVE_BGL
-	debug2("Asking for %u 32CNBlocks, and %u 128CNBlocks "
-	       "from a %u block, starting at ionode %d.",
-	       blockreq.small32, blockreq.small128,
-	       bg_record->node_cnt, start);
+	if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+		info("Asking for %u 32CNBlocks, and %u 128CNBlocks "
+		     "from a %u block, starting at ionode %d.",
+		     blockreq.small32, blockreq.small128,
+		     bg_record->node_cnt, start);
 #else
-	debug2("Asking for %u 16CNBlocks, %u 32CNBlocks, "
-	       "%u 64CNBlocks, %u 128CNBlocks, and %u 256CNBlocks"
-	       "from a %u block, starting at ionode %d.",
-	       blockreq.small16, blockreq.small32,
-	       blockreq.small64, blockreq.small128,
-	       blockreq.small256, bg_record->node_cnt, start);
+	if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+		info("Asking for %u 16CNBlocks, %u 32CNBlocks, "
+		     "%u 64CNBlocks, %u 128CNBlocks, and %u 256CNBlocks "
+		     "from a %u block, starting at ionode %d.",
+		     blockreq.small16, blockreq.small32,
+		     blockreq.small64, blockreq.small128,
+		     blockreq.small256, bg_record->node_cnt, start);
 #endif
 	handle_small_record_request(new_blocks, &blockreq, bg_record, start);
 
@@ -617,8 +702,9 @@ static int _breakup_blocks(List block_list, List new_blocks,
 	int cnodes = request->procs / bg_conf->cpu_ratio;
 	int curr_bp_bit = -1;
 
-	debug2("cpu_count= %d cnodes=%d o_free=%d o_small=%d",
-	       request->procs, cnodes, only_free, only_small);
+	if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+		info("cpu_count=%d cnodes=%d o_free=%d o_small=%d",
+		     request->procs, cnodes, only_free, only_small);
 
 	switch(cnodes) {
 	case 16:
@@ -647,49 +733,59 @@ static int _breakup_blocks(List block_list, List new_blocks,
 	 */
 	itr = list_iterator_create(block_list);
 	while ((bg_record = list_next(itr))) {
+		if (bg_record->free_cnt) {
+			if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+				info("%s being free for other job(s), skipping",
+				     bg_record->bg_block_id);
+			continue;
+		}
 		/* never look at a block if a job is running */
-		if(bg_record->job_running != NO_JOB_RUNNING)
+		if (bg_record->job_running != NO_JOB_RUNNING)
 			continue;
 		/* on the third time through look for just a block
 		 * that isn't used */
 
 		/* check for free blocks on the first and second time */
-		if(only_free && (bg_record->state != RM_PARTITION_FREE))
+		if (only_free && (bg_record->state != RM_PARTITION_FREE))
 			continue;
 
 		/* check small blocks first */
-		if(only_small && (bg_record->node_cnt > bg_conf->bp_node_cnt))
+		if (only_small && (bg_record->node_cnt > bg_conf->bp_node_cnt))
 			continue;
 
 		if (request->avail_node_bitmap &&
 		    !bit_super_set(bg_record->bitmap,
 				   request->avail_node_bitmap)) {
-			debug2("bg block %s has nodes not usable by this job",
-			       bg_record->bg_block_id);
+			if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+				info("bg block %s has nodes not usable "
+				     "by this job",
+				     bg_record->bg_block_id);
 			continue;
 		}
 
-		if(bg_record->node_cnt == cnodes) {
-			debug2("found it here %s, %s",
-			       bg_record->bg_block_id,
-			       bg_record->nodes);
+		if (bg_record->node_cnt == cnodes) {
+			if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+				info("found it here %s, %s",
+				     bg_record->bg_block_id,
+				     bg_record->nodes);
 			request->save_name = xstrdup_printf(
 				"%c%c%c",
 				alpha_num[bg_record->start[X]],
 				alpha_num[bg_record->start[Y]],
 				alpha_num[bg_record->start[Z]]);
+
 			rc = SLURM_SUCCESS;
 			goto finished;
 		}
 		/* lets see if we can combine some small ones */
-		if(bg_record->node_cnt < cnodes) {
+		if (bg_record->node_cnt < cnodes) {
 			char bitstring[BITSIZE];
 			bitstr_t *bitstr = NULL;
 			int num_over = 0;
 			int num_cnodes = bg_record->node_cnt;
 			int rec_bp_bit = bit_ffs(bg_record->bitmap);
 
-			if(curr_bp_bit != rec_bp_bit) {
+			if (curr_bp_bit != rec_bp_bit) {
 				/* Got a different node than
 				 * previously, since the list should
 				 * be in order of nodes for small blocks
@@ -704,29 +800,29 @@ static int _breakup_blocks(List block_list, List new_blocks,
 			   overlapping blocks here.  If that is the
 			   case only add that which doesn't overlap.
 			*/
-			if((num_over = bit_overlap(
-				    ionodes, bg_record->ionode_bitmap))) {
+			if ((num_over = bit_overlap(
+				     ionodes, bg_record->ionode_bitmap))) {
 				/* Since the smallest block size is
 				   the number of cnodes in an io node,
 				   just multiply the num_over by that to
 				   get the number of cnodes to remove.
 				*/
-				if((num_cnodes -=
-				    num_over * bg_conf->smallest_block) <= 0)
+				if ((num_cnodes -=
+				     num_over * bg_conf->smallest_block) <= 0)
 					continue;
 			}
 			bit_or(ionodes, bg_record->ionode_bitmap);
 
 			/* check and see if the bits set are a valid
 			   combo */
-			if(bit_itr) {
-				while((bitstr = list_next(bit_itr))) {
-					if(bit_super_set(ionodes, bitstr))
+			if (bit_itr) {
+				while ((bitstr = list_next(bit_itr))) {
+					if (bit_super_set(ionodes, bitstr))
 						break;
 				}
 				list_iterator_reset(bit_itr);
 			}
-			if(!bitstr) {
+			if (!bitstr) {
 				bit_nclear(ionodes, 0, (bg_conf->numpsets-1));
 				bit_or(ionodes, bg_record->ionode_bitmap);
 				total_cnode_cnt = num_cnodes =
@@ -735,18 +831,19 @@ static int _breakup_blocks(List block_list, List new_blocks,
 				total_cnode_cnt += num_cnodes;
 
 			bit_fmt(bitstring, BITSIZE, ionodes);
-			debug2("1 adding %s %s %d got %d set "
-			       "ionodes %s total is %s",
-			       bg_record->bg_block_id, bg_record->nodes,
-			       num_cnodes, total_cnode_cnt,
-			       bg_record->ionodes, bitstring);
-			if(total_cnode_cnt == cnodes) {
+			if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+				info("combine adding %s %s %d got %d set "
+				     "ionodes %s total is %s",
+				     bg_record->bg_block_id, bg_record->nodes,
+				     num_cnodes, total_cnode_cnt,
+				     bg_record->ionodes, bitstring);
+			if (total_cnode_cnt == cnodes) {
 				request->save_name = xstrdup_printf(
 					"%c%c%c",
 					alpha_num[bg_record->start[X]],
 					alpha_num[bg_record->start[Y]],
 					alpha_num[bg_record->start[Z]]);
-				if(!my_block_list) {
+				if (!my_block_list) {
 					rc = SLURM_SUCCESS;
 					goto finished;
 				}
@@ -765,48 +862,65 @@ static int _breakup_blocks(List block_list, List new_blocks,
 		break;
 	}
 
-	if(bg_record) {
-		bg_record_t *found_record = NULL;
+	if (bg_record) {
+		/* It appears we don't need this original record
+		 * anymore, just work off the copy if indeed it is a copy. */
 
-		if(bg_record->original) {
-			debug3("This was a copy");
-			found_record = bg_record->original;
-		} else {
-			debug3("looking for original");
-			found_record = find_org_in_bg_list(
-				bg_lists->main, bg_record);
-		}
-		if(!found_record) {
-			error("this record wasn't found in the list!");
-			rc = SLURM_ERROR;
-			goto finished;
+		/* bg_record_t *found_record = NULL; */
+
+		/* if (bg_record->original) { */
+		/* 	if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK) */
+		/* 		info("1 This was a copy %s", */
+		/* 		     bg_record->bg_block_id); */
+		/* 	found_record = bg_record->original; */
+		/* } else { */
+		/* 	if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK) */
+		/* 		info("looking for original"); */
+		/* 	found_record = find_org_in_bg_list( */
+		/* 		bg_lists->main, bg_record); */
+		/* } */
+		if ((bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK)
+		    && bg_record->original
+		    && (bg_record->original->magic != BLOCK_MAGIC)) {
+			info("This record %s has bad magic, it must be "
+			     "getting freed.  No worries it will all be "
+			     "figured out later.",
+			     bg_record->bg_block_id);
 		}
 
-		format_node_name(found_record, tmp_char, sizeof(tmp_char));
+		/* if (!found_record || found_record->magic != BLOCK_MAGIC) { */
+		/* 	error("this record wasn't found in the list!"); */
+		/* 	rc = SLURM_ERROR; */
+		/* 	goto finished; */
+		/* } */
 
-		debug2("going to split %s, %s",
-		       found_record->bg_block_id,
-		       tmp_char);
+		if (bg_conf->slurm_debug_flags & DEBUG_FLAG_BG_PICK) {
+			format_node_name(bg_record, tmp_char,
+					 sizeof(tmp_char));
+			info("going to split %s, %s",
+			     bg_record->bg_block_id,
+			     tmp_char);
+		}
 		request->save_name = xstrdup_printf(
 			"%c%c%c",
-			alpha_num[found_record->start[X]],
-			alpha_num[found_record->start[Y]],
-			alpha_num[found_record->start[Z]]);
-		if(!my_block_list) {
+			alpha_num[bg_record->start[X]],
+			alpha_num[bg_record->start[Y]],
+			alpha_num[bg_record->start[Z]]);
+		if (!my_block_list) {
 			rc = SLURM_SUCCESS;
 			goto finished;
 		}
-		_split_block(block_list, new_blocks, found_record, cnodes);
+		_split_block(block_list, new_blocks, bg_record, cnodes);
 		rc = SLURM_SUCCESS;
 		goto finished;
 	}
 
 finished:
-	if(bit_itr)
+	if (bit_itr)
 		list_iterator_destroy(bit_itr);
 
 	FREE_NULL_BITMAP(ionodes);
-	if(itr)
+	if (itr)
 		list_iterator_destroy(itr);
 
 	return rc;
