@@ -148,7 +148,7 @@ static slurm_errtab_t slurm_errtab[] = {
  * of the plugin.  If major and minor revisions are desired, the major
  * version number may be multiplied by a suitable magnitude constant such
  * as 100 or 1000.  Various SLURM versions will likely require a certain
- * minimum versions for their plugins as this API matures.
+ * minimum version for their plugins as this API matures.
  */
 const char plugin_name[]        = "switch Quadrics Elan3 or Elan4 plugin";
 const char plugin_type[]        = "switch/elan";
@@ -370,7 +370,7 @@ int switch_p_build_jobinfo ( switch_jobinfo_t *switch_job, char *nodelist,
 				/* allocs hw context */
 	}
 
-	bit_free(nodeset);
+	FREE_NULL_BITMAP(nodeset);
 	return error_code;
 }
 
