@@ -40,6 +40,7 @@
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
 #include "src/common/parse_time.h"
+#include "src/common/uid.h"
 #include "sacct.h"
 #include <time.h>
 
@@ -56,7 +57,7 @@ void *acct_db_conn = NULL;
 List print_fields_list = NULL;
 ListIterator print_fields_itr = NULL;
 int field_count = 0;
-List qos_list = NULL;
+List g_qos_list = NULL;
 
 void _help_fields_msg(void)
 {
@@ -75,22 +76,22 @@ void _help_fields_msg(void)
 
 static char *_convert_to_id(char *name, bool gid)
 {
-	if(gid) {
-		struct group *grp;
-		if (!(grp=getgrnam(name))) {
+	if (gid) {
+		gid_t gid;
+		if ( gid_from_string( name, &gid ) != 0 ) {
 			fprintf(stderr, "Invalid group id: %s\n", name);
 			exit(1);
 		}
 		xfree(name);
-		name = xstrdup_printf("%d", grp->gr_gid);
+		name = xstrdup_printf( "%d", (int) gid );
 	} else {
-		struct passwd *pwd;
-		if (!(pwd=getpwnam(name))) {
+		uid_t uid;
+		if ( uid_from_string( name, &uid ) != 0 ) {
 			fprintf(stderr, "Invalid user id: %s\n", name);
 			exit(1);
 		}
 		xfree(name);
-		name = xstrdup_printf("%d", pwd->pw_uid);
+		name = xstrdup_printf( "%d", (int) uid );
 	}
 	return name;
 }
@@ -129,10 +130,7 @@ static int _addto_id_char_list(List char_list, char *names, bool gid)
 					name = xmalloc((i-start+1));
 					memcpy(name, names+start, (i-start));
 					//info("got %s %d", name, i-start);
-					if (!isdigit((int) *name)) {
-						name = _convert_to_id(
-							name, gid);
-					}
+					name = _convert_to_id( name, gid );
 
 					while((tmp_char = list_next(itr))) {
 						if(!strcasecmp(tmp_char, name))
@@ -160,10 +158,7 @@ static int _addto_id_char_list(List char_list, char *names, bool gid)
 		if((i-start) > 0) {
 			name = xmalloc((i-start)+1);
 			memcpy(name, names+start, (i-start));
-
-			if (!isdigit((int) *name)) {
-				name = _convert_to_id(name, gid);
-			}
+			name = _convert_to_id(name, gid);
 
 			while((tmp_char = list_next(itr))) {
 				if(!strcasecmp(tmp_char, name))
@@ -214,7 +209,7 @@ static int _addto_state_char_list(List char_list, char *names)
 				if((i-start) > 0) {
 					name = xmalloc((i-start+1));
 					memcpy(name, names+start, (i-start));
-					c = decode_state_char(name);
+					c = job_state_num(name);
 					if (c == -1)
 						fatal("unrecognized job "
 						      "state value");
@@ -247,7 +242,7 @@ static int _addto_state_char_list(List char_list, char *names)
 		if((i-start) > 0) {
 			name = xmalloc((i-start)+1);
 			memcpy(name, names+start, (i-start));
-			c = decode_state_char(name);
+			c = job_state_num(name);
 			if (c == -1)
 				fatal("unrecognized job state value");
 			xfree(name);
@@ -274,8 +269,8 @@ static int _addto_step_list(List step_list, char *names)
 {
 	int i=0, start=0;
 	char *name = NULL, *dot = NULL;
-	jobacct_selected_step_t *selected_step = NULL;
-	jobacct_selected_step_t *curr_step = NULL;
+	slurmdb_selected_step_t *selected_step = NULL;
+	slurmdb_selected_step_t *curr_step = NULL;
 
 	ListIterator itr = NULL;
 	char quote_c = '\0';
@@ -308,15 +303,21 @@ static int _addto_step_list(List step_list, char *names)
 					memcpy(name, names+start, (i-start));
 
 					selected_step = xmalloc(
-						sizeof(jobacct_selected_step_t));
+						sizeof(slurmdb_selected_step_t));
 					dot = strstr(name, ".");
 					if (dot == NULL) {
 						debug2("No jobstep requested");
 						selected_step->stepid = NO_VAL;
 					} else {
 						*dot++ = 0;
-						selected_step->stepid =
-							atoi(dot);
+						/* can't use NO_VAL
+						 * since that means all */
+						if (!strcmp(dot, "batch"))
+							selected_step->stepid =
+								INFINITE;
+						else
+							selected_step->stepid =
+								atoi(dot);
 					}
 					selected_step->jobid = atoi(name);
 					xfree(name);
@@ -335,7 +336,7 @@ static int _addto_step_list(List step_list, char *names)
 							    selected_step);
 						count++;
 					} else
-						destroy_jobacct_selected_step(
+						slurmdb_destroy_selected_step(
 							selected_step);
 					list_iterator_reset(itr);
 				}
@@ -349,14 +350,18 @@ static int _addto_step_list(List step_list, char *names)
 			memcpy(name, names+start, (i-start));
 
 			selected_step =
-				xmalloc(sizeof(jobacct_selected_step_t));
+				xmalloc(sizeof(slurmdb_selected_step_t));
 			dot = strstr(name, ".");
 			if (dot == NULL) {
 				debug2("No jobstep requested");
 				selected_step->stepid = NO_VAL;
 			} else {
 				*dot++ = 0;
-				selected_step->stepid = atoi(dot);
+				/* can't use NO_VAL since that means all */
+				if (!strcmp(dot, "batch"))
+					selected_step->stepid = INFINITE;
+				else
+					selected_step->stepid = atoi(dot);
 			}
 			selected_step->jobid = atoi(name);
 			xfree(name);
@@ -372,7 +377,7 @@ static int _addto_step_list(List step_list, char *names)
 				list_append(step_list, selected_step);
 				count++;
 			} else
-				destroy_jobacct_selected_step(
+				slurmdb_destroy_selected_step(
 					selected_step);
 		}
 	}
@@ -396,8 +401,6 @@ sacct [<OPTION>]                                                            \n\
 	           Equivalent to '--format=jobstep,state,error'. This option\n\
 	           has no effect if --dump is specified.                    \n\
      -c, --completion: Use job completion instead of accounting data.       \n\
-     -C, --clusters:                                                        \n\
-                   Only send data about these clusters. -1 for all clusters.\n\
      -d, --dump:   Dump the raw data records                                \n\
      -D, --duplicates:                                                      \n\
 	           If SLURM job ids are reset, some job numbers will        \n\
@@ -428,11 +431,20 @@ sacct [<OPTION>]                                                            \n\
 	           Format is <job(.step)>. Display information about this   \n\
                    job or comma-separated list of jobs. The default is all  \n\
                    jobs. Adding .step will display the specfic job step of  \n\
-                   that job.                                                \n\
+                   that job. (A step id of 'batch' will display the         \n\
+                   information about the batch step.)                       \n\
+     -k, --timelimit-min:                                                   \n\
+                   Only send data about jobs with this timelimit.           \n\
+                   If used with timelimit_max this will be the minimum      \n\
+                   timelimit of the range.  Default is no restriction.      \n\
+     -K, --timelimit-max:                                                   \n\
+                   Ignored by itself, but if timelimit_min is set this will \n\
+                   be the maximum timelimit of the range.  Default is no    \n\
+                   restriction.                                             \n\
      -l, --long:                                                            \n\
 	           Equivalent to specifying                                 \n\
-	           '--fields=jobid,jobname,partition,maxvsize,maxvsizenode, \n\
-                             maxvsizetask,avevsize,maxrss,maxrssnode,       \n\
+	           '--format=jobid,jobname,partition,maxvmsize,maxvmsizenode,\n\
+                             maxvmsizetask,avevmsize,maxrss,maxrssnode,     \n\
                              maxrsstask,averss,maxpages,maxpagesnode,       \n\
                              maxpagestask,avepages,mincpu,mincpunode,       \n\
                              mincputask,avecpu,ntasks,alloccpus,elapsed,    \n\
@@ -441,6 +453,8 @@ sacct [<OPTION>]                                                            \n\
 	           Display jobs ran on all clusters. By default, only jobs  \n\
                    ran on the cluster from where sacct is called are        \n\
                    displayed.                                               \n\
+     -M, --clusters:                                                        \n\
+                   Only send data about these clusters. -1 for all clusters.\n\
      -n, --noheader:                                                        \n\
 	           No header will be added to the beginning of output.      \n\
                    The default is to print a header; the option has no effect\n\
@@ -456,19 +470,19 @@ sacct [<OPTION>]                                                            \n\
                    primarily for debugging.                                 \n\
      -p, --parsable: output will be '|' delimited with a '|' at the end     \n\
      -P, --parsable2: output will be '|' delimited without a '|' at the end \n\
+     -q, --qos:                                                             \n\
+                   Only send data about jobs using these qos.  Default is all.\n\
      -r, --partition:                                                       \n\
 	           Comma separated list of partitions to select jobs and    \n\
                    job steps from. The default is all partitions.           \n\
      -s, --state:                                                           \n\
 	           Select jobs based on their current state or the state    \n\
                    they were in during the time period given: running (r),  \n\
-	           completed (cd), failed (f), timeout (to), and            \n\
-                   node_fail (nf).                                          \n\
+                   completed (cd), failed (f), timeout (to), resizing (rs)  \n\
+                   and node_fail (nf).                                      \n\
      -S, --starttime:                                                       \n\
                    Select jobs eligible after this time.  Default is        \n\
-                   midnight of current day.  If states are given with the -s\n\
-                   option then return jobs in this state at this time, 'now'\n\
-                   is also used as the default time.                        \n\
+                   midnight of current day.                                 \n\
      -T, --truncate:                                                        \n\
                    Truncate time.  So if a job started before --starttime   \n\
                    the start time would be truncated to --starttime.        \n\
@@ -508,48 +522,24 @@ void _usage(void)
 void _init_params()
 {
 	memset(&params, 0, sizeof(sacct_parameters_t));
-	params.job_cond = xmalloc(sizeof(acct_job_cond_t));
+	params.job_cond = xmalloc(sizeof(slurmdb_job_cond_t));
 	params.job_cond->without_usage_truncation = 1;
 }
 
-int decode_state_char(char *state)
-{
-	if (!strncasecmp(state, "p", 1))
-		return JOB_PENDING; 	/* we should never see this */
-	else if (!strncasecmp(state, "r", 1))
-		return JOB_RUNNING;
-	else if (!strncasecmp(state, "su", 1))
-		return JOB_SUSPENDED;
-	else if (!strncasecmp(state, "cd", 2))
-		return JOB_COMPLETE;
-	else if (!strncasecmp(state, "ca", 2))
-		return JOB_CANCELLED;
-	else if (!strncasecmp(state, "f", 1))
-		return JOB_FAILED;
-	else if (!strncasecmp(state, "to", 1))
-		return JOB_TIMEOUT;
-	else if (!strncasecmp(state, "nf", 1))
-		return JOB_NODE_FAIL;
-	else
-		return -1; // unknown
-}
-
 int get_data(void)
 {
-	jobacct_job_rec_t *job = NULL;
-	jobacct_step_rec_t *step = NULL;
+	slurmdb_job_rec_t *job = NULL;
+	slurmdb_step_rec_t *step = NULL;
 
 	ListIterator itr = NULL;
 	ListIterator itr_step = NULL;
-	acct_job_cond_t *job_cond = params.job_cond;
+	slurmdb_job_cond_t *job_cond = params.job_cond;
 
 	if(params.opt_completion) {
 		jobs = g_slurm_jobcomp_get_jobs(job_cond);
 		return SLURM_SUCCESS;
 	} else {
-
-		jobs = jobacct_storage_g_get_jobs_cond(acct_db_conn, getuid(),
-						       job_cond);
+		jobs = slurmdb_jobs_get(acct_db_conn, job_cond);
 	}
 
 	if (params.opt_fdump)
@@ -587,7 +577,7 @@ int get_data(void)
 			job->sys_cpu_usec +=
 				step->sys_cpu_usec;
 			/* get the max for all the sacct_t struct */
-			aggregate_sacct(&job->sacct, &step->sacct);
+			aggregate_stats(&job->stats, &step->stats);
 		}
 		list_iterator_destroy(itr_step);
 	}
@@ -601,26 +591,24 @@ void parse_command_line(int argc, char **argv)
 	extern int optind;
 	int c, i, optionIndex = 0;
 	char *end = NULL, *start = NULL, *acct_type = NULL;
-	jobacct_selected_step_t *selected_step = NULL;
+	slurmdb_selected_step_t *selected_step = NULL;
 	ListIterator itr = NULL;
 	struct stat stat_buf;
 	char *dot = NULL;
 	bool brief_output = FALSE, long_output = FALSE;
 	bool all_users = 0;
 	bool all_clusters = 0;
-	acct_job_cond_t *job_cond = params.job_cond;
+	slurmdb_job_cond_t *job_cond = params.job_cond;
 	log_options_t opts = LOG_OPTS_STDERR_ONLY ;
 	int verbosity;		/* count of -v options */
 	bool set;
 
 	static struct option long_options[] = {
 		{"allusers", 0,0, 'a'},
-		{"allclusters", 0,0, 'L'},
 		{"accounts", 1, 0, 'A'},
 		{"allocations", 0, &params.opt_allocs,  1},
 		{"brief", 0, 0, 'b'},
 		{"completion", 0, &params.opt_completion, 'c'},
-		{"clusters", 1, 0, 'C'},
 		{"dump", 0, 0, 'd'},
 		{"duplicates", 0, &params.opt_dup, 1},
 		{"helpformat", 0, 0, 'e'},
@@ -634,7 +622,12 @@ void parse_command_line(int argc, char **argv)
 		{"nnodes", 1, 0, 'i'},
 		{"ncpus", 1, 0, 'I'},
 		{"jobs", 1, 0, 'j'},
+		{"timelimit-min", 1, 0, 'k'},
+		{"timelimit-max", 1, 0, 'K'},
 		{"long", 0, 0, 'l'},
+		{"allclusters", 0,0, 'L'},
+		{"cluster", 1, 0, 'M'},
+		{"clusters", 1, 0, 'M'},
 		{"nodelist", 1, 0, 'N'},
 		{"noheader", 0, 0, 'n'},
 		{"fields", 1, 0, 'o'},
@@ -642,6 +635,7 @@ void parse_command_line(int argc, char **argv)
 		{"formatted_dump", 0, 0, 'O'},
 		{"parsable", 0, 0, 'p'},
 		{"parsable2", 0, 0, 'P'},
+		{"qos", 1, 0, 'q'},
 		{"partition", 1, 0, 'r'},
 		{"state", 1, 0, 's'},
 		{"starttime", 1, 0, 'S'},
@@ -664,7 +658,7 @@ void parse_command_line(int argc, char **argv)
 
 	while (1) {		/* now cycle through the command line */
 		c = getopt_long(argc, argv,
-				"aA:bcC:dDeE:f:g:hi:I:j:lLnN:o:OpPr:s:S:Ttu:vVW:x:X",
+				"aA:bcC:dDeE:f:g:hi:I:j:k:K:lLM:nN:o:OpPq:r:s:S:Ttu:vVW:x:X",
 				long_options, &optionIndex);
 		if (c == -1)
 			break;
@@ -685,6 +679,10 @@ void parse_command_line(int argc, char **argv)
 			params.opt_completion = 1;
 			break;
 		case 'C':
+			/* 'C' is deprecated since 'M' is cluster on
+			   everything else.
+			*/
+		case 'M':
 			if(!strcasecmp(optarg, "-1")) {
 				all_clusters = 1;
 				break;
@@ -706,6 +704,8 @@ void parse_command_line(int argc, char **argv)
 			break;
 		case 'E':
 			job_cond->usage_end = parse_time(optarg, 1);
+			if (job_cond->usage_end == 0)
+				exit(1);
 			break;
 		case 'f':
 			xfree(params.opt_filein);
@@ -750,7 +750,7 @@ void parse_command_line(int argc, char **argv)
 			break;
 		case 'j':
 			if ((strspn(optarg, "0123456789, ") < strlen(optarg))
-			    && (strspn(optarg, ".0123456789, ")
+			    && (strspn(optarg, ".batch0123456789, ")
 				< strlen(optarg))) {
 				fprintf(stderr, "Invalid jobs list: %s\n",
 					optarg);
@@ -759,21 +759,27 @@ void parse_command_line(int argc, char **argv)
 
 			if(!job_cond->step_list)
 				job_cond->step_list = list_create(
-					destroy_jobacct_selected_step);
+					slurmdb_destroy_selected_step);
 			_addto_step_list(job_cond->step_list, optarg);
 			break;
+		case 'k':
+			job_cond->timelimit_min = time_str2mins(optarg);
+			if (((int32_t)job_cond->timelimit_min <= 0)
+			    && (job_cond->timelimit_min != INFINITE))
+				fatal("Invalid time limit specification");
+			break;
+		case 'K':
+			job_cond->timelimit_max = time_str2mins(optarg);
+			if (((int32_t)job_cond->timelimit_max <= 0)
+			    && (job_cond->timelimit_max != INFINITE))
+				fatal("Invalid time limit specification");
+			break;
 		case 'L':
 			all_clusters = 1;
 			break;
 		case 'l':
 			long_output = true;
 			break;
-		case 'o':
-			xstrfmtcat(params.opt_field_list, "%s,", optarg);
-			break;
-		case 'O':
-			params.opt_fdump = 1;
-			break;
 		case 'n':
 			print_fields_have_header = 0;
 			break;
@@ -785,6 +791,12 @@ void parse_command_line(int argc, char **argv)
 			}
 			job_cond->used_nodes = xstrdup(optarg);
 			break;
+		case 'o':
+			xstrfmtcat(params.opt_field_list, "%s,", optarg);
+			break;
+		case 'O':
+			params.opt_fdump = 1;
+			break;
 		case 'p':
 			print_fields_parsable_print =
 				PRINT_FIELDS_PARSABLE_ENDING;
@@ -793,6 +805,19 @@ void parse_command_line(int argc, char **argv)
 			print_fields_parsable_print =
 				PRINT_FIELDS_PARSABLE_NO_ENDING;
 			break;
+		case 'q':
+			if(!g_qos_list)
+				g_qos_list = slurmdb_qos_get(
+					acct_db_conn, NULL);
+
+			if(!job_cond->qos_list)
+				job_cond->qos_list =
+					list_create(slurm_destroy_char);
+
+			if(!slurmdb_addto_qos_char_list(job_cond->qos_list,
+							g_qos_list, optarg, 0))
+				fatal("problem processing qos list");
+			break;
 		case 'r':
 			if(!job_cond->partition_list)
 				job_cond->partition_list =
@@ -810,6 +835,8 @@ void parse_command_line(int argc, char **argv)
 			break;
 		case 'S':
 			job_cond->usage_start = parse_time(optarg, 1);
+			if (job_cond->usage_start == 0)
+				exit(1);
 			break;
 		case 'T':
 			job_cond->without_usage_truncation = 0;
@@ -876,21 +903,19 @@ void parse_command_line(int argc, char **argv)
 	job_cond->without_steps = params.opt_allocs;
 
 	if(!job_cond->usage_start && !job_cond->step_list) {
+		struct tm start_tm;
 		job_cond->usage_start = time(NULL);
-		if(!job_cond->state_list) {
-			struct tm start_tm;
 
-			if(!localtime_r(&job_cond->usage_start, &start_tm)) {
-				error("Couldn't get localtime from %d",
-				      job_cond->usage_start);
-				return;
-			}
-			start_tm.tm_sec = 0;
-			start_tm.tm_min = 0;
-			start_tm.tm_hour = 0;
-			start_tm.tm_isdst = -1;
-			job_cond->usage_start = mktime(&start_tm);
+		if (!localtime_r(&job_cond->usage_start, &start_tm)) {
+			error("Couldn't get localtime from %ld",
+			      (long)job_cond->usage_start);
+			return;
 		}
+		start_tm.tm_sec = 0;
+		start_tm.tm_min = 0;
+		start_tm.tm_hour = 0;
+		start_tm.tm_isdst = -1;
+		job_cond->usage_start = mktime(&start_tm);
 	}
 
 	if(verbosity > 0) {
@@ -905,7 +930,7 @@ void parse_command_line(int argc, char **argv)
 			end_char[strlen(end_char)-1] = '\0';
 		} else
 			end_char = xstrdup("Now");
-		info("Jobs eligible from %s - %s\n", start_char, end_char);
+		info("Jobs eligible from %s - %s", start_char, end_char);
 		xfree(start_char);
 		xfree(end_char);
 	}
@@ -917,7 +942,7 @@ void parse_command_line(int argc, char **argv)
 	      "\topt_fdump=%d\n"
 	      "\topt_field_list=%s\n"
 	      "\topt_help=%d\n"
-	      "\topt_allocs=%d\n",
+	      "\topt_allocs=%d",
 	      params.opt_completion,
 	      params.opt_dump,
 	      params.opt_dup,
@@ -947,7 +972,7 @@ void parse_command_line(int argc, char **argv)
 			exit(1);
 		}
 		xfree(acct_type);
-		acct_db_conn = acct_storage_g_get_connection(false, 0, false);
+		acct_db_conn = slurmdb_connection_get();
 		if(errno != SLURM_SUCCESS) {
 			error("Problem talking to the database: %m");
 			exit(1);
@@ -961,13 +986,13 @@ void parse_command_line(int argc, char **argv)
 			list_destroy(job_cond->cluster_list);
 			job_cond->cluster_list = NULL;
 		}
-		debug2("Clusters requested:\tall\n");
+		debug2("Clusters requested:\tall");
 	} else if (job_cond->cluster_list
 		   && list_count(job_cond->cluster_list)) {
-		debug2( "Clusters requested:\n");
+		debug2( "Clusters requested:");
 		itr = list_iterator_create(job_cond->cluster_list);
 		while((start = list_next(itr)))
-			debug2("\t: %s\n", start);
+			debug2("\t: %s", start);
 		list_iterator_destroy(itr);
 	} else if(!job_cond->cluster_list
 		  || !list_count(job_cond->cluster_list)) {
@@ -996,7 +1021,7 @@ void parse_command_line(int argc, char **argv)
 			list_destroy(job_cond->userid_list);
 			job_cond->userid_list = NULL;
 		}
-		debug2("Userids requested:\tall\n");
+		debug2("Userids requested:\tall");
 	} else if (job_cond->userid_list && list_count(job_cond->userid_list)) {
 		debug2("Userids requested:");
 		itr = list_iterator_create(job_cond->userid_list);
@@ -1013,10 +1038,10 @@ void parse_command_line(int argc, char **argv)
 	}
 
 	if (job_cond->groupid_list && list_count(job_cond->groupid_list)) {
-		debug2("Groupids requested:\n");
+		debug2("Groupids requested:");
 		itr = list_iterator_create(job_cond->groupid_list);
 		while((start = list_next(itr)))
-			debug2("\t: %s\n", start);
+			debug2("\t: %s", start);
 		list_iterator_destroy(itr);
 	}
 
@@ -1025,10 +1050,17 @@ void parse_command_line(int argc, char **argv)
 		debug2("Partitions requested:");
 		itr = list_iterator_create(job_cond->partition_list);
 		while((start = list_next(itr)))
-			debug2("\t: %s\n", start);
+			debug2("\t: %s", start);
 		list_iterator_destroy(itr);
 	}
 
+	/* specific qos' requested? */
+	if (job_cond->qos_list && list_count(job_cond->qos_list)) {
+		start = get_qos_complete_str(g_qos_list, job_cond->qos_list);
+		debug2("QOS requested\t: %s\n", start);
+		xfree(start);
+	}
+
 	/* specific jobs requested? */
 	if (job_cond->step_list && list_count(job_cond->step_list)) {
 		debug2("Jobs requested:");
@@ -1064,6 +1096,19 @@ void parse_command_line(int argc, char **argv)
 		list_iterator_destroy(itr);
 	}
 
+	if (job_cond->timelimit_min) {
+		char time_str[128], tmp1[32], tmp2[32];
+		mins2time_str(job_cond->timelimit_min, tmp1, sizeof(tmp1));
+		sprintf(time_str, "%s", tmp1);
+		if(job_cond->timelimit_max) {
+			int len = strlen(tmp1);
+			mins2time_str(job_cond->timelimit_max,
+				      tmp2, sizeof(tmp2));
+			sprintf(time_str+len, " - %s", tmp2);
+		}
+		debug2("Timelimit requested\t: %s", time_str);
+	}
+
 	/* select the output fields */
 	if(brief_output) {
 		if(params.opt_completion)
@@ -1155,20 +1200,20 @@ void do_dump(void)
 {
 	ListIterator itr = NULL;
 	ListIterator itr_step = NULL;
-	jobacct_job_rec_t *job = NULL;
-	jobacct_step_rec_t *step = NULL;
+	slurmdb_job_rec_t *job = NULL;
+	slurmdb_step_rec_t *step = NULL;
 	struct tm ts;
 
 	itr = list_iterator_create(jobs);
 	while((job = list_next(itr))) {
-		if(job->sacct.min_cpu == NO_VAL)
-			job->sacct.min_cpu = 0;
+		if(job->stats.cpu_min == NO_VAL)
+			job->stats.cpu_min = 0;
 
 		if(list_count(job->steps)) {
-			job->sacct.ave_cpu /= list_count(job->steps);
-			job->sacct.ave_rss /= list_count(job->steps);
-			job->sacct.ave_vsize /= list_count(job->steps);
-			job->sacct.ave_pages /= list_count(job->steps);
+			job->stats.cpu_ave /= list_count(job->steps);
+			job->stats.rss_ave /= list_count(job->steps);
+			job->stats.vsize_ave /= list_count(job->steps);
+			job->stats.pages_ave /= list_count(job->steps);
 		}
 
 		/* JOB_START */
@@ -1234,25 +1279,25 @@ void do_dump(void)
 			       (int)step->user_cpu_usec,
 			       (int)step->sys_cpu_sec,
 			       (int)step->sys_cpu_usec,
-			       step->sacct.max_vsize/1024,
-			       step->sacct.max_rss/1024);
+			       step->stats.vsize_max/1024,
+			       step->stats.rss_max/1024);
 			/* Data added in Slurm v1.1 */
 			printf("%u %u %.2f %u %u %.2f %d %u %u %.2f "
 			       "%.u %u %u %.2f %s %s %s\n",
-			       step->sacct.max_vsize_id.nodeid,
-			       step->sacct.max_vsize_id.taskid,
-			       step->sacct.ave_vsize/1024,
-			       step->sacct.max_rss_id.nodeid,
-			       step->sacct.max_rss_id.taskid,
-			       step->sacct.ave_rss/1024,
-			       step->sacct.max_pages,
-			       step->sacct.max_pages_id.nodeid,
-			       step->sacct.max_pages_id.taskid,
-			       step->sacct.ave_pages,
-			       step->sacct.min_cpu,
-			       step->sacct.min_cpu_id.nodeid,
-			       step->sacct.min_cpu_id.taskid,
-			       step->sacct.ave_cpu,
+			       step->stats.vsize_max_nodeid,
+			       step->stats.vsize_max_taskid,
+			       step->stats.vsize_ave/1024,
+			       step->stats.rss_max_nodeid,
+			       step->stats.rss_max_taskid,
+			       step->stats.rss_ave/1024,
+			       step->stats.pages_max,
+			       step->stats.pages_max_nodeid,
+			       step->stats.pages_max_taskid,
+			       step->stats.pages_ave,
+			       step->stats.cpu_min,
+			       step->stats.cpu_min_nodeid,
+			       step->stats.cpu_min_taskid,
+			       step->stats.cpu_ave,
 			       step->stepname,
 			       step->nodes,
 			       job->account);
@@ -1292,25 +1337,25 @@ void do_dump(void)
 			       (int)job->user_cpu_usec,
 			       (int)job->sys_cpu_sec,
 			       (int)job->sys_cpu_usec,
-			       job->sacct.max_vsize/1024,
-			       job->sacct.max_rss/1024);
+			       job->stats.vsize_max/1024,
+			       job->stats.rss_max/1024);
 			/* Data added in Slurm v1.1 */
 			printf("%u %u %.2f %u %u %.2f %d %u %u %.2f "
 			       "%.u %u %u %.2f %s %s %s %d\n",
-			       job->sacct.max_vsize_id.nodeid,
-			       job->sacct.max_vsize_id.taskid,
-			       job->sacct.ave_vsize/1024,
-			       job->sacct.max_rss_id.nodeid,
-			       job->sacct.max_rss_id.taskid,
-			       job->sacct.ave_rss/1024,
-			       job->sacct.max_pages,
-			       job->sacct.max_pages_id.nodeid,
-			       job->sacct.max_pages_id.taskid,
-			       job->sacct.ave_pages,
-			       job->sacct.min_cpu,
-			       job->sacct.min_cpu_id.nodeid,
-			       job->sacct.min_cpu_id.taskid,
-			       job->sacct.ave_cpu,
+			       job->stats.vsize_max_nodeid,
+			       job->stats.vsize_max_taskid,
+			       job->stats.vsize_ave/1024,
+			       job->stats.rss_max_nodeid,
+			       job->stats.rss_max_taskid,
+			       job->stats.rss_ave/1024,
+			       job->stats.pages_max,
+			       job->stats.pages_max_nodeid,
+			       job->stats.pages_max_taskid,
+			       job->stats.pages_ave,
+			       job->stats.cpu_min,
+			       job->stats.cpu_min_nodeid,
+			       job->stats.cpu_min_taskid,
+			       job->stats.cpu_ave,
 			       "-",
 			       job->nodes,
 			       job->account,
@@ -1356,7 +1401,7 @@ void do_help(void)
 		_usage();
 		break;
 	default:
-		debug2("sacct bug: params.opt_help=%d\n",
+		debug2("sacct bug: params.opt_help=%d",
 			params.opt_help);
 	}
 }
@@ -1373,23 +1418,23 @@ void do_list(void)
 {
 	ListIterator itr = NULL;
 	ListIterator itr_step = NULL;
-	jobacct_job_rec_t *job = NULL;
-	jobacct_step_rec_t *step = NULL;
+	slurmdb_job_rec_t *job = NULL;
+	slurmdb_step_rec_t *step = NULL;
 
 	if(!jobs)
 		return;
 
 	itr = list_iterator_create(jobs);
 	while((job = list_next(itr))) {
-		if(job->sacct.min_cpu == NO_VAL)
-			job->sacct.min_cpu = 0;
+		if(job->stats.cpu_min == NO_VAL)
+			job->stats.cpu_min = 0;
 
 		if(list_count(job->steps)) {
 			int cnt = list_count(job->steps);
-			job->sacct.ave_cpu /= (double)cnt;
-			job->sacct.ave_rss /= (double)cnt;
-			job->sacct.ave_vsize /= (double)cnt;
-			job->sacct.ave_pages /= (double)cnt;
+			job->stats.cpu_ave /= (double)cnt;
+			job->stats.rss_ave /= (double)cnt;
+			job->stats.vsize_ave /= (double)cnt;
+			job->stats.pages_ave /= (double)cnt;
 		}
 
 		if (job->show_full)
@@ -1447,16 +1492,16 @@ void sacct_fini()
 		list_destroy(print_fields_list);
 	if(jobs)
 		list_destroy(jobs);
-	if(qos_list)
-		list_destroy(qos_list);
+	if(g_qos_list)
+		list_destroy(g_qos_list);
 
 	if(params.opt_completion)
 		g_slurm_jobcomp_fini();
 	else {
-		acct_storage_g_close_connection(&acct_db_conn);
+		slurmdb_connection_close(&acct_db_conn);
 		slurm_acct_storage_fini();
 	}
 	xfree(params.opt_field_list);
 	xfree(params.opt_filein);
-	destroy_acct_job_cond(params.job_cond);
+	slurmdb_destroy_job_cond(params.job_cond);
 }
