@@ -222,6 +222,7 @@ int main(int argc, char *argv[])
 	 * a) input is from a terminal (stdin has valid termios attributes),
 	 * b) controlling terminal exists (non-negative tpgid),
 	 * c) salloc is not run in allocation-only (--no-shell) mode,
+	 * NOTE: d and e below are configuration dependent
 	 * d) salloc runs in its own process group (true in interactive
 	 *    shells that support job control),
 	 * e) salloc has been configured at compile-time to support background
@@ -237,10 +238,10 @@ int main(int argc, char *argv[])
 			error("no controlling terminal: please set --no-shell");
 			exit(error_exit);
 		}
+#ifdef SALLOC_RUN_FOREGROUND
 	} else if ((!opt.no_shell) && (pid == getpgrp())) {
 		if (tpgid == pid)
 			is_interactive = true;
-#ifdef SALLOC_RUN_FOREGROUND
 		while (tcgetpgrp(STDIN_FILENO) != pid) {
 			if (!is_interactive) {
 				error("Waiting for program to be placed in "
@@ -249,8 +250,12 @@ int main(int argc, char *argv[])
 			}
 			killpg(pid, SIGTTIN);
 		}
-#endif
 	}
+#else
+	} else if (!opt.no_shell) {
+		is_interactive = true;
+	}
+#endif
 	/*
 	 * Reset saved tty attributes at exit, in case a child
 	 * process died before properly resetting terminal.
@@ -487,6 +492,9 @@ relinquish:
 
 		if (WIFEXITED(status)) {
 			rc = WEXITSTATUS(status);
+		} else if (WIFSTOPPED(status)) {
+			/* Terminate stopped child process */
+			_forward_signal(SIGKILL);
 		} else if (WIFSIGNALED(status)) {
 			verbose("Command \"%s\" was terminated by signal %d",
 				command_argv[0], WTERMSIG(status));
