@@ -2,7 +2,8 @@
  *  sinfo.c - Report overall state the system
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
+ *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
+ *  Portions Copyright (C) 2010 SchedMD <http://www.schedmd.com>.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Joey Ekstrom <ekstrom1@llnl.gov>, Morris Jette <jette1@llnl.gov>
  *  CODE-OCEC-09-009. All rights reserved.
@@ -47,10 +48,8 @@
 #include "src/sinfo/sinfo.h"
 #include "src/sinfo/print.h"
 
-#ifdef HAVE_BG
-# include "src/plugins/select/bluegene/wrap_rm_api.h"
-# include "src/plugins/select/bluegene/plugin/bluegene.h"
-#endif
+#include "src/plugins/select/bluegene/wrap_rm_api.h"
+#include "src/plugins/select/bluegene/plugin/bluegene.h"
 
 /********************
  * Global Variables *
@@ -70,13 +69,16 @@ static sinfo_data_t *_create_sinfo(partition_info_t* part_ptr,
 				   uint16_t part_inx, node_info_t *node_ptr,
 				   uint32_t node_scaling);
 static bool _filter_out(node_info_t *node_ptr);
+static int  _get_info(bool clear_old);
 static void _sinfo_list_delete(void *data);
-static bool _match_node_data(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr);
+static bool _match_node_data(sinfo_data_t *sinfo_ptr,
+                             node_info_t *node_ptr);
 static bool _match_part_data(sinfo_data_t *sinfo_ptr,
                              partition_info_t* part_ptr);
+static int  _multi_cluster(List clusters);
 static int  _query_server(partition_info_msg_t ** part_pptr,
 			  node_info_msg_t ** node_pptr,
-			  block_info_msg_t ** block_pptr);
+			  block_info_msg_t ** block_pptr, bool clear_old);
 static void _sort_hostlist(List sinfo_list);
 static int  _strcmp(char *data1, char *data2);
 static void _update_sinfo(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr,
@@ -92,10 +94,6 @@ static int _handle_subgrps(List sinfo_list, uint16_t part_num,
 int main(int argc, char *argv[])
 {
 	log_options_t opts = LOG_OPTS_STDERR_ONLY;
-	partition_info_msg_t *partition_msg = NULL;
-	node_info_msg_t *node_msg = NULL;
-	block_info_msg_t *block_msg = NULL;
-	List sinfo_list = NULL;
 	int rc = 0;
 
 	log_init(xbasename(argv[0]), opts, SYSLOG_FACILITY_USER, NULL);
@@ -110,22 +108,12 @@ int main(int argc, char *argv[])
 		    (params.iterate || params.verbose || params.long_output))
 			print_date();
 
-		if (_query_server(&partition_msg, &node_msg, &block_msg)
-		    != 0)
+		if (!params.clusters) {
+			if (_get_info(false))
+				rc = 1;
+		} else if (_multi_cluster(params.clusters) != 0)
 			rc = 1;
-		else if (params.bg_flag)
-			(void) _bg_report(block_msg);
-		else {
-			sinfo_list = list_create(_sinfo_list_delete);
-			_build_sinfo_data(sinfo_list, partition_msg, node_msg);
-	 		sort_sinfo_list(sinfo_list);
-			print_sinfo_list(sinfo_list);
-		}
 		if (params.iterate) {
-			if (sinfo_list) {
-				list_destroy(sinfo_list);
-				sinfo_list = NULL;
-			}
 			printf("\n");
 			sleep(params.iterate);
 		} else
@@ -135,58 +123,50 @@ int main(int argc, char *argv[])
 	exit(rc);
 }
 
-static char *_conn_type_str(int conn_type)
+static int _multi_cluster(List clusters)
 {
-	switch (conn_type) {
-		case (SELECT_MESH):
-			return "MESH";
-		case (SELECT_TORUS):
-			return "TORUS";
-		case (SELECT_SMALL):
-			return "SMALL";
+	ListIterator itr;
+	bool first = true;
+	int rc = 0, rc2;
+
+	itr = list_iterator_create(clusters);
+	while ((working_cluster_rec = list_next(itr))) {
+		if (first)
+			first = false;
+		else
+			printf("\n");
+		printf("CLUSTER: %s\n", working_cluster_rec->name);
+		rc2 = _get_info(true);
+		rc = MAX(rc, rc2);
 	}
-	return "?";
+	list_iterator_destroy(itr);
+
+	return rc;
 }
 
-static char *_node_use_str(int node_use)
+/* clear_old IN - if set then don't preserve old info (it might be from
+ *		  another cluster) */
+static int _get_info(bool clear_old)
 {
-	switch (node_use) {
-		case (SELECT_COPROCESSOR_MODE):
-			return "COPROCESSOR";
-		case (SELECT_VIRTUAL_NODE_MODE):
-			return "VIRTUAL";
+	partition_info_msg_t *partition_msg = NULL;
+	node_info_msg_t *node_msg = NULL;
+	block_info_msg_t *block_msg = NULL;
+	List sinfo_list = NULL;
+	int rc = 0;
+
+	if (_query_server(&partition_msg, &node_msg, &block_msg, clear_old))
+		rc = 1;
+	else if (params.bg_flag)
+		(void) _bg_report(block_msg);
+	else {
+		sinfo_list = list_create(_sinfo_list_delete);
+		_build_sinfo_data(sinfo_list, partition_msg, node_msg);
+		sort_sinfo_list(sinfo_list);
+		print_sinfo_list(sinfo_list);
+		FREE_NULL_LIST(sinfo_list);
 	}
-	return "?";
-}
 
-static char *_part_state_str(int state)
-{
-	static char tmp[16];
-
-#ifdef HAVE_BG
-	switch (state) {
-#ifdef HAVE_BGL
-		case RM_PARTITION_BUSY:
-			return "BUSY";
-#else
-		case RM_PARTITION_REBOOTING:
-			return "REBOOTING";
-#endif
-		case RM_PARTITION_CONFIGURING:
-			return "CONFIG";
-		case RM_PARTITION_DEALLOCATING:
-			return "DEALLOC";
-		case RM_PARTITION_ERROR:
-			return "ERROR";
-		case RM_PARTITION_FREE:
-			return "FREE";
-		case RM_PARTITION_READY:
-			return "READY";
-	}
-#endif
-
-	snprintf(tmp, sizeof(tmp), "%d", state);
-	return tmp;
+	return rc;
 }
 
 /*
@@ -211,11 +191,11 @@ static int _bg_report(block_info_msg_t *block_ptr)
 		       block_ptr->block_array[i].bg_block_id,
 		       block_ptr->block_array[i].nodes,
 		       block_ptr->block_array[i].owner_name,
-		       _part_state_str(
+		       bg_block_state_string(
 			       block_ptr->block_array[i].state),
-		       _conn_type_str(
+		       conn_type_string(
 			       block_ptr->block_array[i].conn_type),
-		       _node_use_str(
+		       node_use_string(
 			       block_ptr->block_array[i].node_use));
 	}
 
@@ -226,18 +206,20 @@ static int _bg_report(block_info_msg_t *block_ptr)
  * _query_server - download the current server state
  * part_pptr IN/OUT - partition information message
  * node_pptr IN/OUT - node information message
+ * block_pptr IN/OUT - BlueGene block data
+ * clear_old IN - If set, then always replace old data, needed when going 
+ *		  between clusters.
  * RET zero or error code
  */
 static int
 _query_server(partition_info_msg_t ** part_pptr,
 	      node_info_msg_t ** node_pptr,
-	      block_info_msg_t ** block_pptr)
+	      block_info_msg_t ** block_pptr, bool clear_old)
 {
 	static partition_info_msg_t *old_part_ptr = NULL, *new_part_ptr;
 	static node_info_msg_t *old_node_ptr = NULL, *new_node_ptr;
-#ifdef HAVE_BG
 	static block_info_msg_t *old_bg_ptr = NULL, *new_bg_ptr;
-#endif
+
 	int error_code;
 	uint16_t show_flags = 0;
 
@@ -245,19 +227,20 @@ _query_server(partition_info_msg_t ** part_pptr,
 		show_flags |= SHOW_ALL;
 
 	if (old_part_ptr) {
-		error_code =
-		    slurm_load_partitions(old_part_ptr->last_update,
-					  &new_part_ptr, show_flags);
+		if (clear_old)
+			old_part_ptr->last_update = 0;
+		error_code = slurm_load_partitions(old_part_ptr->last_update,
+						   &new_part_ptr, show_flags);
 		if (error_code == SLURM_SUCCESS)
 			slurm_free_partition_info_msg(old_part_ptr);
 		else if (slurm_get_errno() == SLURM_NO_CHANGE_IN_DATA) {
 			error_code = SLURM_SUCCESS;
 			new_part_ptr = old_part_ptr;
 		}
-	} else
-		error_code =
-		    slurm_load_partitions((time_t) NULL, &new_part_ptr,
-					  show_flags);
+	} else {
+		error_code = slurm_load_partitions((time_t) NULL, &new_part_ptr,
+						   show_flags);
+	}
 	if (error_code) {
 		slurm_perror("slurm_load_partitions");
 		return error_code;
@@ -267,18 +250,20 @@ _query_server(partition_info_msg_t ** part_pptr,
 	*part_pptr = new_part_ptr;
 
 	if (old_node_ptr) {
-		error_code =
-		    slurm_load_node(old_node_ptr->last_update,
-				    &new_node_ptr, show_flags);
+		if (clear_old)
+			old_node_ptr->last_update = 0;
+		error_code = slurm_load_node(old_node_ptr->last_update,
+					     &new_node_ptr, show_flags);
 		if (error_code == SLURM_SUCCESS)
 			slurm_free_node_info_msg(old_node_ptr);
 		else if (slurm_get_errno() == SLURM_NO_CHANGE_IN_DATA) {
 			error_code = SLURM_SUCCESS;
 			new_node_ptr = old_node_ptr;
 		}
-	} else
+	} else {
 		error_code = slurm_load_node((time_t) NULL, &new_node_ptr,
-				show_flags);
+					     show_flags);
+	}
 
 	if (error_code) {
 		slurm_perror("slurm_load_node");
@@ -287,27 +272,35 @@ _query_server(partition_info_msg_t ** part_pptr,
 	old_node_ptr = new_node_ptr;
 	*node_pptr = new_node_ptr;
 
-#ifdef HAVE_BG
-	if (old_bg_ptr) {
-		error_code = slurm_load_block_info(old_bg_ptr->last_update,
-						   &new_bg_ptr);
-		if (error_code == SLURM_SUCCESS)
-			slurm_free_block_info_msg(&old_bg_ptr);
-		else if (slurm_get_errno() == SLURM_NO_CHANGE_IN_DATA) {
-			error_code = SLURM_SUCCESS;
-			new_bg_ptr = old_bg_ptr;
+	if (!params.bg_flag)
+		return SLURM_SUCCESS;
+
+	if (params.cluster_flags & CLUSTER_FLAG_BG) {
+		if (old_bg_ptr) {
+			if (clear_old)
+				old_bg_ptr->last_update = 0;
+			error_code = slurm_load_block_info(
+				old_bg_ptr->last_update,
+				&new_bg_ptr, show_flags);
+			if (error_code == SLURM_SUCCESS)
+				slurm_free_block_info_msg(old_bg_ptr);
+			else if (slurm_get_errno() == SLURM_NO_CHANGE_IN_DATA) {
+				error_code = SLURM_SUCCESS;
+				new_bg_ptr = old_bg_ptr;
+			}
+		} else {
+			error_code = slurm_load_block_info((time_t) NULL,
+							   &new_bg_ptr,
+							   show_flags);
 		}
-	} else {
-		error_code = slurm_load_block_info((time_t) NULL,
-						   &new_bg_ptr);
 	}
+
 	if (error_code) {
 		slurm_perror("slurm_load_block");
 		return error_code;
 	}
 	old_bg_ptr = new_bg_ptr;
 	*block_pptr = new_bg_ptr;
-#endif
 	return SLURM_SUCCESS;
 }
 
@@ -514,6 +507,11 @@ static bool _match_node_data(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr)
 	    (_strcmp(node_ptr->features, sinfo_ptr->features)))
 		return false;
 
+	if (sinfo_ptr->nodes &&
+	    params.match_flags.gres_flag &&
+	    (_strcmp(node_ptr->gres, sinfo_ptr->gres)))
+		return false;
+
 	if (sinfo_ptr->nodes &&
 	    params.match_flags.reason_flag &&
 	    (_strcmp(node_ptr->reason, sinfo_ptr->reason)))
@@ -535,6 +533,7 @@ static bool _match_node_data(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr)
 	if (params.match_flags.cpus_flag &&
 	    ((node_ptr->cpus / g_node_scaling) != sinfo_ptr->min_cpus))
 		return false;
+
 	if (params.match_flags.sockets_flag &&
 	    (node_ptr->sockets     != sinfo_ptr->min_sockets))
 		return false;
@@ -600,13 +599,18 @@ static bool _match_part_data(sinfo_data_t *sinfo_ptr,
 		return false;
 
 	if (params.match_flags.root_flag &&
-	    (part_ptr->root_only != sinfo_ptr->part_info->root_only))
+	    ((part_ptr->flags & PART_FLAG_ROOT_ONLY) !=
+	     (sinfo_ptr->part_info->flags & PART_FLAG_ROOT_ONLY)))
 		return false;
 
 	if (params.match_flags.share_flag &&
 	    (part_ptr->max_share != sinfo_ptr->part_info->max_share))
 		return false;
 
+	if (params.match_flags.preempt_mode_flag &&
+	    (part_ptr->preempt_mode != sinfo_ptr->part_info->preempt_mode))
+		return false;
+
 	if (params.match_flags.priority_flag &&
 	    (part_ptr->priority != sinfo_ptr->part_info->priority))
 		return false;
@@ -629,7 +633,10 @@ static void _update_sinfo(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr,
 	if (sinfo_ptr->nodes_total == 0) {	/* first node added */
 		sinfo_ptr->node_state = node_ptr->node_state;
 		sinfo_ptr->features   = node_ptr->features;
+		sinfo_ptr->gres       = node_ptr->gres;
 		sinfo_ptr->reason     = node_ptr->reason;
+		sinfo_ptr->reason_time= node_ptr->reason_time;
+		sinfo_ptr->reason_uid = node_ptr->reason_uid;
 		sinfo_ptr->min_cpus   = single_node_cpus;
 		sinfo_ptr->max_cpus   = single_node_cpus;
 		sinfo_ptr->min_sockets = node_ptr->sockets;
@@ -698,53 +705,55 @@ static void _update_sinfo(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr,
 				     SELECT_NODEDATA_SUBCNT,
 				     NODE_STATE_ERROR,
 				     &error_cpus);
-#ifdef HAVE_BG
 
-	if(!params.match_flags.state_flag && (used_cpus || error_cpus)) {
-		/* We only get one shot at this (because all states
-		   are combined together), so we need to make
-		   sure we get all the subgrps accounted. (So use
-		   g_node_scaling for safe measure) */
-		total_nodes = g_node_scaling;
+	if (params.cluster_flags & CLUSTER_FLAG_BG) {
+		if (!params.match_flags.state_flag &&
+		    (used_cpus || error_cpus)) {
+			/* We only get one shot at this (because all states
+			   are combined together), so we need to make
+			   sure we get all the subgrps accounted. (So use
+			   g_node_scaling for safe measure) */
+			total_nodes = g_node_scaling;
 
-		sinfo_ptr->nodes_alloc += used_cpus;
-		sinfo_ptr->nodes_other += error_cpus;
-		sinfo_ptr->nodes_idle +=
-			(total_nodes - (used_cpus + error_cpus));
-		used_cpus *= single_node_cpus;
-		error_cpus *= single_node_cpus;
-	} else {
-		/* process only for this subgrp and then return */
-		total_cpus = total_nodes * single_node_cpus;
-
-		if ((base_state == NODE_STATE_ALLOCATED)
-		    ||  (node_ptr->node_state & NODE_STATE_COMPLETING)) {
-			sinfo_ptr->nodes_alloc += total_nodes;
-			sinfo_ptr->cpus_alloc += total_cpus;
-		} else if (IS_NODE_DRAIN(node_ptr) ||
-			   (base_state == NODE_STATE_DOWN)) {
-			sinfo_ptr->nodes_other += total_nodes;
-			sinfo_ptr->cpus_other += total_cpus;
+			sinfo_ptr->nodes_alloc += used_cpus;
+			sinfo_ptr->nodes_other += error_cpus;
+			sinfo_ptr->nodes_idle +=
+				(total_nodes - (used_cpus + error_cpus));
+			used_cpus *= single_node_cpus;
+			error_cpus *= single_node_cpus;
 		} else {
-			sinfo_ptr->nodes_idle += total_nodes;
-			sinfo_ptr->cpus_idle += total_cpus;
+			/* process only for this subgrp and then return */
+			total_cpus = total_nodes * single_node_cpus;
+
+			if ((base_state == NODE_STATE_ALLOCATED) ||
+			    (node_ptr->node_state & NODE_STATE_COMPLETING)) {
+				sinfo_ptr->nodes_alloc += total_nodes;
+				sinfo_ptr->cpus_alloc += total_cpus;
+			} else if (IS_NODE_DRAIN(node_ptr) ||
+				   (base_state == NODE_STATE_DOWN)) {
+				sinfo_ptr->nodes_other += total_nodes;
+				sinfo_ptr->cpus_other += total_cpus;
+			} else {
+				sinfo_ptr->nodes_idle += total_nodes;
+				sinfo_ptr->cpus_idle += total_cpus;
+			}
+
+			sinfo_ptr->nodes_total += total_nodes;
+			sinfo_ptr->cpus_total += total_cpus;
+
+			return;
 		}
-
-		sinfo_ptr->nodes_total += total_nodes;
-		sinfo_ptr->cpus_total += total_cpus;
-
-		return;
+	} else {
+		if ((base_state == NODE_STATE_ALLOCATED) ||
+		    IS_NODE_COMPLETING(node_ptr))
+			sinfo_ptr->nodes_alloc += total_nodes;
+		else if (IS_NODE_DRAIN(node_ptr)
+			 || (base_state == NODE_STATE_DOWN))
+			sinfo_ptr->nodes_other += total_nodes;
+		else
+			sinfo_ptr->nodes_idle += total_nodes;
 	}
-#else
-	if ((base_state == NODE_STATE_ALLOCATED) ||
-	    IS_NODE_COMPLETING(node_ptr))
-		sinfo_ptr->nodes_alloc += total_nodes;
-	 else if (IS_NODE_DRAIN(node_ptr) ||
-		  (base_state == NODE_STATE_DOWN))
-		 sinfo_ptr->nodes_other += total_nodes;
-	 else
-		sinfo_ptr->nodes_idle += total_nodes;
-#endif
+
 	sinfo_ptr->nodes_total += total_nodes;
 
 
@@ -760,11 +769,6 @@ static void _update_sinfo(sinfo_data_t *sinfo_ptr, node_info_t *node_ptr,
 		sinfo_ptr->cpus_other += total_cpus;
 	} else
 		sinfo_ptr->cpus_idle += total_cpus;
-
-	/* info("count is now %d %d %d %d %d", */
-	/*      sinfo_ptr->cpus_alloc, sinfo_ptr->cpus_idle, */
-	/*      sinfo_ptr->cpus_other, sinfo_ptr->cpus_total, */
-	/*      sinfo_ptr->nodes_total); */
 }
 
 static int _insert_node_ptr(List sinfo_list, uint16_t part_num,
@@ -775,15 +779,16 @@ static int _insert_node_ptr(List sinfo_list, uint16_t part_num,
 	sinfo_data_t *sinfo_ptr = NULL;
 	ListIterator itr = NULL;
 
-#ifdef HAVE_BG
-	uint16_t error_cpus = 0;
-	select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-				     SELECT_NODEDATA_SUBCNT,
-				     NODE_STATE_ERROR,
-				     &error_cpus);
-	if(error_cpus && !node_ptr->reason)
-		node_ptr->reason = xstrdup("Block(s) in error state");
-#endif
+	if (params.cluster_flags & CLUSTER_FLAG_BG) {
+		uint16_t error_cpus = 0;
+		select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
+					     SELECT_NODEDATA_SUBCNT,
+					     NODE_STATE_ERROR,
+					     &error_cpus);
+
+		if (error_cpus && !node_ptr->reason)
+			node_ptr->reason = xstrdup("Block(s) in error state");
+	}
 
 	itr = list_iterator_create(sinfo_list);
 	while ((sinfo_ptr = list_next(itr))) {
@@ -802,6 +807,7 @@ static int _insert_node_ptr(List sinfo_list, uint16_t part_num,
 		list_append(sinfo_list,
 			    _create_sinfo(part_ptr, part_num,
 					  node_ptr, node_scaling));
+
 	return rc;
 }
 
@@ -809,31 +815,18 @@ static int _handle_subgrps(List sinfo_list, uint16_t part_num,
 			   partition_info_t *part_ptr,
 			   node_info_t *node_ptr, uint32_t node_scaling)
 {
-/* 	char *tmp_char = NULL, *orig_name = NULL; */
-/* 	bitstr_t *tmp_bitmap = NULL; */
 	uint16_t size;
 	int *node_state;
-/* 	bitstr_t *bitmap = NULL; */
 	int i=0, state_cnt = 2;
 	ListIterator iterator = NULL;
-/* 	bool set = 0; */
 	enum node_states state[] =
 		{ NODE_STATE_ALLOCATED, NODE_STATE_ERROR };
-/* 	if(select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,  */
-/* 					SELECT_NODEDATA_BITMAP_SIZE, */
-/* 					0, */
-/* 					&size) != SLURM_SUCCESS) */
-/* 		return SLURM_ERROR; */
-
-/* 	bitmap = bit_alloc(size); */
 
 	/* If we ever update the hostlist stuff to support this stuff
 	 * then we can use this to tack on the end of the node name
 	 * the subgrp stuff.  On bluegene systems this would be nice
 	 * to see the ionodes in certain states.
 	 */
-/* 	orig_name = node_ptr->name; */
-/* 	node_ptr->name = NULL; */
 	if (params.state_list)
 		iterator = list_iterator_create(params.state_list);
 
@@ -857,49 +850,13 @@ static int _handle_subgrps(List sinfo_list, uint16_t part_num,
 						state[i],
 						&size) == SLURM_SUCCESS
 		   && size) {
-/* 			if(select_g_select_nodeinfo_get( */
-/* 				   node_ptr->select_nodeinfo,  */
-/* 				   SELECT_NODEDATA_BITMAP, */
-/* 				   state[i], */
-/* 				   &tmp_bitmap) != SLURM_SUCCESS) */
-/* 				continue; */
-
-/* 			if(select_g_select_nodeinfo_get( */
-/* 				   node_ptr->select_nodeinfo,  */
-/* 				   SELECT_NODEDATA_STR, */
-/* 				   state[i], */
-/* 				   &tmp_char) != SLURM_SUCCESS) { */
-/* 				FREE_NULL_BITMAP(tmp_bitmap); */
-/* 				continue; */
-/* 			} */
-/* 			bit_or(bitmap, tmp_bitmap); */
-/* 			FREE_NULL_BITMAP(tmp_bitmap); */
-/* 			xfree(node_ptr->name); */
-/* 			node_ptr->name = xstrdup_printf("%s[%s]", */
-/* 						  orig_name, tmp_char); */
-/* 			xfree(tmp_char); */
 			node_scaling -= size;
 			node_ptr->node_state &= NODE_STATE_FLAGS;
 			node_ptr->node_state |= state[i];
-			/* info("1 %s got %s of %u", node_ptr->name, */
-			/*      node_state_string(node_ptr->node_state), size); */
 			_insert_node_ptr(sinfo_list, part_num, part_ptr,
 					 node_ptr, size);
-/* 			set = 1; */
 		}
 	}
-/* 	xfree(node_ptr->name); */
-/* 	if(set) { */
-/* 		char bitstring[BITSIZE]; */
-/* 		bit_not(bitmap); */
-/* 		bit_fmt(bitstring, BITSIZE, bitmap); */
-
-/* 		node_ptr->name = xstrdup_printf("%s[%s]", */
-/* 						orig_name, bitstring); */
-/* 	} else { */
-/* 		xfree(node_ptr->name); */
-/* 		node_ptr->name = orig_name; */
-/* 	} */
 
 	/* now handle the idle */
 	if(iterator) {
@@ -917,17 +874,10 @@ static int _handle_subgrps(List sinfo_list, uint16_t part_num,
 	}
 	node_ptr->node_state &= NODE_STATE_FLAGS;
 	node_ptr->node_state |= NODE_STATE_IDLE;
-	/* info("2 %s got %s of %u", node_ptr->name, */
-	/*      node_state_string(node_ptr->node_state), node_scaling); */
 	if((int)node_scaling > 0)
 		_insert_node_ptr(sinfo_list, part_num, part_ptr,
 				 node_ptr, node_scaling);
 
-/* 	FREE_NULL_BITMAP(bitmap); */
-/* 	if(set) { */
-/* 		xfree(node_ptr->name); */
-/* 		node_ptr->name = orig_name; */
-/* 	} */
 	return SLURM_SUCCESS;
 }
 
