@@ -1273,6 +1273,7 @@ static void *_slurmctld_background(void *no_data)
 	static time_t last_trigger;
 	static time_t last_node_acct;
 	static time_t last_ctld_bu_ping;
+	static time_t last_uid_update;
 	static bool ping_msg_sent = false;
 	time_t now;
 	int no_resp_msg_interval, ping_interval, purge_job_interval;
@@ -1308,6 +1309,7 @@ static void *_slurmctld_background(void *no_data)
 	last_purge_job_time = last_trigger = last_health_check_time = now;
 	last_timelimit_time = last_assert_primary_time = now;
 	last_no_resp_msg_time = last_resv_time = last_ctld_bu_ping = now;
+	last_uid_update = now;
 
 	if ((slurmctld_conf.min_job_age > 0) &&
 	    (slurmctld_conf.min_job_age < PURGE_JOB_INTERVAL)) {
@@ -1521,6 +1523,17 @@ static void *_slurmctld_background(void *no_data)
 			(void) _shutdown_backup_controller(0);
 		}
 		unlock_slurmctld(config_read_lock);
+
+		if (difftime(now, last_uid_update) >= 3600) {
+			/* Make sure we update the uids in the
+			 * assoc_mgr if there were any users
+			 * with unknown uids at the time of startup.
+			 */
+			now = time(NULL);
+			last_uid_update = now;
+			assoc_mgr_set_missing_uids();
+		}
+
 		END_TIMER2("_slurmctld_background");
 	}
 
