@@ -1304,12 +1304,13 @@ extern void pack_all_node (char **buffer_ptr, int *buffer_size,
  * IN uid - user issuing request
  * IN show_flags - job step filtering options
  * OUT buffer - location to store data, pointers automatically advanced
+ * IN protocol_version - slurm protocol version of client
  * RET - 0 or error code
  * NOTE: MUST free_buf buffer
  */
-extern int pack_ctld_job_step_info_response_msg(uint32_t job_id,
-			uint32_t step_id, uid_t uid,
-			uint16_t show_flags, Buf buffer);
+extern int pack_ctld_job_step_info_response_msg(
+	uint32_t job_id, uint32_t step_id, uid_t uid,
+	uint16_t show_flags, Buf buffer, uint16_t protocol_version);
 
 /*
  * pack_all_part - dump all partition information for all partitions in
