@@ -54,9 +54,12 @@
 
 #include "src/common/bitstring.h"
 #include "src/common/list.h"
+#include "src/common/slurmdbd_defs.h"
+#include "src/common/slurm_protocol_defs.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
+
 #include "src/slurmctld/locks.h"
 #include "src/slurmctld/slurmctld.h"
 #include "src/slurmctld/state_save.h"
@@ -65,7 +68,9 @@
 #define MAX_PROG_TIME 300	/* maximum run time for program */
 
 /* Change TRIGGER_STATE_VERSION value when changing the state save format */
-#define TRIGGER_STATE_VERSION      "VER002"
+#define TRIGGER_STATE_VERSION      "VER003"
+#define TRIGGER_2_2_STATE_VERSION  "VER003"	/* SLURM version 2.2 */
+#define TRIGGER_2_1_STATE_VERSION  "VER002"	/* SLURM version 2.1 */
 
 List trigger_list;
 uint32_t next_trigger_id = 1;
@@ -76,6 +81,23 @@ bitstr_t *trigger_fail_nodes_bitmap = NULL;
 bitstr_t *trigger_up_nodes_bitmap   = NULL;
 static bool trigger_block_err = false;
 static bool trigger_node_reconfig = false;
+static bool trigger_pri_ctld_fail = false;
+static bool trigger_pri_ctld_res_op = false;
+static bool trigger_pri_ctld_res_ctrl = false;
+static bool trigger_pri_ctld_acct_buffer_full = false;
+static bool trigger_bu_ctld_fail = false;
+static bool trigger_bu_ctld_res_op = false;
+static bool trigger_bu_ctld_as_ctrl = false;
+static bool trigger_pri_dbd_fail = false;
+static bool trigger_pri_dbd_res_op = false;
+static bool trigger_pri_db_fail = false;
+static bool trigger_pri_db_res_op = false;
+
+/* Current trigger pull states (saved and restored) */
+uint8_t ctld_failure = 0;
+uint8_t bu_ctld_failure = 0;
+uint8_t db_failure = 0;
+uint8_t dbd_failure = 0;
 
 typedef struct trig_mgr_info {
 	uint32_t trig_id;	/* trigger ID */
@@ -84,7 +106,7 @@ typedef struct trig_mgr_info {
 	bitstr_t *nodes_bitmap;	/* bitmap of requested nodes (if applicable) */
 	uint32_t job_id;	/* job ID (if applicable) */
 	struct job_record *job_ptr; /* pointer to job record (if applicable) */
-	uint16_t trig_type;	/* TRIGGER_TYPE_* */
+	uint32_t trig_type;	/* TRIGGER_TYPE_* */
 	time_t   trig_time;	/* offset (pending) or time stamp (complete) */
 	uint32_t user_id;	/* user requesting trigger */
 	uint32_t group_id;	/* user's group id (pending) or pid (complete) */
@@ -101,40 +123,6 @@ void _trig_del(void *x) {
 	xfree(tmp);
 }
 
-static char *_res_type(uint16_t res_type)
-{
-	if      (res_type == TRIGGER_RES_TYPE_JOB)
-		return "job";
-	else if (res_type == TRIGGER_RES_TYPE_NODE)
-		return "node";
-	else
-		return "unknown";
-}
-
-static char *_trig_type(uint16_t trig_type)
-{
-	if      (trig_type == TRIGGER_TYPE_UP)
-		return "up";
-	else if (trig_type == TRIGGER_TYPE_DOWN)
-		return "down";
-	else if (trig_type == TRIGGER_TYPE_DRAINED)
-		return "drained";
-	else if (trig_type == TRIGGER_TYPE_FAIL)
-		return "fail";
-	else if (trig_type == TRIGGER_TYPE_IDLE)
-		return "idle";
-	else if (trig_type == TRIGGER_TYPE_TIME)
-		return "time";
-	else if (trig_type == TRIGGER_TYPE_FINI)
-		return "fini";
-	else if (trig_type == TRIGGER_TYPE_RECONFIG)
-		return "reconfig";
-	else if (trig_type == TRIGGER_TYPE_BLOCK_ERR)
-		return "block_err";
-	else
-		return "unknown";
-}
-
 static int _trig_offset(uint16_t offset)
 {
 	static int rc;
@@ -150,7 +138,7 @@ static void _dump_trigger_msg(char *header, trigger_info_msg_t *msg)
 	if ((slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) == 0)
 		return;
 
-	info(header);
+	info("%s", header);
 	if ((msg == NULL) || (msg->record_count == 0)) {
 		info("Trigger has no entries");
 		return;
@@ -159,16 +147,22 @@ static void _dump_trigger_msg(char *header, trigger_info_msg_t *msg)
 	info("INDEX TRIG_ID RES_TYPE RES_ID TRIG_TYPE OFFSET UID PROGRAM");
 	for (i=0; i<msg->record_count; i++) {
 		info("trigger[%u] %u %s %s %s %d %u %s", i,
-			msg->trigger_array[i].trig_id,
-			_res_type(msg->trigger_array[i].res_type),
-			msg->trigger_array[i].res_id,
-			_trig_type(msg->trigger_array[i].trig_type),
-			_trig_offset(msg->trigger_array[i].offset),
-			msg->trigger_array[i].user_id,
-			msg->trigger_array[i].program);
+		     msg->trigger_array[i].trig_id,
+		     trigger_res_type(msg->trigger_array[i].res_type),
+		     msg->trigger_array[i].res_id,
+		     trigger_type(msg->trigger_array[i].trig_type),
+		     _trig_offset(msg->trigger_array[i].offset),
+		     msg->trigger_array[i].user_id,
+		     msg->trigger_array[i].program);
 	}
 }
 
+
+static int _match_all_triggers(void *x, void *key)
+{
+	return 1;
+}
+
 /* Validate trigger program */
 static bool _validate_trigger(trig_mgr_info_t *trig_in)
 {
@@ -196,6 +190,76 @@ static bool _validate_trigger(trig_mgr_info_t *trig_in)
 	return false;
 }
 
+
+extern int trigger_pull(trigger_info_msg_t *msg)
+{
+	int rc = ESRCH;
+	ListIterator trig_iter;
+	trigger_info_t *trig_in;
+	trig_mgr_info_t *trig_test;
+
+	if (trigger_list == NULL) {
+		trigger_list = list_create(_trig_del);
+		if (trigger_list == NULL)
+			fatal("list_create: malloc failure");
+	}
+
+	/* validate the request, designated trigger must be set */
+	_dump_trigger_msg("trigger_pull", msg);
+	if (msg->record_count != 1)
+		return ESRCH;
+	trig_in = msg->trigger_array;
+
+	if ((trig_in->res_type != TRIGGER_RES_TYPE_SLURMCTLD) &&
+	    (trig_in->res_type != TRIGGER_RES_TYPE_SLURMDBD)  &&
+	    (trig_in->res_type != TRIGGER_RES_TYPE_DATABASE)) {
+		return EINVAL;
+	}
+
+	/* now look for a valid request */
+	trig_iter = list_iterator_create(trigger_list);
+	while ((trig_test = list_next(trig_iter))) {
+		if ((trig_test->res_type  == trig_in->res_type) &&
+   		    (trig_test->trig_type == trig_in->trig_type)) {
+			switch(trig_test->trig_type) {
+			case TRIGGER_TYPE_PRI_CTLD_ACCT_FULL:
+				trigger_primary_ctld_acct_full();
+				break;
+			case TRIGGER_TYPE_BU_CTLD_FAIL:
+				trigger_backup_ctld_fail();
+				break;
+			case TRIGGER_TYPE_BU_CTLD_RES_OP:
+				trigger_backup_ctld_res_op();
+				break;
+			case TRIGGER_TYPE_BU_CTLD_AS_CTRL:
+				trigger_backup_ctld_as_ctrl();
+				break;
+			case TRIGGER_TYPE_PRI_DBD_FAIL:
+				trigger_primary_dbd_fail();
+				break;
+			case TRIGGER_TYPE_PRI_DBD_RES_OP:
+				trigger_primary_dbd_res_op();
+				break;
+			case TRIGGER_TYPE_PRI_DB_FAIL:
+				trigger_primary_db_fail();
+				break;
+			case TRIGGER_TYPE_PRI_DB_RES_OP:
+				trigger_primary_db_res_op();
+				break;
+			default:
+				error("trigger_pull call has invalid type: %u",
+				      trig_test->trig_type);
+				rc = EINVAL;
+				break;
+			}
+		}
+		rc = SLURM_SUCCESS;
+	}
+	list_iterator_destroy(trig_iter);
+
+	return rc;
+}
+
 extern int trigger_clear(uid_t uid, trigger_info_msg_t *msg)
 {
 	int rc = ESRCH;
@@ -234,7 +298,7 @@ extern int trigger_clear(uid_t uid, trigger_info_msg_t *msg)
 			continue;
 		if (job_id && (job_id != trig_test->job_id))
 			continue;
-		if (trig_in->user_id && 
+		if (trig_in->user_id &&
 		    (trig_in->user_id != trig_test->user_id))
 			continue;
 		if (trig_test->state == 2)	/* wait for proc termination */
@@ -265,7 +329,7 @@ extern trigger_info_msg_t * trigger_get(uid_t uid, trigger_info_msg_t *msg)
 	resp_data = xmalloc(sizeof(trigger_info_msg_t));
 	resp_data->record_count = list_count(trigger_list);
 	resp_data->trigger_array = xmalloc(sizeof(trigger_info_t) *
-			resp_data->record_count);
+					   resp_data->record_count);
 	trig_iter = list_iterator_create(trigger_list);
 	trig_out = resp_data->trigger_array;
 	while ((trig_in = list_next(trig_iter))) {
@@ -320,7 +384,7 @@ extern int trigger_set(uid_t uid, gid_t gid, trigger_info_msg_t *msg)
 	if (trigger_list == NULL) {
 		trigger_list = list_create(_trig_del);
 	} else if ((uid != 0) &&
-	           (list_count(trigger_list) >= slurmctld_conf.max_job_cnt)) {
+		   (list_count(trigger_list) >= slurmctld_conf.max_job_cnt)) {
 		rc = EAGAIN;
 		goto fini;
 	}
@@ -328,9 +392,9 @@ extern int trigger_set(uid_t uid, gid_t gid, trigger_info_msg_t *msg)
 	_dump_trigger_msg("trigger_set", msg);
 	for (i=0; i<msg->record_count; i++) {
 		if (msg->trigger_array[i].res_type ==
-				TRIGGER_RES_TYPE_JOB) {
+		    TRIGGER_RES_TYPE_JOB) {
 			job_id = (uint32_t) atol(
-				msg->trigger_array[i].res_id);
+				 msg->trigger_array[i].res_id);
 			job_ptr = find_job_record(job_id);
 			if (job_ptr == NULL) {
 				rc = ESLURM_INVALID_JOB_ID;
@@ -387,7 +451,7 @@ fini:	slurm_mutex_unlock(&trigger_mutex);
 
 extern void trigger_node_down(struct node_record *node_ptr)
 {
-        int inx = node_ptr - node_record_table_ptr;
+	int inx = node_ptr - node_record_table_ptr;
 
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_down_nodes_bitmap == NULL)
@@ -398,7 +462,7 @@ extern void trigger_node_down(struct node_record *node_ptr)
 
 extern void trigger_node_drained(struct node_record *node_ptr)
 {
-        int inx = node_ptr - node_record_table_ptr;
+	int inx = node_ptr - node_record_table_ptr;
 
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_drained_nodes_bitmap == NULL)
@@ -421,7 +485,7 @@ extern void trigger_node_failing(struct node_record *node_ptr)
 
 extern void trigger_node_up(struct node_record *node_ptr)
 {
-        int inx = node_ptr - node_record_table_ptr;
+	int inx = node_ptr - node_record_table_ptr;
 
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_up_nodes_bitmap == NULL)
@@ -436,6 +500,98 @@ extern void trigger_reconfig(void)
 	trigger_node_reconfig = true;
 	slurm_mutex_unlock(&trigger_mutex);
 }
+extern void trigger_primary_ctld_fail(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	if (ctld_failure != 1) {
+		trigger_pri_ctld_fail = true;
+		ctld_failure = 1;
+	}
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_primary_ctld_res_op(void)
+{	
+	slurm_mutex_lock(&trigger_mutex);
+	trigger_pri_ctld_res_op = true;
+	ctld_failure = 0;
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_primary_ctld_res_ctrl(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	trigger_pri_ctld_res_ctrl = true;
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_primary_ctld_acct_full(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	trigger_pri_ctld_acct_buffer_full = true;
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_backup_ctld_fail(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	if (bu_ctld_failure != 1) {
+		trigger_bu_ctld_fail = true;
+		bu_ctld_failure = 1;
+	}
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_backup_ctld_res_op(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	trigger_bu_ctld_res_op = true;
+	bu_ctld_failure = 0;
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_backup_ctld_as_ctrl(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	trigger_bu_ctld_as_ctrl = true;
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_primary_dbd_fail(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	if (dbd_failure != 1) {	
+		trigger_pri_dbd_fail = true;
+		dbd_failure = 1;
+	}
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_primary_dbd_res_op(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	trigger_pri_dbd_res_op = true;
+	dbd_failure = 0;
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_primary_db_fail(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+	if (db_failure != 1) {	
+		trigger_pri_db_fail = true;
+		db_failure = 1;
+	}
+	slurm_mutex_unlock(&trigger_mutex);
+}
+
+extern void trigger_primary_db_res_op(void)
+{
+	slurm_mutex_lock(&trigger_mutex);
+		trigger_pri_db_res_op = true;
+		db_failure = 0;
+	slurm_mutex_unlock(&trigger_mutex);
+}
 
 extern void trigger_block_error(void)
 {
@@ -446,13 +602,19 @@ extern void trigger_block_error(void)
 
 static void _dump_trigger_state(trig_mgr_info_t *trig_ptr, Buf buffer)
 {
+	/* write trigger pull state flags */
+	safe_pack8(ctld_failure,    buffer);
+	safe_pack8(bu_ctld_failure, buffer);
+	safe_pack8(dbd_failure,     buffer);
+	safe_pack8(db_failure,      buffer);
+
 	pack32   (trig_ptr->trig_id,   buffer);
 	pack16   (trig_ptr->res_type,  buffer);
 	packstr  (trig_ptr->res_id,    buffer);
 	/* rebuild nodes_bitmap as needed from res_id */
 	/* rebuild job_id as needed from res_id */
 	/* rebuild job_ptr as needed from res_id */
-	pack16   (trig_ptr->trig_type, buffer);
+	pack32   (trig_ptr->trig_type, buffer);
 	pack_time(trig_ptr->trig_time, buffer);
 	pack32   (trig_ptr->user_id,   buffer);
 	pack32   (trig_ptr->group_id,  buffer);
@@ -460,26 +622,50 @@ static void _dump_trigger_state(trig_mgr_info_t *trig_ptr, Buf buffer)
 	pack8    (trig_ptr->state,     buffer);
 }
 
-static int _load_trigger_state(Buf buffer)
+static int _load_trigger_state(Buf buffer, uint16_t protocol_version)
 {
 	trig_mgr_info_t *trig_ptr;
 	uint32_t str_len;
 
 	trig_ptr = xmalloc(sizeof(trig_mgr_info_t));
-	safe_unpack32   (&trig_ptr->trig_id,   buffer);
-	safe_unpack16   (&trig_ptr->res_type,  buffer);
-	safe_unpackstr_xmalloc(&trig_ptr->res_id, &str_len, buffer);
-	/* rebuild nodes_bitmap as needed from res_id */
-	/* rebuild job_id as needed from res_id */
-	/* rebuild job_ptr as needed from res_id */
-	safe_unpack16   (&trig_ptr->trig_type, buffer);
-	safe_unpack_time(&trig_ptr->trig_time, buffer);
-	safe_unpack32   (&trig_ptr->user_id,   buffer);
-	safe_unpack32   (&trig_ptr->group_id,  buffer);
-	safe_unpackstr_xmalloc(&trig_ptr->program, &str_len, buffer);
-	safe_unpack8    (&trig_ptr->state,     buffer);
+	if (protocol_version >= SLURM_2_2_PROTOCOL_VERSION) {
+		/* restore trigger pull state flags */
+		safe_unpack8(&ctld_failure, buffer);
+		safe_unpack8(&bu_ctld_failure, buffer);
+		safe_unpack8(&dbd_failure, buffer);
+		safe_unpack8(&db_failure, buffer);
+
+		safe_unpack32   (&trig_ptr->trig_id,   buffer);
+		safe_unpack16   (&trig_ptr->res_type,  buffer);
+		safe_unpackstr_xmalloc(&trig_ptr->res_id, &str_len, buffer);
+		/* rebuild nodes_bitmap as needed from res_id */
+		/* rebuild job_id as needed from res_id */
+		/* rebuild job_ptr as needed from res_id */
+		safe_unpack32   (&trig_ptr->trig_type, buffer);
+		safe_unpack_time(&trig_ptr->trig_time, buffer);
+		safe_unpack32   (&trig_ptr->user_id,   buffer);
+		safe_unpack32   (&trig_ptr->group_id,  buffer);
+		safe_unpackstr_xmalloc(&trig_ptr->program, &str_len, buffer);
+		safe_unpack8    (&trig_ptr->state,     buffer);
+	} else {
+		uint16_t uint16_tmp;
+		safe_unpack32   (&trig_ptr->trig_id,   buffer);
+		safe_unpack16   (&trig_ptr->res_type,  buffer);
+		safe_unpackstr_xmalloc(&trig_ptr->res_id, &str_len, buffer);
+		/* rebuild nodes_bitmap as needed from res_id */
+		/* rebuild job_id as needed from res_id */
+		/* rebuild job_ptr as needed from res_id */
+		safe_unpack16   (&uint16_tmp,          buffer);
+		trig_ptr->trig_type = (uint32_t) uint16_tmp;
+		safe_unpack_time(&trig_ptr->trig_time, buffer);
+		safe_unpack32   (&trig_ptr->user_id,   buffer);
+		safe_unpack32   (&trig_ptr->group_id,  buffer);
+		safe_unpackstr_xmalloc(&trig_ptr->program, &str_len, buffer);
+		safe_unpack8    (&trig_ptr->state,     buffer);
+	}
+
 	if ((trig_ptr->res_type < TRIGGER_RES_TYPE_JOB)  ||
-	    (trig_ptr->res_type > TRIGGER_RES_TYPE_NODE) ||
+	    (trig_ptr->res_type > TRIGGER_RES_TYPE_DATABASE) ||
 	    (trig_ptr->state > 2))
 		goto unpack_error;
 	if (trig_ptr->res_type == TRIGGER_RES_TYPE_JOB) {
@@ -489,13 +675,13 @@ static int _load_trigger_state(Buf buffer)
 		    (trig_ptr->job_ptr == NULL) ||
 		    (IS_JOB_COMPLETED(trig_ptr->job_ptr)))
 			goto unpack_error;
-	} else {
+	} else if (trig_ptr->res_type == TRIGGER_RES_TYPE_NODE) {
 		trig_ptr->job_id = 0;
 		trig_ptr->job_ptr = NULL;
 		if ((trig_ptr->res_id != NULL)   &&
 		    (trig_ptr->res_id[0] != '*') &&
 		    (node_name2bitmap(trig_ptr->res_id, false,
-				&trig_ptr->nodes_bitmap) != 0))
+				      &trig_ptr->nodes_bitmap) != 0))
 			goto unpack_error;
 	}
 
@@ -579,24 +765,25 @@ extern int trigger_state_save(void)
 		if (rc && !error_code)
 			error_code = rc;
 	}
-	if (error_code)
+	if (error_code) {
 		(void) unlink(new_file);
-	else {			/* file shuffle */
+	} else {			/* file shuffle */
 		(void) unlink(old_file);
-		if(link(reg_file, old_file))
+		if (link(reg_file, old_file)) {
 			debug4("unable to create link for %s -> %s: %m",
 			       reg_file, old_file);
+		}
 		(void) unlink(reg_file);
-		if(link(new_file, reg_file))
+		if (link(new_file, reg_file)) {
 			debug4("unable to create link for %s -> %s: %m",
 			       new_file, reg_file);
+		}
 		(void) unlink(new_file);
 	}
 	xfree(old_file);
 	xfree(reg_file);
 	xfree(new_file);
 	unlock_state_files();
-
 	free_buf(buffer);
 	return error_code;
 }
@@ -636,6 +823,7 @@ extern int trigger_state_restore(void)
 {
 	int data_allocated, data_read = 0, error_code = 0;
 	uint32_t data_size = 0;
+	uint16_t protocol_version = (uint16_t) NO_VAL;
 	int state_fd, trigger_cnt = 0;
 	char *data = NULL, *state_file;
 	Buf buffer;
@@ -676,7 +864,15 @@ extern int trigger_state_restore(void)
 
 	buffer = create_buf(data, data_size);
 	safe_unpackstr_xmalloc(&ver_str, &ver_str_len, buffer);
-	if (strcmp(ver_str, TRIGGER_STATE_VERSION) != 0) {
+	if (ver_str) {
+		if (!strcmp(ver_str, TRIGGER_STATE_VERSION)) {
+			protocol_version = SLURM_PROTOCOL_VERSION;
+		} else if (!strcmp(ver_str, TRIGGER_2_1_STATE_VERSION)) {
+			protocol_version = SLURM_2_1_PROTOCOL_VERSION;
+		}
+	}
+
+	if (protocol_version == (uint16_t) NO_VAL) {
 		error("Can't recover trigger state, data version "
 		      "incompatible");
 		xfree(ver_str);
@@ -686,9 +882,10 @@ extern int trigger_state_restore(void)
 	xfree(ver_str);
 
 	safe_unpack_time(&buf_time, buffer);
-
+	if (trigger_list)
+		list_delete_all (trigger_list, _match_all_triggers, NULL);
 	while (remaining_buf(buffer) > 0) {
-		error_code = _load_trigger_state(buffer);
+		error_code = _load_trigger_state(buffer, protocol_version);
 		if (error_code != SLURM_SUCCESS)
 			goto unpack_error;
 		trigger_cnt++;
@@ -706,6 +903,7 @@ fini:	verbose("State of %d triggers recovered", trigger_cnt);
 static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 {
 	if ((trig_in->job_ptr == NULL) ||
+	    (trig_in->job_ptr->magic != JOB_MAGIC) ||
 	    (trig_in->job_ptr->job_id != trig_in->job_id))
 		trig_in->job_ptr = find_job_record(trig_in->job_ptr->job_id);
 
@@ -716,7 +914,7 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 			info("trigger[%u] event for job %u fini",
-				trig_in->trig_id, trig_in->job_id);
+			     trig_in->trig_id, trig_in->job_id);
 		}
 		return;
 	}
@@ -724,7 +922,7 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 	if (trig_in->job_ptr == NULL) {
 		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 			info("trigger[%u] for defunct job %u",
-				trig_in->trig_id, trig_in->job_id);
+			     trig_in->trig_id, trig_in->job_id);
 		}
 		trig_in->state = 2;
 		trig_in->trig_time = now;
@@ -739,7 +937,7 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 			trig_in->trig_time = now;
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for job %u time",
-					trig_in->trig_id, trig_in->job_id);
+				     trig_in->trig_id, trig_in->job_id);
 			}
 			return;
 		}
@@ -751,11 +949,11 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 				trigger_down_nodes_bitmap)) {
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for job %u down",
-					trig_in->trig_id, trig_in->job_id);
+				     trig_in->trig_id, trig_in->job_id);
 			}
 			trig_in->state = 1;
 			trig_in->trig_time = now +
-					(trig_in->trig_time - 0x8000);
+					    (trig_in->trig_time - 0x8000);
 			return;
 		}
 	}
@@ -766,11 +964,11 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 				trigger_fail_nodes_bitmap)) {
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for job %u node fail",
-					trig_in->trig_id, trig_in->job_id);
+				     trig_in->trig_id, trig_in->job_id);
 			}
 			trig_in->state = 1;
 			trig_in->trig_time = now +
-					(trig_in->trig_time - 0x8000);
+					     (trig_in->trig_time - 0x8000);
 			return;
 		}
 	}
@@ -781,10 +979,10 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 				trigger_up_nodes_bitmap)) {
 			trig_in->state = 1;
 			trig_in->trig_time = now +
-					(0x8000 - trig_in->trig_time);
+					    (0x8000 - trig_in->trig_time);
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for job %u up",
-					trig_in->trig_id, trig_in->job_id);
+				     trig_in->trig_id, trig_in->job_id);
 			}
 			return;
 		}
@@ -794,7 +992,7 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 {
 	if ((trig_in->trig_type & TRIGGER_TYPE_BLOCK_ERR) &&
-	     trigger_block_err) {
+	    trigger_block_err) {
 		trig_in->state = 1;
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS)
@@ -803,84 +1001,84 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 	}
 
 	if ((trig_in->trig_type & TRIGGER_TYPE_DOWN) &&
-	     trigger_down_nodes_bitmap               &&
-	     (bit_ffs(trigger_down_nodes_bitmap) != -1)) {
+	    trigger_down_nodes_bitmap               &&
+	    (bit_ffs(trigger_down_nodes_bitmap) != -1)) {
 		if (trig_in->nodes_bitmap == NULL) {	/* all nodes */
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trigger_down_nodes_bitmap);
+					  trigger_down_nodes_bitmap);
 			trig_in->state = 1;
 		} else if (bit_overlap(trig_in->nodes_bitmap,
-					trigger_down_nodes_bitmap)) {
+				       trigger_down_nodes_bitmap)) {
 			bit_and(trig_in->nodes_bitmap,
-					trigger_down_nodes_bitmap);
+				trigger_down_nodes_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trig_in->nodes_bitmap);
+					 trig_in->nodes_bitmap);
 			trig_in->state = 1;
 		}
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now +
-					(trig_in->trig_time - 0x8000);
+					     (trig_in->trig_time - 0x8000);
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for node %s down",
-					trig_in->trig_id, trig_in->res_id);
+				     trig_in->trig_id, trig_in->res_id);
 			}
 			return;
 		}
 	}
 
 	if ((trig_in->trig_type & TRIGGER_TYPE_DRAINED) &&
-	     trigger_drained_nodes_bitmap               &&
-	     (bit_ffs(trigger_drained_nodes_bitmap) != -1)) {
+	    trigger_drained_nodes_bitmap               &&
+	    (bit_ffs(trigger_drained_nodes_bitmap) != -1)) {
 		if (trig_in->nodes_bitmap == NULL) {	/* all nodes */
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trigger_drained_nodes_bitmap);
+					  trigger_drained_nodes_bitmap);
 			trig_in->state = 1;
 		} else if (bit_overlap(trig_in->nodes_bitmap,
-					trigger_drained_nodes_bitmap)) {
+				       trigger_drained_nodes_bitmap)) {
 			bit_and(trig_in->nodes_bitmap,
-					trigger_drained_nodes_bitmap);
+				trigger_drained_nodes_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trig_in->nodes_bitmap);
+					  trig_in->nodes_bitmap);
 			trig_in->state = 1;
 		}
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now +
-					(trig_in->trig_time - 0x8000);
+					     (trig_in->trig_time - 0x8000);
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for node %s drained",
-					trig_in->trig_id, trig_in->res_id);
+				     trig_in->trig_id, trig_in->res_id);
 			}
 			return;
 		}
 	}
 
 	if ((trig_in->trig_type & TRIGGER_TYPE_FAIL) &&
-	     trigger_fail_nodes_bitmap               &&
-	     (bit_ffs(trigger_fail_nodes_bitmap) != -1)) {
+	    trigger_fail_nodes_bitmap               &&
+	    (bit_ffs(trigger_fail_nodes_bitmap) != -1)) {
 		if (trig_in->nodes_bitmap == NULL) {	/* all nodes */
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trigger_fail_nodes_bitmap);
+					  trigger_fail_nodes_bitmap);
 			trig_in->state = 1;
 		} else if (bit_overlap(trig_in->nodes_bitmap,
-					trigger_fail_nodes_bitmap)) {
+				       trigger_fail_nodes_bitmap)) {
 			bit_and(trig_in->nodes_bitmap,
-					trigger_fail_nodes_bitmap);
+				trigger_fail_nodes_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trig_in->nodes_bitmap);
+					  trig_in->nodes_bitmap);
 			trig_in->state = 1;
 		}
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now +
-					(trig_in->trig_time - 0x8000);
+					     (trig_in->trig_time - 0x8000);
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for node %s fail",
-					trig_in->trig_id, trig_in->res_id);
+				     trig_in->trig_id, trig_in->res_id);
 			}
 			return;
 		}
@@ -904,58 +1102,58 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		if (trig_in->nodes_bitmap == NULL) {    /* all nodes */
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trigger_idle_node_bitmap);
+					  trigger_idle_node_bitmap);
 			trig_in->state = 1;
 		} else if (bit_overlap(trig_in->nodes_bitmap,
-				trigger_idle_node_bitmap)) {
+				       trigger_idle_node_bitmap)) {
 			bit_and(trig_in->nodes_bitmap,
 				trigger_idle_node_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trig_in->nodes_bitmap);
+					  trig_in->nodes_bitmap);
 			trig_in->state = 1;
 		}
-		bit_free(trigger_idle_node_bitmap);
+		FREE_NULL_BITMAP(trigger_idle_node_bitmap);
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now;
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for node %s idle",
-					trig_in->trig_id, trig_in->res_id);
+				     trig_in->trig_id, trig_in->res_id);
 			}
 			return;
 		}
 	}
 
 	if ((trig_in->trig_type & TRIGGER_TYPE_UP) &&
-	     trigger_up_nodes_bitmap               &&
-	     (bit_ffs(trigger_up_nodes_bitmap) != -1)) {
+	    trigger_up_nodes_bitmap               &&
+	    (bit_ffs(trigger_up_nodes_bitmap) != -1)) {
 		if (trig_in->nodes_bitmap == NULL) {	/* all nodes */
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trigger_up_nodes_bitmap);
+					  trigger_up_nodes_bitmap);
 			trig_in->state = 1;
 		} else if (bit_overlap(trig_in->nodes_bitmap,
-					trigger_up_nodes_bitmap)) {
+				       trigger_up_nodes_bitmap)) {
 			bit_and(trig_in->nodes_bitmap,
-					trigger_up_nodes_bitmap);
+				trigger_up_nodes_bitmap);
 			xfree(trig_in->res_id);
 			trig_in->res_id = bitmap2node_name(
-					trig_in->nodes_bitmap);
+					  trig_in->nodes_bitmap);
 			trig_in->state = 1;
 		}
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now +
-					(trig_in->trig_time - 0x8000);
+					     (trig_in->trig_time - 0x8000);
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("trigger[%u] for node %s up",
-					trig_in->trig_id, trig_in->res_id);
+				     trig_in->trig_id, trig_in->res_id);
 			}
 			return;
 		}
 	}
 
 	if ((trig_in->trig_type & TRIGGER_TYPE_RECONFIG) &&
-	     trigger_node_reconfig) {
+	    trigger_node_reconfig) {
 		trig_in->state = 1;
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
@@ -966,6 +1164,149 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 	}
 }
 
+static void _trigger_slurmctld_event(trig_mgr_info_t *trig_in, time_t now)
+{
+	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_CTLD_FAIL) &&
+	    trigger_pri_ctld_fail) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("primary_slurmctld_failure");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for primary_slurmctld_failure",
+			     trig_in->trig_id);
+		}
+		return;
+	}
+	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_CTLD_RES_OP) &&
+	    trigger_pri_ctld_res_op) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id =
+			xstrdup("primary_slurmctld_resumed_operation");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for primary_slurmctld_resumed_"
+			     "operation", trig_in->trig_id);
+		}
+		return;
+	}
+	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_CTLD_RES_CTRL) &&
+	    trigger_pri_ctld_res_ctrl) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("primary_slurmctld_resumed_control");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for primary_slurmctld_resumed_"
+			     "control", trig_in->trig_id);
+		}
+		return;
+	}
+	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_CTLD_ACCT_FULL) &&
+	    trigger_pri_ctld_acct_buffer_full) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("primary_slurmctld_acct_buffer_full");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for primary_slurmctld_acct_"
+			     "buffer_full", trig_in->trig_id);
+		}
+		return;
+	}
+	if ((trig_in->trig_type & TRIGGER_TYPE_BU_CTLD_FAIL) &&
+	    trigger_bu_ctld_fail) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("backup_slurmctld_failure");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for backup_slurmctld_failure",
+			     trig_in->trig_id);
+		}
+		return;
+	}
+	if ((trig_in->trig_type & TRIGGER_TYPE_BU_CTLD_RES_OP) &&
+	    trigger_bu_ctld_res_op) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("backup_slurmctld_resumed_operation");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for backup_slurmctld_resumed_"
+			     "operation", trig_in->trig_id);
+		}
+		return;
+	}
+	if ((trig_in->trig_type & TRIGGER_TYPE_BU_CTLD_AS_CTRL) &&
+	    trigger_bu_ctld_as_ctrl) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("backup_slurmctld_assumed_control");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for bu_slurmctld_assumed_control",
+			     trig_in->trig_id);
+		}
+		return;
+	}
+}
+
+static void _trigger_slurmdbd_event(trig_mgr_info_t *trig_in, time_t now)
+{
+	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_DBD_FAIL) &&
+	    trigger_pri_dbd_fail) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("primary_slurmdbd_failure");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS)
+			info("trigger[%u] for primary_slurmcdbd_failure",
+			     trig_in->trig_id);
+		return;
+	}
+	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_DBD_RES_OP) &&
+	    trigger_pri_dbd_res_op) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("primary_slurmdbd_resumed_operation");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for primary_slurmdbd_resumed_"
+			     "operation", trig_in->trig_id);
+		}
+		return;
+	}
+}
+
+static void _trigger_database_event(trig_mgr_info_t *trig_in, time_t now)
+{
+	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_DB_FAIL) &&
+	    trigger_pri_db_fail) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("primary_database_failure");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for primary_database_failure",
+			     trig_in->trig_id);
+		}
+		return;
+	}
+	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_DB_RES_OP) &&
+	    trigger_pri_db_res_op) {
+		trig_in->state = 1;
+		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
+		xfree(trig_in->res_id);
+		trig_in->res_id = xstrdup("primary_database_resumed_operation");
+		if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
+			info("trigger[%u] for primary_database_resumed_"
+			     "operation", trig_in->trig_id);
+		}
+		return;
+	}
+}
 /* Ideally we would use the existing proctrack plugin to prevent any
  * processes from escaping our control, but that plugin is tied
  * to various slurmd data structures. We just the process group ID
@@ -1045,6 +1386,17 @@ static void _clear_event_triggers(void)
 	}
 	trigger_node_reconfig = false;
 	trigger_block_err = false;
+	trigger_pri_ctld_fail = false;
+	trigger_pri_ctld_res_op = false;
+	trigger_pri_ctld_res_ctrl = false;
+	trigger_pri_ctld_acct_buffer_full = false;
+	trigger_bu_ctld_fail = false;
+	trigger_bu_ctld_res_op = false;
+	trigger_bu_ctld_as_ctrl = false;
+	trigger_pri_dbd_fail = false;
+	trigger_pri_dbd_res_op = false;
+	trigger_pri_db_fail = false;
+	trigger_pri_db_res_op = false;
 }
 
 extern void trigger_process(void)
@@ -1052,13 +1404,10 @@ extern void trigger_process(void)
 	ListIterator trig_iter;
 	trig_mgr_info_t *trig_in;
 	time_t now = time(NULL);
-	slurmctld_lock_t job_node_read_lock =
-		{ NO_LOCK, READ_LOCK, READ_LOCK, NO_LOCK };
 	bool state_change = false;
 	pid_t rc;
 	int prog_stat;
 
-	lock_slurmctld(job_node_read_lock);
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_list == NULL)
 		trigger_list = list_create(_trig_del);
@@ -1068,17 +1417,25 @@ extern void trigger_process(void)
 		if (trig_in->state == 0) {
 			if (trig_in->res_type == TRIGGER_RES_TYPE_JOB)
 				_trigger_job_event(trig_in, now);
-			else
+			else if (trig_in->res_type == TRIGGER_RES_TYPE_NODE)
 				_trigger_node_event(trig_in, now);
+			else if (trig_in->res_type ==
+				 TRIGGER_RES_TYPE_SLURMCTLD)
+				_trigger_slurmctld_event(trig_in, now);
+			else if (trig_in->res_type ==
+				 TRIGGER_RES_TYPE_SLURMDBD)
+				_trigger_slurmdbd_event(trig_in, now);
+			else /* TRIGGER_RES_TYPE_DATABASE */
+			 	_trigger_database_event(trig_in, now);
 		}
 		if ((trig_in->state == 1) &&
 		    (trig_in->trig_time <= now)) {
 			if (slurm_get_debug_flags() & DEBUG_FLAG_TRIGGERS) {
 				info("launching program for trigger[%u]",
-					trig_in->trig_id);
+				     trig_in->trig_id);
 				info("  uid=%u gid=%u program=%s arg=%s",
-					trig_in->user_id, trig_in->group_id,
-					trig_in->program, trig_in->res_id);
+				     trig_in->user_id, trig_in->group_id,
+				     trig_in->program, trig_in->res_id);
 			}
 			trig_in->state = 2;
 			trig_in->trig_time = now;
@@ -1086,7 +1443,7 @@ extern void trigger_process(void)
 			_trigger_run_program(trig_in);
 		} else if ((trig_in->state == 2) &&
 			   (difftime(now, trig_in->trig_time) >
-					MAX_PROG_TIME)) {
+			    MAX_PROG_TIME)) {
 			if (trig_in->group_id != 0) {
 				killpg(trig_in->group_id, SIGKILL);
 				rc = waitpid(trig_in->group_id, &prog_stat,
@@ -1095,8 +1452,8 @@ extern void trigger_process(void)
 					info("trigger uid=%u type=%s:%s "
 					     "exit=%u:%u",
 					     trig_in->user_id,
-					     _res_type(trig_in->res_type),
-					     _trig_type(trig_in->trig_type),
+					     trigger_res_type(trig_in->res_type),
+					     trigger_type(trig_in->trig_type),
 					     WIFEXITED(prog_stat),
 					     WTERMSIG(prog_stat));
 				}
@@ -1121,9 +1478,9 @@ extern void trigger_process(void)
 			if ((rc > 0) && prog_stat) {
 				info("trigger uid=%u type=%s:%s exit=%u:%u",
 				     trig_in->user_id,
-				     _res_type(trig_in->res_type),
-				     _trig_type(trig_in->trig_type),
-				     WIFEXITED(prog_stat), 
+				     trigger_res_type(trig_in->res_type),
+				     trigger_type(trig_in->trig_type),
+				     WIFEXITED(prog_stat),
 				     WTERMSIG(prog_stat));
 			}
 			if ((rc == trig_in->group_id) ||
@@ -1134,7 +1491,6 @@ extern void trigger_process(void)
 	list_iterator_destroy(trig_iter);
 	_clear_event_triggers();
 	slurm_mutex_unlock(&trigger_mutex);
-	unlock_slurmctld(job_node_read_lock);
 	if (state_change)
 		schedule_trigger_save();
 }
