@@ -2,7 +2,7 @@
  *  sort.c - squeue sorting functions
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
+ *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>, et. al.
  *  CODE-OCEC-09-009. All rights reserved.
@@ -51,6 +51,7 @@
 
 static bool reverse_order;
 
+static int _sort_job_by_gres(void *void1, void *void2);
 static int _sort_job_by_group_id(void *void1, void *void2);
 static int _sort_job_by_group_name(void *void1, void *void2);
 static int _sort_job_by_id(void *void1, void *void2);
@@ -64,11 +65,11 @@ static int _sort_job_by_time_start(void *void1, void *void2);
 static int _sort_job_by_time_used(void *void1, void *void2);
 static int _sort_job_by_node_list(void *void1, void *void2);
 static int _sort_job_by_num_nodes(void *void1, void *void2);
-static int _sort_job_by_num_procs(void *void1, void *void2);
+static int _sort_job_by_num_cpus(void *void1, void *void2);
 static int _sort_job_by_num_sct(void *void1, void *void2);
-static int _sort_job_by_min_sockets(void *void1, void *void2);
-static int _sort_job_by_min_cores(void *void1, void *void2);
-static int _sort_job_by_min_threads(void *void1, void *void2);
+static int _sort_job_by_sockets(void *void1, void *void2);
+static int _sort_job_by_cores(void *void1, void *void2);
+static int _sort_job_by_threads(void *void1, void *void2);
 static int _sort_job_by_min_memory(void *void1, void *void2);
 static int _sort_job_by_min_tmp_disk(void *void1, void *void2);
 static int _sort_job_by_partition(void *void1, void *void2);
@@ -77,6 +78,7 @@ static int _sort_job_by_user_id(void *void1, void *void2);
 static int _sort_job_by_user_name(void *void1, void *void2);
 static int _sort_job_by_reservation(void *void1, void *void2);
 
+static int _sort_step_by_gres(void *void1, void *void2);
 static int _sort_step_by_id(void *void1, void *void2);
 static int _sort_step_by_node_list(void *void1, void *void2);
 static int _sort_step_by_partition(void *void1, void *void2);
@@ -105,10 +107,12 @@ void sort_job_list(List job_list)
 		if ((i > 0) && (params.sort[i-1] == '-'))
 			reverse_order = true;
 
-		if      (params.sort[i] == 'c')
+		if      (params.sort[i] == 'b')
+			list_sort(job_list, _sort_job_by_gres);
+		else if (params.sort[i] == 'c')
 			;	/* sort_job_by_min_cpus_per_node */
 		else if (params.sort[i] == 'C')
-			list_sort(job_list, _sort_job_by_num_procs);
+			list_sort(job_list, _sort_job_by_num_cpus);
 		else if (params.sort[i] == 'd')
 			list_sort(job_list, _sort_job_by_min_tmp_disk);
 		else if (params.sort[i] == 'D')
@@ -124,15 +128,15 @@ void sort_job_list(List job_list)
 		else if (params.sort[i] == 'h')
 			;	/* sort_job_by_shared */
 		else if (params.sort[i] == 'H')
-			list_sort(job_list, _sort_job_by_min_sockets);
+			list_sort(job_list, _sort_job_by_sockets);
 		else if (params.sort[i] == 'i')
 			list_sort(job_list, _sort_job_by_id);
 		else if (params.sort[i] == 'I')
-			list_sort(job_list, _sort_job_by_min_cores);
+			list_sort(job_list, _sort_job_by_cores);
 		else if (params.sort[i] == 'j')
 			list_sort(job_list, _sort_job_by_name);
 		else if (params.sort[i] == 'J')
-			list_sort(job_list, _sort_job_by_min_threads);
+			list_sort(job_list, _sort_job_by_threads);
 		else if (params.sort[i] == 'l')
 			list_sort(job_list, _sort_job_by_time_limit);
 		else if (params.sort[i] == 'L')
@@ -165,9 +169,11 @@ void sort_job_list(List job_list)
 			list_sort(job_list, _sort_job_by_reservation);
 		else if (params.sort[i] == 'z')
 			list_sort(job_list, _sort_job_by_num_sct);
-		else
+		else {
 			error("Invalid sort specification: %c",
 			      params.sort[i]);
+			exit(1);
+		}
 	}
 }
 
@@ -192,7 +198,9 @@ void sort_step_list(List step_list)
 		if ((i > 0) && (params.sort[i-1] == '-'))
 			reverse_order = true;
 
-		if      (params.sort[i] == 'i')
+		if      (params.sort[i] == 'b')
+			list_sort(step_list, _sort_step_by_gres);
+		else if (params.sort[i] == 'i')
 			list_sort(step_list, _sort_step_by_id);
 		else if (params.sort[i] == 'N')
 			list_sort(step_list, _sort_step_by_node_list);
@@ -214,6 +222,24 @@ void sort_step_list(List step_list)
 /*****************************************************************************
  * Local Job Sort Functions
  *****************************************************************************/
+static int _sort_job_by_gres(void *void1, void *void2)
+{
+	int diff;
+	job_info_t *job1 = (job_info_t *) void1;
+	job_info_t *job2 = (job_info_t *) void2;
+	char *val1 = "", *val2 = "";
+
+	if (job1->gres)
+		val1 = job1->gres;
+	if (job2->gres)
+		val2 = job2->gres;
+	diff = strcmp(val1, val2);
+
+	if (reverse_order)
+		diff = -diff;
+	return diff;
+}
+
 static int _sort_job_by_group_id(void *void1, void *void2)
 {
 	int diff;
@@ -345,13 +371,13 @@ static int _sort_job_by_num_nodes(void *void1, void *void2)
 	return diff;
 }
 
-static int _sort_job_by_num_procs(void *void1, void *void2)
+static int _sort_job_by_num_cpus(void *void1, void *void2)
 {
 	int diff;
 	job_info_t *job1 = (job_info_t *) void1;
 	job_info_t *job2 = (job_info_t *) void2;
 
-	diff = job1->num_procs - job2->num_procs;
+	diff = job1->num_cpus - job2->num_cpus;
 
 	if (reverse_order)
 		diff = -diff;
@@ -364,9 +390,9 @@ static int _sort_job_by_num_sct(void *void1, void *void2)
 	job_info_t *job1 = (job_info_t *) void1;
 	job_info_t *job2 = (job_info_t *) void2;
 
-	diffs = job1->min_sockets - job2->min_sockets;
-	diffc = job1->min_cores - job2->min_cores;
-	difft = job1->min_threads - job2->min_threads;
+	diffs = job1->sockets_per_node - job2->sockets_per_node;
+	diffc = job1->cores_per_socket - job2->cores_per_socket;
+	difft = job1->threads_per_core - job2->threads_per_core;
 
 	if (reverse_order) {
 		diffs = -diffs;
@@ -381,39 +407,39 @@ static int _sort_job_by_num_sct(void *void1, void *void2)
 		return difft;
 }
 
-static int _sort_job_by_min_sockets(void *void1, void *void2)
+static int _sort_job_by_sockets(void *void1, void *void2)
 {
 	int diff;
 	job_info_t *job1 = (job_info_t *) void1;
 	job_info_t *job2 = (job_info_t *) void2;
 
-	diff = job1->min_sockets - job2->min_sockets;
+	diff = job1->sockets_per_node - job2->sockets_per_node;
 
 	if (reverse_order)
 		diff = -diff;
 	return diff;
 }
 
-static int _sort_job_by_min_cores(void *void1, void *void2)
+static int _sort_job_by_cores(void *void1, void *void2)
 {
 	int diff;
 	job_info_t *job1 = (job_info_t *) void1;
 	job_info_t *job2 = (job_info_t *) void2;
 
-	diff = job1->min_cores - job2->min_cores;
+	diff = job1->cores_per_socket - job2->cores_per_socket;
 
 	if (reverse_order)
 		diff = -diff;
 	return diff;
 }
 
-static int _sort_job_by_min_threads(void *void1, void *void2)
+static int _sort_job_by_threads(void *void1, void *void2)
 {
 	int diff;
 	job_info_t *job1 = (job_info_t *) void1;
 	job_info_t *job2 = (job_info_t *) void2;
 
-	diff = job1->min_threads - job2->min_threads;
+	diff = job1->threads_per_core - job2->threads_per_core;
 
 	if (reverse_order)
 		diff = -diff;
@@ -426,9 +452,9 @@ static int _sort_job_by_min_memory(void *void1, void *void2)
 	job_info_t *job1 = (job_info_t *) void1;
 	job_info_t *job2 = (job_info_t *) void2;
 
-	job1->job_min_memory &= (~MEM_PER_CPU);
-	job2->job_min_memory &= (~MEM_PER_CPU);
-	diff = job1->job_min_memory - job2->job_min_memory;
+	job1->pn_min_memory &= (~MEM_PER_CPU);
+	job2->pn_min_memory &= (~MEM_PER_CPU);
+	diff = job1->pn_min_memory - job2->pn_min_memory;
 
 	if (reverse_order)
 		diff = -diff;
@@ -441,7 +467,7 @@ static int _sort_job_by_min_tmp_disk(void *void1, void *void2)
 	job_info_t *job1 = (job_info_t *) void1;
 	job_info_t *job2 = (job_info_t *) void2;
 
-	diff = job1->job_min_tmp_disk - job2->job_min_tmp_disk;
+	diff = job1->pn_min_tmp_disk - job2->pn_min_tmp_disk;
 
 	if (reverse_order)
 		diff = -diff;
@@ -666,6 +692,24 @@ static int _sort_job_by_reservation(void *void1, void *void2)
 /*****************************************************************************
  * Local Step Sort Functions
  *****************************************************************************/
+static int _sort_step_by_gres(void *void1, void *void2)
+{
+	int diff;
+	job_step_info_t *step1 = (job_step_info_t *) void1;
+	job_step_info_t *step2 = (job_step_info_t *) void2;
+	char *val1 = "", *val2 = "";
+
+	if (step1->gres)
+		val1 = step1->gres;
+	if (step2->gres)
+		val2 = step2->gres;
+	diff = strcmp(val1, val2);
+
+	if (reverse_order)
+		diff = -diff;
+	return diff;
+}
+
 static int _sort_step_by_id(void *void1, void *void2)
 {
 	int diff;
