@@ -38,36 +38,9 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
+#include "src/common/uid.h"
 #include "job_reports.h"
 
-typedef struct {
-	List jobs; /* This should be a NULL destroy since we are just
-		    * putting a pointer to a jobacct_job_rect_t here
-		    * not allocating any new memory */
-	uint32_t min_size; /* smallest size of job in cpus here 0 if first */
-	uint32_t max_size; /* largest size of job in cpus here INFINITE if
-			    * last */
-	uint32_t count; /* count of jobs */
-	uint64_t cpu_secs; /* how many cpus secs taken up by this
-			    * grouping */
-} local_grouping_t;
-
-typedef struct {
-	char *acct; /*account name */
-	uint64_t cpu_secs; /* how many cpus secs taken up by this
-			    * acct */
-	List groups; /* Names of what are being grouped char *'s*/
-	uint32_t lft;
-	uint32_t rgt;
-} acct_grouping_t;
-
-typedef struct {
-	char *cluster; /*cluster name */
-	uint64_t cpu_secs; /* how many cpus secs taken up by this
-			    * cluster */
-	List acct_list; /* containing acct_grouping_t's */
-} cluster_grouping_t;
-
 enum {
 	PRINT_JOB_ACCOUNT,
 	PRINT_JOB_CLUSTER,
@@ -84,37 +57,7 @@ static List print_fields_list = NULL; /* types are of print_field_t */
 static List grouping_print_fields_list = NULL; /* types are of print_field_t */
 static int print_job_count = 0;
 static bool flat_view = false;
-
-static void _destroy_local_grouping(void *object)
-{
-	local_grouping_t *local_grouping = (local_grouping_t *)object;
-	if(local_grouping) {
-		list_destroy(local_grouping->jobs);
-		xfree(local_grouping);
-	}
-}
-
-static void _destroy_acct_grouping(void *object)
-{
-	acct_grouping_t *acct_grouping = (acct_grouping_t *)object;
-	if(acct_grouping) {
-		xfree(acct_grouping->acct);
-		if(acct_grouping->groups)
-			list_destroy(acct_grouping->groups);
-		xfree(acct_grouping);
-	}
-}
-
-static void _destroy_cluster_grouping(void *object)
-{
-	cluster_grouping_t *cluster_grouping = (cluster_grouping_t *)object;
-	if(cluster_grouping) {
-		xfree(cluster_grouping->cluster);
-		if(cluster_grouping->acct_list)
-			list_destroy(cluster_grouping->acct_list);
-		xfree(cluster_grouping);
-	}
-}
+static bool individual_grouping = 0;
 
 /*
  * Comparator used for sorting clusters alphabetically
@@ -124,8 +67,9 @@ static void _destroy_cluster_grouping(void *object)
  *           -1: cluster_a < cluster_b
  *
  */
-extern int _sort_cluster_grouping_dec(cluster_grouping_t *cluster_a,
-				      cluster_grouping_t *cluster_b)
+static int _sort_cluster_grouping_dec(
+	slurmdb_report_cluster_grouping_t *cluster_a,
+	slurmdb_report_cluster_grouping_t *cluster_b)
 {
 	int diff = 0;
 
@@ -150,15 +94,37 @@ extern int _sort_cluster_grouping_dec(cluster_grouping_t *cluster_a,
  *           -1: acct_a < acct_b
  *
  */
-extern int _sort_acct_grouping_dec(acct_grouping_t *acct_a,
-				   acct_grouping_t *acct_b)
+static int _sort_acct_grouping_dec(slurmdb_report_acct_grouping_t *acct_a,
+				   slurmdb_report_acct_grouping_t *acct_b)
 {
 	int diff = 0;
 
+	char tmp_acct_a[200];
+	char tmp_acct_b[200];
+	char *wckey_a = NULL, *wckey_b = NULL;
+
 	if(!acct_a->acct || !acct_b->acct)
 		return 0;
 
-	diff = strcmp(acct_a->acct, acct_b->acct);
+	snprintf(tmp_acct_a, sizeof(tmp_acct_a), "%s", acct_a->acct);
+	snprintf(tmp_acct_b, sizeof(tmp_acct_b), "%s", acct_b->acct);
+	if ((wckey_a = strstr(tmp_acct_a, ":")))
+		*wckey_a++ = 0;
+
+	if ((wckey_b = strstr(tmp_acct_b, ":")))
+		*wckey_b++ = 0;
+
+	diff = strcmp(tmp_acct_a, tmp_acct_b);
+
+	if (diff > 0)
+		return 1;
+	else if (diff < 0)
+		return -1;
+
+	if(!wckey_a || !wckey_b)
+		return 0;
+
+	diff = strcmp(wckey_a, wckey_b);
 
 	if (diff > 0)
 		return 1;
@@ -168,8 +134,21 @@ extern int _sort_acct_grouping_dec(acct_grouping_t *acct_a,
 	return 0;
 }
 
+
+static char *_string_to_uid( char *name )
+{
+	uid_t uid;
+	if ( uid_from_string( name, &uid ) != 0 ) {
+		fprintf(stderr, "Invalid user id: %s\n", name);
+		exit(1);
+	}
+	xfree(name);
+	return xstrdup_printf( "%d", (int) uid );
+}
+
+
 /* returns number of objects added to list */
-extern int _addto_uid_char_list(List char_list, char *names)
+static int _addto_uid_char_list(List char_list, char *names)
 {
 	int i=0, start=0;
 	char *name = NULL, *tmp_char = NULL;
@@ -202,19 +181,7 @@ extern int _addto_uid_char_list(List char_list, char *names)
 					name = xmalloc((i-start+1));
 					memcpy(name, names+start, (i-start));
 					//info("got %s %d", name, i-start);
-					if (!isdigit((int) *name)) {
-						struct passwd *pwd;
-						if (!(pwd=getpwnam(name))) {
-							fprintf(stderr,
-								"Invalid user "
-								"id: %s\n",
-								name);
-							exit(1);
-						}
-						xfree(name);
-						name = xstrdup_printf(
-							"%d", pwd->pw_uid);
-					}
+					name = _string_to_uid( name );
 
 					while((tmp_char = list_next(itr))) {
 						if(!strcasecmp(tmp_char, name))
@@ -242,18 +209,7 @@ extern int _addto_uid_char_list(List char_list, char *names)
 		if((i-start) > 0) {
 			name = xmalloc((i-start)+1);
 			memcpy(name, names+start, (i-start));
-
-			if (!isdigit((int) *name)) {
-				struct passwd *pwd;
-				if (!(pwd=getpwnam(name))) {
-					fprintf(stderr,
-						"Invalid user id: %s\n",
-						name);
-					exit(1);
-				}
-				xfree(name);
-				name = xstrdup_printf("%d", pwd->pw_uid);
-			}
+			name = _string_to_uid( name );
 
 			while((tmp_char = list_next(itr))) {
 				if(!strcasecmp(tmp_char, name))
@@ -272,7 +228,7 @@ extern int _addto_uid_char_list(List char_list, char *names)
 }
 
 static int _set_cond(int *start, int argc, char *argv[],
-		     acct_job_cond_t *job_cond,
+		     slurmdb_job_cond_t *job_cond,
 		     List format_list, List grouping_list)
 {
 	int i;
@@ -311,7 +267,9 @@ static int _set_cond(int *start, int argc, char *argv[],
 					      argv[i]+end);
 			set = 1;
 		} else if (!strncasecmp (argv[i], "Accounts",
-					 MAX(command_len, 2))) {
+					 MAX(command_len, 2))
+			   || !strncasecmp(argv[i], "Acct",
+					   MAX(command_len, 4))) {
 			if(!job_cond->acct_list)
 				job_cond->acct_list =
 					list_create(slurm_destroy_char);
@@ -342,13 +300,15 @@ static int _set_cond(int *start, int argc, char *argv[],
 			set = 1;
 		} else if (!strncasecmp (argv[i], "grouping",
 					 MAX(command_len, 2))) {
-			if(grouping_list)
+			if(!strncasecmp(argv[i]+end, "individual", 1)) {
+				individual_grouping = 1;
+			} else if(grouping_list)
 				slurm_addto_char_list(grouping_list,
 						      argv[i]+end);
 		} else if (!strncasecmp (argv[i], "Jobs",
 					 MAX(command_len, 1))) {
 			char *end_char = NULL, *start_char = argv[i]+end;
-			jobacct_selected_step_t *selected_step = NULL;
+			slurmdb_selected_step_t *selected_step = NULL;
 			char *dot = NULL;
 			if(!job_cond->step_list)
 				job_cond->step_list =
@@ -362,7 +322,7 @@ static int _set_cond(int *start, int argc, char *argv[],
 				if(!(int)*start_char)
 					continue;
 				selected_step = xmalloc(
-					sizeof(jobacct_selected_step_t));
+					sizeof(slurmdb_selected_step_t));
 				list_append(job_cond->step_list, selected_step);
 
 				dot = strstr(start_char, ".");
@@ -437,7 +397,7 @@ static int _set_cond(int *start, int argc, char *argv[],
 	*/
 	start_time = job_cond->usage_start;
 	end_time = job_cond->usage_end;
-	set_start_end_time(&start_time, &end_time);
+	slurmdb_report_set_start_end_time(&start_time, &end_time);
 	job_cond->usage_start = start_time;
 	job_cond->usage_end = end_time;
 
@@ -475,7 +435,8 @@ static int _setup_print_fields_list(List format_list)
 		command_len = strlen(object);
 
 		field = xmalloc(sizeof(print_field_t));
-		if(!strncasecmp("Account", object, MAX(command_len, 1))) {
+		if(!strncasecmp("Account", object, MAX(command_len, 1))
+		   || !strncasecmp("Acct", object, MAX(command_len, 4))) {
 			field->type = PRINT_JOB_ACCOUNT;
 			field->name = xstrdup("Account");
 			field->len = 9;
@@ -567,10 +528,14 @@ static int _setup_grouping_print_fields_list(List grouping_list)
 			field->type = PRINT_JOB_COUNT;
 		else
 			field->type = PRINT_JOB_SIZE;
-		field->name = xstrdup_printf("%u-%u cpus", last_size, size-1);
-		if(time_format == SREPORT_TIME_SECS_PER
-		   || time_format == SREPORT_TIME_MINS_PER
-		   || time_format == SREPORT_TIME_HOURS_PER)
+		if(individual_grouping)
+			field->name = xstrdup_printf("%u cpus", size);
+		else
+			field->name = xstrdup_printf("%u-%u cpus",
+						     last_size, size-1);
+		if(time_format == SLURMDB_REPORT_TIME_SECS_PER
+		   || time_format == SLURMDB_REPORT_TIME_MINS_PER
+		   || time_format == SLURMDB_REPORT_TIME_HOURS_PER)
 			field->len = 20;
 		else
 			field->len = 13;
@@ -578,7 +543,7 @@ static int _setup_grouping_print_fields_list(List grouping_list)
 		if(print_job_count)
 			field->print_routine = print_fields_uint;
 		else
-			field->print_routine = sreport_print_time;
+			field->print_routine = slurmdb_report_print_time;
 		last_size = size;
 		last_object = object;
 		if((tmp_char = strstr(object, "\%"))) {
@@ -590,23 +555,24 @@ static int _setup_grouping_print_fields_list(List grouping_list)
 	}
 	list_iterator_destroy(itr);
 
-	if(last_size) {
+	if(last_size && !individual_grouping) {
 		field = xmalloc(sizeof(print_field_t));
 		if(print_job_count)
 			field->type = PRINT_JOB_COUNT;
 		else
 			field->type = PRINT_JOB_SIZE;
+
 		field->name = xstrdup_printf(">= %u cpus", last_size);
-		if(time_format == SREPORT_TIME_SECS_PER
-		   || time_format == SREPORT_TIME_MINS_PER
-		   || time_format == SREPORT_TIME_HOURS_PER)
+		if(time_format == SLURMDB_REPORT_TIME_SECS_PER
+		   || time_format == SLURMDB_REPORT_TIME_MINS_PER
+		   || time_format == SLURMDB_REPORT_TIME_HOURS_PER)
 			field->len = 20;
 		else
 			field->len = 13;
 		if(print_job_count)
 			field->print_routine = print_fields_uint;
 		else
-			field->print_routine = sreport_print_time;
+			field->print_routine = slurmdb_report_print_time;
 		if((tmp_char = strstr(last_object, "\%"))) {
 			int newlen = atoi(tmp_char+1);
 			if(newlen)
@@ -621,41 +587,36 @@ static int _setup_grouping_print_fields_list(List grouping_list)
 extern int job_sizes_grouped_by_top_acct(int argc, char *argv[])
 {
 	int rc = SLURM_SUCCESS;
-	acct_job_cond_t *job_cond = xmalloc(sizeof(acct_job_cond_t));
-	acct_association_cond_t assoc_cond;
-	acct_association_rec_t *assoc = NULL;
+	slurmdb_job_cond_t *job_cond = xmalloc(sizeof(slurmdb_job_cond_t));
+
 	int i=0;
 
+	uint64_t count1, count2;
+
 	ListIterator itr = NULL;
 	ListIterator itr2 = NULL;
 	ListIterator cluster_itr = NULL;
 	ListIterator local_itr = NULL;
 	ListIterator acct_itr = NULL;
-	ListIterator group_itr = NULL;
 
-
-	jobacct_job_rec_t *job = NULL;
-	cluster_grouping_t *cluster_group = NULL;
-	acct_grouping_t *acct_group = NULL;
-	local_grouping_t *local_group = NULL;
+	slurmdb_report_cluster_grouping_t *cluster_group = NULL;
+	slurmdb_report_acct_grouping_t *acct_group = NULL;
+	slurmdb_report_job_grouping_t *job_group = NULL;
 
 	print_field_t *field = NULL;
 	print_field_t total_field;
 	uint32_t total_time = 0;
-	sreport_time_format_t temp_format;
+	slurmdb_report_time_format_t temp_format;
 
-	List job_list = NULL;
-	List cluster_list = NULL;
+	List slurmdb_report_cluster_grouping_list = NULL;
 	List assoc_list = NULL;
 
-	List tmp_acct_list = NULL;
-
 	List format_list = list_create(slurm_destroy_char);
 	List grouping_list = list_create(slurm_destroy_char);
 
-	List header_list = list_create(NULL);
+	List header_list = NULL;
 
-//	sreport_time_format_t temp_time_format = time_format;
+//	slurmdb_report_time_format_t temp_time_format = time_format;
 
 	print_fields_list = list_create(destroy_print_field);
 
@@ -664,30 +625,21 @@ extern int job_sizes_grouped_by_top_acct(int argc, char *argv[])
 	if(!list_count(format_list))
 		slurm_addto_char_list(format_list, "Cl,a");
 
-	if(!list_count(grouping_list))
+	if(!individual_grouping && !list_count(grouping_list))
 		slurm_addto_char_list(grouping_list, "50,250,500,1000");
 
 	_setup_print_fields_list(format_list);
 	list_destroy(format_list);
 
-	_setup_grouping_print_fields_list(grouping_list);
-
-	/* we don't want to actually query by accounts in the jobs
-	   here since we may be looking for sub accounts of a specific
-	   account.
-	*/
-	tmp_acct_list = job_cond->acct_list;
-	job_cond->acct_list = NULL;
-	job_list = jobacct_storage_g_get_jobs_cond(db_conn, my_uid, job_cond);
-	job_cond->acct_list = tmp_acct_list;
-	tmp_acct_list = NULL;
-
-	if(!job_list) {
-		exit_code=1;
-		fprintf(stderr, " Problem with job query.\n");
+	if(!(slurmdb_report_cluster_grouping_list =
+	     slurmdb_report_job_sizes_grouped_by_top_account(db_conn,
+		     job_cond, grouping_list, flat_view))) {
+		exit_code = 1;
 		goto end_it;
 	}
 
+	_setup_grouping_print_fields_list(grouping_list);
+
 	if(print_fields_have_header) {
 		char start_char[20];
 		char end_char[20];
@@ -710,89 +662,7 @@ extern int job_sizes_grouped_by_top_acct(int argc, char *argv[])
 	}
 	total_time = job_cond->usage_end - job_cond->usage_start;
 
-	cluster_list = list_create(_destroy_cluster_grouping);
-
-	cluster_itr = list_iterator_create(cluster_list);
-	group_itr = list_iterator_create(grouping_list);
-
-	if(flat_view)
-		goto no_assocs;
-
-	memset(&assoc_cond, 0, sizeof(acct_association_cond_t));
-	assoc_cond.id_list = job_cond->associd_list;
-	assoc_cond.cluster_list = job_cond->cluster_list;
-	/* don't limit associations to having the partition_list */
-	//assoc_cond.partition_list = job_cond->partition_list;
-	if(!job_cond->acct_list || !list_count(job_cond->acct_list)) {
-		if(job_cond->acct_list)
-			list_destroy(job_cond->acct_list);
-		job_cond->acct_list = list_create(NULL);
-		list_append(job_cond->acct_list, "root");
-	}
-	assoc_cond.parent_acct_list = job_cond->acct_list;
-	assoc_list = acct_storage_g_get_associations(db_conn, my_uid,
-						     &assoc_cond);
-
-	if(!assoc_list) {
-		debug2(" No assoc list given.\n");
-		goto no_assocs;
-	}
-
-	itr = list_iterator_create(assoc_list);
-	while((assoc = list_next(itr))) {
-		while((cluster_group = list_next(cluster_itr))) {
-			if(!strcmp(assoc->cluster, cluster_group->cluster))
-				break;
-		}
-		if(!cluster_group) {
-			cluster_group =
-				xmalloc(sizeof(cluster_grouping_t));
-			cluster_group->cluster = xstrdup(assoc->cluster);
-			cluster_group->acct_list =
-				list_create(_destroy_acct_grouping);
-			list_append(cluster_list, cluster_group);
-		}
-
-		acct_itr = list_iterator_create(cluster_group->acct_list);
-		while((acct_group = list_next(acct_itr))) {
-			if(!strcmp(assoc->acct, acct_group->acct))
-				break;
-		}
-		list_iterator_destroy(acct_itr);
-
-		if(!acct_group) {
-			uint32_t last_size = 0;
-			char *group = NULL;
-			acct_group = xmalloc(sizeof(acct_grouping_t));
-			acct_group->acct = xstrdup(assoc->acct);
-			acct_group->lft = assoc->lft;
-			acct_group->rgt = assoc->rgt;
-			acct_group->groups =
-				list_create(_destroy_local_grouping);
-			list_append(cluster_group->acct_list, acct_group);
-			while((group = list_next(group_itr))) {
-				local_group = xmalloc(sizeof(local_grouping_t));
-				local_group->jobs = list_create(NULL);
-				local_group->min_size = last_size;
-				last_size = atoi(group);
-				local_group->max_size = last_size-1;
-				list_append(acct_group->groups, local_group);
-			}
-			if(last_size) {
-				local_group = xmalloc(sizeof(local_grouping_t));
-				local_group->jobs = list_create(NULL);
-				local_group->min_size = last_size;
-				local_group->max_size = INFINITE;
-				list_append(acct_group->groups, local_group);
-			}
-			list_iterator_reset(group_itr);
-		}
-		list_iterator_reset(cluster_itr);
-	}
-	list_iterator_destroy(itr);
-no_assocs:
-	itr = list_iterator_create(job_list);
-
+	header_list = list_create(NULL);
 	list_append_list(header_list, print_fields_list);
 	list_append_list(header_list, grouping_print_fields_list);
 
@@ -800,122 +670,21 @@ no_assocs:
 	total_field.type = PRINT_JOB_SIZE;
 	total_field.name = xstrdup("% of cluster");
 	total_field.len = 12;
-	total_field.print_routine = sreport_print_time;
+	total_field.print_routine = slurmdb_report_print_time;
 	list_append(header_list, &total_field);
 
 	print_fields_header(header_list);
 	list_destroy(header_list);
 
-	while((job = list_next(itr))) {
-		char *local_cluster = "UNKNOWN";
-		char *local_account = "UNKNOWN";
-
-		if(!job->elapsed) {
-			/* here we don't care about jobs that didn't
-			 * really run here */
-			continue;
-		}
-		if(job->cluster)
-			local_cluster = job->cluster;
-		if(job->account)
-			local_account = job->account;
-
-		list_iterator_reset(cluster_itr);
-		while((cluster_group = list_next(cluster_itr))) {
-			if(!strcmp(local_cluster, cluster_group->cluster))
-				break;
-		}
-		if(!cluster_group) {
-			/* here we are only looking for groups that
-			 * were added with the associations above
-			 */
-			if(!flat_view)
-				continue;
-			cluster_group =
-				xmalloc(sizeof(cluster_grouping_t));
-			cluster_group->cluster = xstrdup(local_cluster);
-			cluster_group->acct_list =
-				list_create(_destroy_acct_grouping);
-			list_append(cluster_list, cluster_group);
-		}
-
-		acct_itr = list_iterator_create(cluster_group->acct_list);
-		while((acct_group = list_next(acct_itr))) {
-			if(!flat_view
-			   && (acct_group->lft != (uint32_t)NO_VAL)
-			   && (job->lft != (uint32_t)NO_VAL)) {
-				/* keep separate since we don't want
-				 * to so a strcmp if we don't have to
-				 */
-				if(job->lft > acct_group->lft
-				   && job->lft < acct_group->rgt)
-					break;
-			} else if(!strcmp(acct_group->acct, local_account))
-				break;
-		}
-		list_iterator_destroy(acct_itr);
-
-		if(!acct_group) {
-			char *group = NULL;
-			uint32_t last_size = 0;
-			/* here we are only looking for groups that
-			 * were added with the associations above
-			 */
-			if(!flat_view)
-				continue;
-
-			acct_group = xmalloc(sizeof(acct_grouping_t));
-			acct_group->acct = xstrdup(local_account);
-			acct_group->groups =
-				list_create(_destroy_local_grouping);
-			list_append(cluster_group->acct_list, acct_group);
-
-			while((group = list_next(group_itr))) {
-				local_group = xmalloc(sizeof(local_grouping_t));
-				local_group->jobs = list_create(NULL);
-				local_group->min_size = last_size;
-				last_size = atoi(group);
-				local_group->max_size = last_size-1;
-				list_append(acct_group->groups, local_group);
-			}
-			if(last_size) {
-				local_group = xmalloc(sizeof(local_grouping_t));
-				local_group->jobs = list_create(NULL);
-				local_group->min_size = last_size;
-				local_group->max_size = INFINITE;
-				list_append(acct_group->groups, local_group);
-			}
-			list_iterator_reset(group_itr);
-		}
-
-		local_itr = list_iterator_create(acct_group->groups);
-		while((local_group = list_next(local_itr))) {
-			uint64_t total_secs = 0;
-			if((job->alloc_cpus < local_group->min_size)
-			   || (job->alloc_cpus > local_group->max_size))
-				continue;
-			list_append(local_group->jobs, job);
-			local_group->count++;
-			total_secs = (uint64_t)job->elapsed
-				* (uint64_t)job->alloc_cpus;
-			local_group->cpu_secs += total_secs;
-			acct_group->cpu_secs += total_secs;
-			cluster_group->cpu_secs += total_secs;
-		}
-		list_iterator_destroy(local_itr);
-	}
-	list_iterator_destroy(group_itr);
-	list_destroy(grouping_list);
-	list_iterator_destroy(itr);
-
-//	time_format = SREPORT_TIME_PERCENT;
+//	time_format = SLURMDB_REPORT_TIME_PERCENT;
 
 	itr = list_iterator_create(print_fields_list);
 	itr2 = list_iterator_create(grouping_print_fields_list);
-	list_sort(cluster_list, (ListCmpF)_sort_cluster_grouping_dec);
-	list_iterator_reset(cluster_itr);
+	list_sort(slurmdb_report_cluster_grouping_list,
+		  (ListCmpF)_sort_cluster_grouping_dec);
+	cluster_itr =
+		list_iterator_create(slurmdb_report_cluster_grouping_list);
 	while((cluster_group = list_next(cluster_itr))) {
-
 		list_sort(cluster_group->acct_list,
 			  (ListCmpF)_sort_acct_grouping_dec);
 		acct_itr = list_iterator_create(cluster_group->acct_list);
@@ -942,20 +711,20 @@ no_assocs:
 			}
 			list_iterator_reset(itr);
 			local_itr = list_iterator_create(acct_group->groups);
-			while((local_group = list_next(local_itr))) {
+			while((job_group = list_next(local_itr))) {
 				field = list_next(itr2);
 				switch(field->type) {
 				case PRINT_JOB_SIZE:
 					field->print_routine(
 						field,
-						local_group->cpu_secs,
+						job_group->cpu_secs,
 						acct_group->cpu_secs,
 						0);
 					break;
 				case PRINT_JOB_COUNT:
 					field->print_routine(
 						field,
-						local_group->count,
+						job_group->count,
 						0);
 					break;
 				default:
@@ -969,10 +738,16 @@ no_assocs:
 			list_iterator_destroy(local_itr);
 
 			temp_format = time_format;
-			time_format = SREPORT_TIME_PERCENT;
+			time_format = SLURMDB_REPORT_TIME_PERCENT;
+			if (!print_job_count) {
+				count1 = acct_group->cpu_secs;
+				count2 = cluster_group->cpu_secs;
+			} else {
+				count1 = acct_group->count;
+				count2 = cluster_group->count;
+			}
 			total_field.print_routine(&total_field,
-						  acct_group->cpu_secs,
-						  cluster_group->cpu_secs, 1);
+						  count1, count2, 1);
 			time_format = temp_format;
 			printf("\n");
 		}
@@ -983,14 +758,18 @@ no_assocs:
 //	time_format = temp_time_format;
 
 end_it:
+	xfree(total_field.name);
 	if(print_job_count)
 		print_job_count = 0;
 
-	destroy_acct_job_cond(job_cond);
+	if(individual_grouping)
+		individual_grouping = 0;
 
-	if(job_list) {
-		list_destroy(job_list);
-		job_list = NULL;
+	slurmdb_destroy_job_cond(job_cond);
+
+	if(grouping_list) {
+		list_destroy(grouping_list);
+		grouping_list = NULL;
 	}
 
 	if(assoc_list) {
@@ -998,9 +777,9 @@ end_it:
 		assoc_list = NULL;
 	}
 
-	if(cluster_list) {
-		list_destroy(cluster_list);
-		cluster_list = NULL;
+	if(slurmdb_report_cluster_grouping_list) {
+		list_destroy(slurmdb_report_cluster_grouping_list);
+		slurmdb_report_cluster_grouping_list = NULL;
 	}
 
 	if(print_fields_list) {
@@ -1019,38 +798,35 @@ end_it:
 extern int job_sizes_grouped_by_wckey(int argc, char *argv[])
 {
 	int rc = SLURM_SUCCESS;
-	acct_job_cond_t *job_cond = xmalloc(sizeof(acct_job_cond_t));
-	acct_wckey_cond_t wckey_cond;
-	acct_wckey_rec_t *wckey = NULL;
+	slurmdb_job_cond_t *job_cond = xmalloc(sizeof(slurmdb_job_cond_t));
 	int i=0;
 
+	uint64_t count1, count2;
+
 	ListIterator itr = NULL;
 	ListIterator itr2 = NULL;
 	ListIterator cluster_itr = NULL;
 	ListIterator local_itr = NULL;
 	ListIterator acct_itr = NULL;
-	ListIterator group_itr = NULL;
 
-	jobacct_job_rec_t *job = NULL;
-	cluster_grouping_t *cluster_group = NULL;
-	acct_grouping_t *acct_group = NULL;
-	local_grouping_t *local_group = NULL;
+	slurmdb_report_cluster_grouping_t *cluster_group = NULL;
+	slurmdb_report_acct_grouping_t *acct_group = NULL;
+	slurmdb_report_job_grouping_t *job_group = NULL;
 
 	print_field_t *field = NULL;
 	print_field_t total_field;
 	uint32_t total_time = 0;
-	sreport_time_format_t temp_format;
+	slurmdb_report_time_format_t temp_format;
 
-	List job_list = NULL;
-	List cluster_list = NULL;
+	List slurmdb_report_cluster_grouping_list = NULL;
 	List wckey_list = NULL;
 
 	List format_list = list_create(slurm_destroy_char);
 	List grouping_list = list_create(slurm_destroy_char);
 
-	List header_list = list_create(NULL);
+	List header_list = NULL;
 
-//	sreport_time_format_t temp_time_format = time_format;
+//	slurmdb_report_time_format_t temp_time_format = time_format;
 
 	print_fields_list = list_create(destroy_print_field);
 
@@ -1059,31 +835,20 @@ extern int job_sizes_grouped_by_wckey(int argc, char *argv[])
 	if(!list_count(format_list))
 		slurm_addto_char_list(format_list, "Cl,wc");
 
-	if(!list_count(grouping_list))
+	if(!individual_grouping && !list_count(grouping_list))
 		slurm_addto_char_list(grouping_list, "50,250,500,1000");
 
 	_setup_print_fields_list(format_list);
 	list_destroy(format_list);
 
-	_setup_grouping_print_fields_list(grouping_list);
-
-	/* we don't want to actually query by wckeys in the jobs
-	   here since we may be looking for sub accounts of a specific
-	   account.
-	*/
-	job_list = jobacct_storage_g_get_jobs_cond(db_conn, my_uid, job_cond);
-
-	if(!job_list) {
-		exit_code=1;
-		fprintf(stderr, " Problem with job query.\n");
+	if(!(slurmdb_report_cluster_grouping_list =
+	     slurmdb_report_job_sizes_grouped_by_wckey(db_conn,
+		     job_cond, grouping_list))) {
+		exit_code = 1;
 		goto end_it;
 	}
 
-	memset(&wckey_cond, 0, sizeof(acct_wckey_cond_t));
-	wckey_cond.name_list = job_cond->wckey_list;
-	wckey_cond.cluster_list = job_cond->cluster_list;
-
-	wckey_list = acct_storage_g_get_wckeys(db_conn, my_uid, &wckey_cond);
+	_setup_grouping_print_fields_list(grouping_list);
 
 	if(print_fields_have_header) {
 		char start_char[20];
@@ -1107,70 +872,7 @@ extern int job_sizes_grouped_by_wckey(int argc, char *argv[])
 	}
 	total_time = job_cond->usage_end - job_cond->usage_start;
 
-	cluster_list = list_create(_destroy_cluster_grouping);
-
-	cluster_itr = list_iterator_create(cluster_list);
-	group_itr = list_iterator_create(grouping_list);
-
-	if(!wckey_list) {
-		debug2(" No wckey list given.\n");
-		goto no_assocs;
-	}
-
-	itr = list_iterator_create(wckey_list);
-	while((wckey = list_next(itr))) {
-		while((cluster_group = list_next(cluster_itr))) {
-			if(!strcmp(wckey->cluster, cluster_group->cluster))
-				break;
-		}
-		if(!cluster_group) {
-			cluster_group =
-				xmalloc(sizeof(cluster_grouping_t));
-			cluster_group->cluster = xstrdup(wckey->cluster);
-			cluster_group->acct_list =
-				list_create(_destroy_acct_grouping);
-			list_append(cluster_list, cluster_group);
-		}
-
-		acct_itr = list_iterator_create(cluster_group->acct_list);
-		while((acct_group = list_next(acct_itr))) {
-			if(!strcmp(wckey->name, acct_group->acct))
-				break;
-		}
-		list_iterator_destroy(acct_itr);
-
-		if(!acct_group) {
-			uint32_t last_size = 0;
-			char *group = NULL;
-			acct_group = xmalloc(sizeof(acct_grouping_t));
-			acct_group->acct = xstrdup(wckey->name);
-			acct_group->lft = wckey->id;
-			acct_group->groups =
-				list_create(_destroy_local_grouping);
-			list_append(cluster_group->acct_list, acct_group);
-			while((group = list_next(group_itr))) {
-				local_group = xmalloc(sizeof(local_grouping_t));
-				local_group->jobs = list_create(NULL);
-				local_group->min_size = last_size;
-				last_size = atoi(group);
-				local_group->max_size = last_size-1;
-				list_append(acct_group->groups, local_group);
-			}
-			if(last_size) {
-				local_group = xmalloc(sizeof(local_grouping_t));
-				local_group->jobs = list_create(NULL);
-				local_group->min_size = last_size;
-				local_group->max_size = INFINITE;
-				list_append(acct_group->groups, local_group);
-			}
-			list_iterator_reset(group_itr);
-		}
-		list_iterator_reset(cluster_itr);
-	}
-	list_iterator_destroy(itr);
-no_assocs:
-	itr = list_iterator_create(job_list);
-
+	header_list = list_create(NULL);
 	list_append_list(header_list, print_fields_list);
 	list_append_list(header_list, grouping_print_fields_list);
 
@@ -1178,78 +880,20 @@ no_assocs:
 	total_field.type = PRINT_JOB_SIZE;
 	total_field.name = xstrdup("% of cluster");
 	total_field.len = 12;
-	total_field.print_routine = sreport_print_time;
+	total_field.print_routine = slurmdb_report_print_time;
 	list_append(header_list, &total_field);
 
 	print_fields_header(header_list);
 	list_destroy(header_list);
 
-	while((job = list_next(itr))) {
-		char *local_cluster = "UNKNOWN";
-		char *local_account = "UNKNOWN";
-
-		if(!job->elapsed) {
-			/* here we don't care about jobs that didn't
-			 * really run here */
-			continue;
-		}
-		if(job->cluster)
-			local_cluster = job->cluster;
-		if(job->account)
-			local_account = job->account;
-
-		list_iterator_reset(cluster_itr);
-		while((cluster_group = list_next(cluster_itr))) {
-			if(!strcmp(local_cluster, cluster_group->cluster))
-				break;
-		}
-		if(!cluster_group) {
-			/* here we are only looking for groups that
-			 * were added with the associations above
-			 */
-			continue;
-		}
-
-		acct_itr = list_iterator_create(cluster_group->acct_list);
-		while((acct_group = list_next(acct_itr))) {
-			if(!strcmp(job->wckey, acct_group->acct))
-				break;
-		}
-		list_iterator_destroy(acct_itr);
-
-		if(!acct_group) {
-			/* here we are only looking for groups that
-			 * were added with the associations above
-			 */
-			continue;
-		}
-
-		local_itr = list_iterator_create(acct_group->groups);
-		while((local_group = list_next(local_itr))) {
-			uint64_t total_secs = 0;
-			if((job->alloc_cpus < local_group->min_size)
-			   || (job->alloc_cpus > local_group->max_size))
-				continue;
-			list_append(local_group->jobs, job);
-			local_group->count++;
-			total_secs = (uint64_t)job->elapsed
-				* (uint64_t)job->alloc_cpus;
-			local_group->cpu_secs += total_secs;
-			acct_group->cpu_secs += total_secs;
-			cluster_group->cpu_secs += total_secs;
-		}
-		list_iterator_destroy(local_itr);
-	}
-	list_iterator_destroy(group_itr);
-	list_destroy(grouping_list);
-	list_iterator_destroy(itr);
-
-//	time_format = SREPORT_TIME_PERCENT;
+//	time_format = SLURMDB_REPORT_TIME_PERCENT;
 
 	itr = list_iterator_create(print_fields_list);
 	itr2 = list_iterator_create(grouping_print_fields_list);
-	list_sort(cluster_list, (ListCmpF)_sort_cluster_grouping_dec);
-	list_iterator_reset(cluster_itr);
+	list_sort(slurmdb_report_cluster_grouping_list,
+		  (ListCmpF)_sort_cluster_grouping_dec);
+	cluster_itr = list_iterator_create(
+		slurmdb_report_cluster_grouping_list);
 	while((cluster_group = list_next(cluster_itr))) {
 		list_sort(cluster_group->acct_list,
 			  (ListCmpF)_sort_acct_grouping_dec);
@@ -1277,20 +921,20 @@ no_assocs:
 			}
 			list_iterator_reset(itr);
 			local_itr = list_iterator_create(acct_group->groups);
-			while((local_group = list_next(local_itr))) {
+			while((job_group = list_next(local_itr))) {
 				field = list_next(itr2);
 				switch(field->type) {
 				case PRINT_JOB_SIZE:
 					field->print_routine(
 						field,
-						local_group->cpu_secs,
+						job_group->cpu_secs,
 						acct_group->cpu_secs,
 						0);
 					break;
 				case PRINT_JOB_COUNT:
 					field->print_routine(
 						field,
-						local_group->count,
+						job_group->count,
 						0);
 					break;
 				default:
@@ -1304,10 +948,16 @@ no_assocs:
 			list_iterator_destroy(local_itr);
 
 			temp_format = time_format;
-			time_format = SREPORT_TIME_PERCENT;
+			time_format = SLURMDB_REPORT_TIME_PERCENT;
+			if (!print_job_count) {
+				count1 = acct_group->cpu_secs;
+				count2 = cluster_group->cpu_secs;
+			} else {
+				count1 = acct_group->count;
+				count2 = cluster_group->count;
+			}
 			total_field.print_routine(&total_field,
-						  acct_group->cpu_secs,
-						  cluster_group->cpu_secs, 1);
+						  count1, count2, 1);
 			time_format = temp_format;
 			printf("\n");
 		}
@@ -1318,14 +968,18 @@ no_assocs:
 //	time_format = temp_time_format;
 
 end_it:
+	xfree(total_field.name);
 	if(print_job_count)
 		print_job_count = 0;
 
-	destroy_acct_job_cond(job_cond);
+	if(individual_grouping)
+		individual_grouping = 0;
 
-	if(job_list) {
-		list_destroy(job_list);
-		job_list = NULL;
+	slurmdb_destroy_job_cond(job_cond);
+
+	if(grouping_list) {
+		list_destroy(grouping_list);
+		grouping_list = NULL;
 	}
 
 	if(wckey_list) {
@@ -1333,9 +987,220 @@ end_it:
 		wckey_list = NULL;
 	}
 
-	if(cluster_list) {
-		list_destroy(cluster_list);
-		cluster_list = NULL;
+	if(slurmdb_report_cluster_grouping_list) {
+		list_destroy(slurmdb_report_cluster_grouping_list);
+		slurmdb_report_cluster_grouping_list = NULL;
+	}
+
+	if(print_fields_list) {
+		list_destroy(print_fields_list);
+		print_fields_list = NULL;
+	}
+
+	if(grouping_print_fields_list) {
+		list_destroy(grouping_print_fields_list);
+		grouping_print_fields_list = NULL;
+	}
+
+	return rc;
+}
+
+extern int job_sizes_grouped_by_top_acct_and_wckey(int argc, char *argv[])
+{
+	int rc = SLURM_SUCCESS;
+	slurmdb_job_cond_t *job_cond = xmalloc(sizeof(slurmdb_job_cond_t));
+
+	int i=0;
+
+	uint64_t count1, count2;
+
+	ListIterator itr = NULL;
+	ListIterator itr2 = NULL;
+	ListIterator cluster_itr = NULL;
+	ListIterator local_itr = NULL;
+	ListIterator acct_itr = NULL;
+
+	slurmdb_report_cluster_grouping_t *cluster_group = NULL;
+	slurmdb_report_acct_grouping_t *acct_group = NULL;
+	slurmdb_report_job_grouping_t *job_group = NULL;
+
+	print_field_t *field = NULL;
+	print_field_t total_field;
+	uint32_t total_time = 0;
+	slurmdb_report_time_format_t temp_format;
+
+	List slurmdb_report_cluster_grouping_list = NULL;
+	List assoc_list = NULL;
+
+	List format_list = list_create(slurm_destroy_char);
+	List grouping_list = list_create(slurm_destroy_char);
+
+	List header_list = NULL;
+
+//	slurmdb_report_time_format_t temp_time_format = time_format;
+
+	print_fields_list = list_create(destroy_print_field);
+
+	_set_cond(&i, argc, argv, job_cond, format_list, grouping_list);
+
+	if(!list_count(format_list))
+		slurm_addto_char_list(format_list, "Cl,a%-20");
+
+	if(!individual_grouping && !list_count(grouping_list))
+		slurm_addto_char_list(grouping_list, "50,250,500,1000");
+
+	_setup_print_fields_list(format_list);
+	list_destroy(format_list);
+
+	if(!(slurmdb_report_cluster_grouping_list =
+	     slurmdb_report_job_sizes_grouped_by_top_account_then_wckey(
+		     db_conn, job_cond, grouping_list, flat_view))) {
+		exit_code = 1;
+		goto end_it;
+	}
+
+	_setup_grouping_print_fields_list(grouping_list);
+
+	if(print_fields_have_header) {
+		char start_char[20];
+		char end_char[20];
+		time_t my_start = job_cond->usage_start;
+		time_t my_end = job_cond->usage_end-1;
+
+		slurm_make_time_str(&my_start, start_char, sizeof(start_char));
+		slurm_make_time_str(&my_end, end_char, sizeof(end_char));
+		printf("----------------------------------------"
+		       "----------------------------------------\n");
+		printf("Job Sizes %s - %s (%d secs)\n",
+		       start_char, end_char,
+		       (int)(job_cond->usage_end - job_cond->usage_start));
+		if(print_job_count)
+			printf("Units are in number of jobs ran\n");
+		else
+			printf("Time reported in %s\n", time_format_string);
+		printf("----------------------------------------"
+		       "----------------------------------------\n");
+	}
+	total_time = job_cond->usage_end - job_cond->usage_start;
+
+	header_list = list_create(NULL);
+	list_append_list(header_list, print_fields_list);
+	list_append_list(header_list, grouping_print_fields_list);
+
+	memset(&total_field, 0, sizeof(print_field_t));
+	total_field.type = PRINT_JOB_SIZE;
+	total_field.name = xstrdup("% of cluster");
+	total_field.len = 12;
+	total_field.print_routine = slurmdb_report_print_time;
+	list_append(header_list, &total_field);
+
+	print_fields_header(header_list);
+	list_destroy(header_list);
+
+//	time_format = SLURMDB_REPORT_TIME_PERCENT;
+
+	itr = list_iterator_create(print_fields_list);
+	itr2 = list_iterator_create(grouping_print_fields_list);
+	list_sort(slurmdb_report_cluster_grouping_list,
+		  (ListCmpF)_sort_cluster_grouping_dec);
+	cluster_itr =
+		list_iterator_create(slurmdb_report_cluster_grouping_list);
+	while((cluster_group = list_next(cluster_itr))) {
+		list_sort(cluster_group->acct_list,
+			  (ListCmpF)_sort_acct_grouping_dec);
+		acct_itr = list_iterator_create(cluster_group->acct_list);
+		while((acct_group = list_next(acct_itr))) {
+
+			while((field = list_next(itr))) {
+				switch(field->type) {
+				case PRINT_JOB_CLUSTER:
+					field->print_routine(
+						field,
+						cluster_group->cluster, 0);
+					break;
+				case PRINT_JOB_ACCOUNT:
+					field->print_routine(field,
+							     acct_group->acct,
+							     0);
+					break;
+				default:
+					field->print_routine(field,
+							     NULL,
+							     0);
+					break;
+				}
+			}
+			list_iterator_reset(itr);
+			local_itr = list_iterator_create(acct_group->groups);
+			while((job_group = list_next(local_itr))) {
+				field = list_next(itr2);
+				switch(field->type) {
+				case PRINT_JOB_SIZE:
+					field->print_routine(
+						field,
+						job_group->cpu_secs,
+						acct_group->cpu_secs,
+						0);
+					break;
+				case PRINT_JOB_COUNT:
+					field->print_routine(
+						field,
+						job_group->count,
+						0);
+					break;
+				default:
+					field->print_routine(field,
+							     NULL,
+							     0);
+					break;
+				}
+			}
+			list_iterator_reset(itr2);
+			list_iterator_destroy(local_itr);
+
+			temp_format = time_format;
+			time_format = SLURMDB_REPORT_TIME_PERCENT;
+			if (!print_job_count) {
+				count1 = acct_group->cpu_secs;
+				count2 = cluster_group->cpu_secs;
+			} else {
+				count1 = acct_group->count;
+				count2 = cluster_group->count;
+			}
+			total_field.print_routine(&total_field,
+						  count1, count2, 1);
+			time_format = temp_format;
+			printf("\n");
+		}
+		list_iterator_destroy(acct_itr);
+	}
+	list_iterator_destroy(itr);
+
+//	time_format = temp_time_format;
+
+end_it:
+	xfree(total_field.name);
+	if(print_job_count)
+		print_job_count = 0;
+
+	if(individual_grouping)
+		individual_grouping = 0;
+
+	slurmdb_destroy_job_cond(job_cond);
+
+	if(grouping_list) {
+		list_destroy(grouping_list);
+		grouping_list = NULL;
+	}
+
+	if(assoc_list) {
+		list_destroy(assoc_list);
+		assoc_list = NULL;
+	}
+
+	if(slurmdb_report_cluster_grouping_list) {
+		list_destroy(slurmdb_report_cluster_grouping_list);
+		slurmdb_report_cluster_grouping_list = NULL;
 	}
 
 	if(print_fields_list) {
