@@ -1,9 +1,9 @@
 /*****************************************************************************\
- *  test1.90.prog.c - Simple test program for SLURM regression test1.90.
- *  Reports SLURM task ID, the CPU mask, and memory mask,
+ *  test1.89.prog.c - Simple test program for SLURM regression test1.89.
+ *  Reports SLURM task ID and the CPU mask,
  *  similar functionality to "taskset" command
  *****************************************************************************
- *  Copyright (C) 2006 The Regents of the University of California.
+ *  Copyright (C) 2005 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov>
  *  CODE-OCEC-09-009. All rights reserved.
@@ -27,67 +27,54 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 #define _GNU_SOURCE
-#include <numa.h>
+#define __USE_GNU
+#include <errno.h>
+#include <sched.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include "config.h"
 
-/* For RedHat systems, run with NEW_TOOLS=0
- * For current Ubuntu, run wtih NEW_TOOLS=1 */
-#define NEW_TOOLS 0
-
-#if NEW_TOOLS
-#define MY_MASK struct bitmask *
-#define MY_TEST numa_bitmask_isbitset
-#else
-#define MY_MASK nodemask_t
-#define MY_TEST nodemask_isset
-#endif
-
-static void _load_cpu_mask(MY_MASK *cpu_mask)
+static void _load_mask(cpu_set_t *mask)
 {
-	*cpu_mask = numa_get_run_node_mask();
+	int rc;
+
+#ifdef SCHED_GETAFFINITY_THREE_ARGS
+	rc = sched_getaffinity((pid_t) 0, (unsigned int) sizeof(cpu_set_t),
+		mask);
+#else
+	rc = sched_getaffinity((pid_t) 0, mask);
+#endif
+	if (rc != 0) {
+		fprintf(stderr, "ERROR: sched_getaffinity: %s\n",
+			strerror(errno));
+		exit(1);
+	}
 }
 
-static void _load_mem_mask(MY_MASK *mem_mask)
+static int _mask_to_int(cpu_set_t *mask)
 {
-	*mem_mask = numa_get_membind();
-}
-
-static unsigned long _mask_to_int(MY_MASK *mask)
-{
-	int i;
-	unsigned long rc = 0;
-	for (i=0; i<NUMA_NUM_NODES; i++) {
-#if NEW_TOOLS
-		if (MY_TEST(*mask, i))
-#else
-		if (MY_TEST(mask, i))
-#endif
+	int i, rc = 0;
+	for (i=0; i<CPU_SETSIZE; i++) {
+		if (CPU_ISSET(i, mask))
 			rc += (1 << i);
 	}
 	return rc;
 }
 
+
 main (int argc, char **argv)
 {
 	char *task_str;
-	MY_MASK cpu_mask;
-	MY_MASK mem_mask;
+	cpu_set_t mask;
 	int task_id;
 
-	if (numa_available() < 0) {
-		fprintf(stderr, "ERROR: numa support not available\n");
-		exit(0);
-	}
-
+	_load_mask(&mask);
 	if ((task_str = getenv("SLURM_PROCID")) == NULL) {
 		fprintf(stderr, "ERROR: getenv(SLURM_PROCID) failed\n");
 		exit(1);
 	}
 	task_id = atoi(task_str);
-	_load_cpu_mask(&cpu_mask);
-	_load_mem_mask(&mem_mask);
-	printf("TASK_ID:%d,CPU_MASK:%lu,MEM_MASK:%lu\n",
-		task_id, _mask_to_int(&cpu_mask), _mask_to_int(&mem_mask));
+	printf("TASK_ID:%d,MASK:%u\n", task_id, _mask_to_int(&mask));
 	exit(0);
 }
